@@ -7,12 +7,12 @@
 
 #include <memory>
 
-#include "ash/ash_constants.h"
 #include "ash/ash_export.h"
-#include "ash/public/cpp/accessibility_types.h"
+#include "ash/public/cpp/ash_constants.h"
 #include "ash/public/interfaces/accessibility_controller.mojom.h"
 #include "ash/session/session_observer.h"
 #include "base/macros.h"
+#include "base/observer_list.h"
 #include "base/time/time.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
 #include "ui/accessibility/ax_enums.mojom.h"
@@ -28,8 +28,14 @@ class Connector;
 namespace ash {
 
 class AccessibilityHighlightController;
+class AccessibilityObserver;
 class ScopedBacklightsForcedOff;
 
+enum AccessibilityNotificationVisibility {
+  A11Y_NOTIFICATION_NONE,
+  A11Y_NOTIFICATION_SHOW,
+};
+
 // The controller for accessibility features in ash. Features can be enabled
 // in chrome's webui settings or the system tray menu (see TrayAccessibility).
 // Uses preferences to communicate with chrome to support mash.
@@ -43,9 +49,22 @@ class ASH_EXPORT AccessibilityController
   // See Shell::RegisterProfilePrefs().
   static void RegisterProfilePrefs(PrefRegistrySimple* registry, bool for_test);
 
+  void AddObserver(AccessibilityObserver* observer);
+  void RemoveObserver(AccessibilityObserver* observer);
+
   // Binds the mojom::AccessibilityController interface to this object.
   void BindRequest(mojom::AccessibilityControllerRequest request);
 
+  // The following functions read and write to their associated preference.
+  // These values are then used to determine whether the accelerator
+  // confirmation dialog for the respective preference has been accepted before.
+  void SetHighContrastAcceleratorDialogAccepted();
+  bool HasHighContrastAcceleratorDialogBeenAccepted() const;
+  void SetScreenMagnifierAcceleratorDialogAccepted();
+  bool HasScreenMagnifierAcceleratorDialogBeenAccepted() const;
+  void SetDockedMagnifierAcceleratorDialogAccepted();
+  bool HasDockedMagnifierAcceleratorDialogBeenAccepted() const;
+
   void SetAutoclickEnabled(bool enabled);
   bool IsAutoclickEnabled() const;
 
@@ -55,6 +74,9 @@ class ASH_EXPORT AccessibilityController
   void SetCursorHighlightEnabled(bool enabled);
   bool IsCursorHighlightEnabled() const;
 
+  void SetDictationEnabled(bool enabled);
+  bool IsDictationEnabled() const;
+
   void SetFocusHighlightEnabled(bool enabled);
   bool IsFocusHighlightEnabled() const;
 
@@ -74,13 +96,17 @@ class ASH_EXPORT AccessibilityController
   void SetSelectToSpeakEnabled(bool enabled);
   bool IsSelectToSpeakEnabled() const;
 
+  void RequestSelectToSpeakStateChange();
+  mojom::SelectToSpeakState GetSelectToSpeakState() const;
+
   void SetStickyKeysEnabled(bool enabled);
   bool IsStickyKeysEnabled() const;
 
   void SetVirtualKeyboardEnabled(bool enabled);
   bool IsVirtualKeyboardEnabled() const;
 
-  bool braille_display_connected() const { return braille_display_connected_; }
+  bool IsDictationActive() const;
+  void SetDictationActive(bool is_active);
 
   // Triggers an accessibility alert to give the user feedback.
   void TriggerAccessibilityAlert(mojom::AccessibilityAlert alert);
@@ -121,11 +147,17 @@ class ASH_EXPORT AccessibilityController
   // countdown.
   void PlaySpokenFeedbackToggleCountdown(int tick_count);
 
+  // Public because a11y features like screen magnifier are managed outside of
+  // this controller.
+  void NotifyAccessibilityStatusChanged();
+
   // mojom::AccessibilityController:
   void SetClient(mojom::AccessibilityControllerClientPtr client) override;
   void SetDarkenScreen(bool darken) override;
   void BrailleDisplayStateChanged(bool connected) override;
   void SetFocusHighlightRect(const gfx::Rect& bounds_in_screen) override;
+  void SetAccessibilityPanelFullscreen(bool fullscreen) override;
+  void SetSelectToSpeakState(mojom::SelectToSpeakState state) override;
 
   // SessionObserver:
   void OnSigninScreenPrefServiceInitialized(PrefService* prefs) override;
@@ -143,6 +175,7 @@ class ASH_EXPORT AccessibilityController
   void UpdateAutoclickDelayFromPref();
   void UpdateCaretHighlightFromPref();
   void UpdateCursorHighlightFromPref();
+  void UpdateDictationFromPref();
   void UpdateFocusHighlightFromPref();
   void UpdateHighContrastFromPref();
   void UpdateLargeCursorFromPref();
@@ -154,6 +187,11 @@ class ASH_EXPORT AccessibilityController
   void UpdateAccessibilityHighlightingFromPrefs();
 
   service_manager::Connector* connector_ = nullptr;
+
+  // The pref service of the currently active user or the signin profile before
+  // user logs in. Can be null in ash_unittests.
+  PrefService* active_user_prefs_ = nullptr;
+
   std::unique_ptr<PrefChangeRegistrar> pref_change_registrar_;
 
   // Binding for mojom::AccessibilityController interface.
@@ -166,6 +204,7 @@ class ASH_EXPORT AccessibilityController
   base::TimeDelta autoclick_delay_;
   bool caret_highlight_enabled_ = false;
   bool cursor_highlight_enabled_ = false;
+  bool dictation_enabled_ = false;
   bool focus_highlight_enabled_ = false;
   bool high_contrast_enabled_ = false;
   bool large_cursor_enabled_ = false;
@@ -175,12 +214,10 @@ class ASH_EXPORT AccessibilityController
   bool select_to_speak_enabled_ = false;
   bool sticky_keys_enabled_ = false;
   bool virtual_keyboard_enabled_ = false;
-  bool braille_display_connected_ = false;
+  bool dictation_active_ = false;
 
-  // TODO(warx): consider removing this and replacing it with a more reliable
-  // way (https://crbug.com/800270).
-  AccessibilityNotificationVisibility spoken_feedback_notification_ =
-      A11Y_NOTIFICATION_NONE;
+  mojom::SelectToSpeakState select_to_speak_state_ =
+      mojom::SelectToSpeakState::kSelectToSpeakStateInactive;
 
   // Used to control the highlights of caret, cursor and focus.
   std::unique_ptr<AccessibilityHighlightController>
@@ -189,6 +226,8 @@ class ASH_EXPORT AccessibilityController
   // Used to force the backlights off to darken the screen.
   std::unique_ptr<ScopedBacklightsForcedOff> scoped_backlights_forced_off_;
 
+  base::ObserverList<AccessibilityObserver> observers_;
+
   DISALLOW_COPY_AND_ASSIGN(AccessibilityController);
 };
 
