@@ -14,9 +14,9 @@
 
 namespace ash {
 
-void RunCallback(uint64_t* window_count, const base::Closure& callback, uint64_t result) {
-  *window_count = result;
-  callback.Run();
+void RunCallback(bool* success, base::RepeatingClosure callback, bool result) {
+  *success = result;
+  std::move(callback).Run();
 }
 
 class AppLaunchTest : public service_manager::test::ServiceTest {
@@ -43,12 +43,12 @@ TEST_F(AppLaunchTest, TestQuickLaunch) {
   connector()->BindInterface(ui::mojom::kServiceName, &test_interface);
 
   base::RunLoop run_loop;
-  uint64_t window_count = 0;
-  test_interface->EnsureClientHasDrawnRootWindows(
+  bool success = false;
+  test_interface->EnsureClientHasDrawnWindow(
       quick_launch::mojom::kServiceName,
-      base::Bind(&RunCallback, &window_count, run_loop.QuitClosure()));
+      base::BindOnce(&RunCallback, &success, run_loop.QuitClosure()));
   run_loop.Run();
-  EXPECT_EQ(1u, window_count);
+  EXPECT_TRUE(success);
 }
 
 }  // namespace ash
