@@ -20,6 +20,9 @@ class APP_LIST_MODEL_EXPORT SearchBoxModelObserver {
   // Invoked when text or voice search flag is changed.
   virtual void Update() = 0;
 
+  // Invoked when the search engine is changed.
+  virtual void SearchEngineChanged() = 0;
+
  protected:
   virtual ~SearchBoxModelObserver() {}
 };
