@@ -13,13 +13,16 @@
 #include "base/strings/utf_string_conversions.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/chromeos/events/keyboard_layout_util.h"
+#include "ui/events/devices/input_device_manager.h"
 #include "ui/events/event_constants.h"
 #include "ui/events/keycodes/dom/dom_code.h"
+#include "ui/events/keycodes/dom/dom_codes.h"
 #include "ui/events/keycodes/dom/dom_key.h"
 #include "ui/events/keycodes/dom/keycode_converter.h"
 #include "ui/events/keycodes/keyboard_code_conversion.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine_manager.h"
+#include "ui/gfx/vector_icon_types.h"
 
 namespace keyboard_shortcut_viewer {
 
@@ -60,9 +63,15 @@ base::Optional<base::string16> GetSpecialStringForKeyboardCode(
       msg_id = IDS_KSV_MODIFIER_SHIFT;
       break;
     case ui::VKEY_COMMAND:
+      // DeviceUsesKeyboardLayout2() relies on InputDeviceManager.
+      DCHECK(ui::InputDeviceManager::HasInstance());
+      DCHECK(ui::InputDeviceManager::GetInstance()->AreDeviceListsComplete());
       msg_id = ui::DeviceUsesKeyboardLayout2() ? IDS_KSV_MODIFIER_LAUNCHER
                                                : IDS_KSV_MODIFIER_SEARCH;
       break;
+    case ui::VKEY_ESCAPE:
+      msg_id = IDS_KSV_KEY_ESCAPE;
+      break;
     case ui::VKEY_SPACE:
       msg_id = IDS_KSV_KEY_SPACE;
       break;
@@ -121,19 +130,20 @@ base::string16 GetStringForKeyboardCode(ui::KeyboardCode key_code) {
   if (key_label)
     return key_label.value();
 
-  ui::DomCode dom_code = ui::UsLayoutKeyboardCodeToDomCode(key_code);
   ui::DomKey dom_key;
-  ui::KeyboardCode keycode_ignored;
-  if (ui::KeyboardLayoutEngineManager::GetKeyboardLayoutEngine()->Lookup(
-          dom_code, 0 /* flags */, &dom_key, &keycode_ignored)) {
+  ui::KeyboardCode key_code_to_compare = ui::VKEY_UNKNOWN;
+  for (const auto& dom_code : ui::dom_codes) {
+    if (!ui::KeyboardLayoutEngineManager::GetKeyboardLayoutEngine()->Lookup(
+            dom_code, /*flags=*/ui::EF_NONE, &dom_key, &key_code_to_compare)) {
+      continue;
+    }
+    if (key_code_to_compare != key_code || !dom_key.IsValid() ||
+        dom_key.IsDeadKey()) {
+      continue;
+    }
     return base::UTF8ToUTF16(ui::KeycodeConverter::DomKeyToKeyString(dom_key));
   }
-
-  // Fall back to US keyboard layout.
-  const bool has_mapping = ui::DomCodeToUsLayoutDomKey(
-      dom_code, 0 /* flags */, &dom_key, &keycode_ignored);
-  DCHECK(has_mapping);
-  return base::UTF8ToUTF16(ui::KeycodeConverter::DomKeyToKeyString(dom_key));
+  return base::string16();
 }
 
 const gfx::VectorIcon* GetVectorIconForKeyboardCode(ui::KeyboardCode key_code) {
@@ -173,6 +183,20 @@ const gfx::VectorIcon* GetVectorIconForKeyboardCode(ui::KeyboardCode key_code) {
 
 const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
   static base::NoDestructor<std::vector<KeyboardShortcutItem>> item_list({
+      {// |categories|
+       {ShortcutCategory::kAccessibility},
+       IDS_KSV_DESCRIPTION_TOGGLE_DOCKED_MAGNIFIER,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_D, ui::EF_COMMAND_DOWN | ui::EF_CONTROL_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kAccessibility},
+       IDS_KSV_DESCRIPTION_TOGGLE_FULLSCREEN_MAGNIFIER,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_M, ui::EF_COMMAND_DOWN | ui::EF_CONTROL_DOWN}}},
+
       {// |categories|
        {ShortcutCategory::kPopular},
        IDS_KSV_DESCRIPTION_LOCK_SCREEN,
@@ -313,7 +337,7 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
         ui::VKEY_G, ui::VKEY_SHIFT, ui::VKEY_UNKNOWN, ui::VKEY_RETURN}},
 
       {// |categories|
-       {ShortcutCategory::kAccessibility},
+       {ShortcutCategory::kPageAndBrowser, ShortcutCategory::kAccessibility},
        IDS_KSV_DESCRIPTION_IDC_FOCUS_BOOKMARKS,
        IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
        // |accelerator_ids|
@@ -402,11 +426,14 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        {{ui::VKEY_P, ui::EF_CONTROL_DOWN}}},
 
       {// |categories|
-       {ShortcutCategory::kPageAndBrowser},
+       {ShortcutCategory::kPopular, ShortcutCategory::kPageAndBrowser},
        IDS_KSV_DESCRIPTION_IDC_RELOAD,
-       IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
+       IDS_KSV_SHORTCUT_IDC_RELOAD,
        // |accelerator_ids|
-       {{ui::VKEY_R, ui::EF_CONTROL_DOWN}}},
+       {},
+       // |shortcut_key_codes|
+       {ui::VKEY_BROWSER_REFRESH, ui::VKEY_CONTROL, ui::VKEY_UNKNOWN,
+        ui::VKEY_R}},
 
       {// |categories|
        {ShortcutCategory::kPageAndBrowser},
@@ -502,7 +529,7 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        {{ui::VKEY_OEM_PLUS, ui::EF_CONTROL_DOWN}}},
 
       {// |categories|
-       {ShortcutCategory::kTextEditing},
+       {ShortcutCategory::kSystemAndDisplay, ShortcutCategory::kTextEditing},
        IDS_KSV_DESCRIPTION_NEXT_IME,
        IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
        // |accelerator_ids|
@@ -516,7 +543,7 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        {{ui::VKEY_M, ui::EF_SHIFT_DOWN | ui::EF_ALT_DOWN}}},
 
       {// |categories|
-       {ShortcutCategory::kTextEditing},
+       {ShortcutCategory::kSystemAndDisplay, ShortcutCategory::kTextEditing},
        IDS_KSV_DESCRIPTION_PREVIOUS_IME,
        IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
        // |accelerator_ids|
@@ -647,15 +674,6 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        // |shortcut_key_codes|
        {ui::VKEY_CONTROL, ui::VKEY_UNKNOWN}},
 
-      {// |categories|
-       {ShortcutCategory::kPopular, ShortcutCategory::kPageAndBrowser},
-       IDS_KSV_DESCRIPTION_IDC_RELOAD,
-       IDS_KSV_SHORTCUT_ONE_KEY,
-       // |accelerator_ids|
-       {},
-       // |shortcut_key_codes|
-       {ui::VKEY_BROWSER_REFRESH}},
-
       {// |categories|
        {ShortcutCategory::kPopular},
        IDS_KSV_DESCRIPTION_KEYBOARD_SHORTCUT_HELPER,
@@ -722,7 +740,7 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        // |accelerator_ids|
        {},
        // |shortcut_key_codes|
-       {ui::VKEY_LWIN, ui::VKEY_COMMAND, ui::VKEY_UNKNOWN, ui::VKEY_DOWN}},
+       {ui::VKEY_LMENU, ui::VKEY_COMMAND, ui::VKEY_UNKNOWN, ui::VKEY_DOWN}},
 
       {// |categories|
        {ShortcutCategory::kPageAndBrowser},
@@ -1120,6 +1138,156 @@ const std::vector<KeyboardShortcutItem>& GetKeyboardShortcutItemList() {
        IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
        // |accelerator_ids|
        {{ui::VKEY_M, ui::EF_COMMAND_DOWN | ui::EF_ALT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_TOGGLE_APP_LIST,
+       IDS_KSV_SHORTCUT_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_LWIN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_TAKE_WINDOW_SCREENSHOT,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_MEDIA_LAUNCH_APP1, ui::EF_CONTROL_DOWN | ui::EF_ALT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_SUSPEND,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_L, ui::EF_SHIFT_DOWN | ui::EF_COMMAND_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_OPEN_GET_HELP,
+       IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_OEM_2, ui::EF_CONTROL_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_OPEN_FEEDBACK_PAGE,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_I, ui::EF_ALT_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_ROTATE_WINDOW,
+       IDS_KSV_SHORTCUT_THREE_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_BROWSER_REFRESH,
+         ui::EF_CONTROL_DOWN | ui::EF_ALT_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_SHOW_STYLUS_TOOLS,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_P, ui::EF_ALT_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_TOGGLE_MAXIMIZED,
+       IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_OEM_PLUS, ui::EF_ALT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_WINDOW_POSITION_CENTER,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_OEM_PLUS, ui::EF_ALT_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_OPEN_CROSH,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_T, ui::EF_CONTROL_DOWN | ui::EF_ALT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kAccessibility},
+       IDS_KSV_DESCRIPTION_TOGGLE_DICTATION,
+       IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_D, ui::EF_COMMAND_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_EXIT,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_Q, ui::EF_CONTROL_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kTabAndWindow},
+       IDS_KSV_DESCRIPTION_UNPIN,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_ESCAPE, ui::EF_SHIFT_DOWN | ui::EF_COMMAND_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kSystemAndDisplay},
+       IDS_KSV_DESCRIPTION_SHOW_IME_MENU_BUBBLE,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_K, ui::EF_SHIFT_DOWN | ui::EF_COMMAND_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_SHOW_IDC_FOCUS_MENU_BAR,
+       IDS_KSV_SHORTCUT_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_F10}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_SHOW_IDC_HOME,
+       IDS_KSV_SHORTCUT_ONE_MODIFIER_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_HOME, ui::EF_ALT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_SHOW_IDC_CLEAR_BROWSING_DATA,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_BACK, ui::EF_CONTROL_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_SHOW_IDC_SHOW_BOOKMARK_MANAGER,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_O, ui::EF_CONTROL_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_IDC_DEV_TOOLS_INSPECT,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_C, ui::EF_CONTROL_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kAccessibility},
+       IDS_KSV_DESCRIPTION_IDC_FOCUS_INACTIVE_POPUP_FOR_ACCESSIBILITY,
+       IDS_KSV_SHORTCUT_TWO_MODIFIERS_ONE_KEY,
+       // |accelerator_ids|
+       {{ui::VKEY_A, ui::EF_ALT_DOWN | ui::EF_SHIFT_DOWN}}},
+
+      {// |categories|
+       {ShortcutCategory::kPageAndBrowser},
+       IDS_KSV_DESCRIPTION_IDC_SHOW_APP_MENU,
+       IDS_KSV_SHORTCUT_IDC_SHOW_APP_MENU,
+       // |accelerator_ids|
+       {},
+       // |shortcut_key_codes|
+       {ui::VKEY_LMENU, ui::VKEY_UNKNOWN, ui::VKEY_E, ui::VKEY_F}},
   });
 
   static bool is_initialized = false;
