@@ -6,20 +6,27 @@
 
 #include <algorithm>
 
-#include "ash/components/resources/grit/ash_components_resources.h"
 #include "ash/components/shortcut_viewer/keyboard_shortcut_viewer_metadata.h"
 #include "ash/components/shortcut_viewer/vector_icons/vector_icons.h"
 #include "ash/components/shortcut_viewer/views/keyboard_shortcut_item_list_view.h"
 #include "ash/components/shortcut_viewer/views/keyboard_shortcut_item_view.h"
 #include "ash/components/shortcut_viewer/views/ksv_search_box_view.h"
 #include "ash/components/strings/grit/ash_components_strings.h"
+#include "ash/public/cpp/app_list/internal_app_id_constants.h"
+#include "ash/public/cpp/resources/grit/ash_public_unscaled_resources.h"
 #include "ash/public/cpp/shelf_item.h"
 #include "ash/public/cpp/window_properties.h"
+#include "base/bind.h"
 #include "base/i18n/string_search.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/metrics/user_metrics.h"
 #include "base/metrics/user_metrics_action.h"
+#include "base/strings/string_number_conversions.h"
+#include "base/time/time.h"
+#include "base/trace_event/trace_event.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/window.h"
+#include "ui/base/accelerators/accelerator.h"
 #include "ui/base/default_style.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
@@ -50,7 +57,7 @@ void SetupSearchIllustrationView(views::View* illustration_view,
                                  int message_id) {
   constexpr int kSearchIllustrationIconSize = 150;
   constexpr SkColor kSearchIllustrationIconColor =
-      SkColorSetARGBMacro(0xFF, 0xDA, 0xDC, 0xE0);
+      SkColorSetARGB(0xFF, 0xDA, 0xDC, 0xE0);
 
   illustration_view->set_owned_by_client();
   constexpr int kTopPadding = 98;
@@ -66,7 +73,7 @@ void SetupSearchIllustrationView(views::View* illustration_view,
   illustration_view->AddChildView(image_view);
 
   constexpr SkColor kSearchIllustrationTextColor =
-      SkColorSetARGBMacro(0xFF, 0x20, 0x21, 0x24);
+      SkColorSetARGB(0xFF, 0x20, 0x21, 0x24);
   views::Label* text = new views::Label(l10n_util::GetStringUTF16(message_id));
   text->SetEnabledColor(kSearchIllustrationTextColor);
   constexpr int kLabelFontSizeDelta = 1;
@@ -91,23 +98,23 @@ KeyboardShortcutView::~KeyboardShortcutView() {
 }
 
 // static
-views::Widget* KeyboardShortcutView::Show(gfx::NativeWindow context) {
+views::Widget* KeyboardShortcutView::Toggle(base::TimeTicks start_time) {
   if (g_ksv_view) {
-    // If there is a KeyboardShortcutView window open already, just activate
-    // it.
-    g_ksv_view->GetWidget()->Activate();
+    if (g_ksv_view->GetWidget()->IsActive())
+      g_ksv_view->GetWidget()->Close();
+    else
+      g_ksv_view->GetWidget()->Activate();
   } else {
+    TRACE_EVENT0("shortcut_viewer", "CreateWidget");
     base::RecordAction(
         base::UserMetricsAction("KeyboardShortcutViewer.CreateWindow"));
 
-    constexpr gfx::Size kKSVWindowSize(800, 512);
-    gfx::Rect window_bounds(kKSVWindowSize);
-    if (context) {
-      window_bounds = context->GetRootWindow()->bounds();
-      window_bounds.ClampToCenteredSize(kKSVWindowSize);
-    }
-    views::Widget::CreateWindowWithContextAndBounds(new KeyboardShortcutView(),
-                                                    context, window_bounds);
+    views::Widget::InitParams params;
+    params.delegate = new KeyboardShortcutView;
+    // Intentionally don't set bounds. The window will be sized and centered
+    // based on CalculatePreferredSize().
+    views::Widget* widget = new views::Widget;
+    widget->Init(params);
 
     // Set frame view Active and Inactive colors, both are SK_ColorWHITE.
     aura::Window* window = g_ksv_view->GetWidget()->GetNativeWindow();
@@ -115,124 +122,45 @@ views::Widget* KeyboardShortcutView::Show(gfx::NativeWindow context) {
     window->SetProperty(ash::kFrameInactiveColorKey, SK_ColorWHITE);
 
     // Set shelf icon.
-    // An app id for keyboard shortcut viewer window, also used to identify the
-    // shelf item. Generated as
-    // crx_file::id_util::GenerateId("org.chromium.keyboardshortcutviewer")
-    static constexpr char kKeyboardShortcutViewerId[] =
-        "dieikdblbimmfmfinbibdlalidbnbchd";
-    const ash::ShelfID shelf_id(kKeyboardShortcutViewerId);
+    const ash::ShelfID shelf_id(app_list::kInternalAppIdKeyboardShortcutViewer);
     window->SetProperty(ash::kShelfIDKey,
                         new std::string(shelf_id.Serialize()));
-    window->SetProperty<int>(ash::kShelfItemTypeKey, ash::TYPE_DIALOG);
+    window->SetProperty<int>(ash::kShelfItemTypeKey, ash::TYPE_APP);
 
     // We don't want the KSV window to have a title (per design), however the
     // shelf uses the window title to set the shelf item's tooltip text. The
     // shelf observes changes to the |kWindowIconKey| property and handles that
     // by initializing the shelf item including its tooltip text.
+    // TODO(wutao): we can remove resource id IDS_KSV_TITLE after implementing
+    // internal app shelf launcher.
     window->SetTitle(l10n_util::GetStringUTF16(IDS_KSV_TITLE));
     gfx::ImageSkia* icon =
         ui::ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
-            IDR_KEYBOARD_SHORTCUT_VIEWER_APP_ICON);
+            IDR_SHORTCUT_VIEWER_LOGO_192);
     // The new gfx::ImageSkia instance is owned by the window itself.
     window->SetProperty(aura::client::kWindowIconKey,
                         new gfx::ImageSkia(*icon));
 
+    g_ksv_view->AddAccelerator(
+        ui::Accelerator(ui::VKEY_W, ui::EF_CONTROL_DOWN));
+
     g_ksv_view->GetWidget()->Show();
-    g_ksv_view->RequestFocusForActiveTab();
+    g_ksv_view->search_box_view_->search_box()->RequestFocus();
+
+    UMA_HISTOGRAM_TIMES("Keyboard.ShortcutViewer.StartupTime",
+                        base::TimeTicks::Now() - start_time);
   }
   return g_ksv_view->GetWidget();
 }
 
-KeyboardShortcutView::KeyboardShortcutView() {
-  DCHECK_EQ(g_ksv_view, nullptr);
-  g_ksv_view = this;
-
-  // Default background is transparent.
-  SetBackground(views::CreateSolidBackground(SK_ColorWHITE));
-  InitViews();
-}
-
-void KeyboardShortcutView::InitViews() {
-  // Init search box view.
-  search_box_view_ = std::make_unique<KSVSearchBoxView>(this);
-  search_box_view_->Init();
-  AddChildView(search_box_view_.get());
-
-  // Init no search result illustration view.
-  search_no_result_view_ = std::make_unique<views::View>();
-  SetupSearchIllustrationView(search_no_result_view_.get(),
-                              kKsvSearchNoResultIcon, IDS_KSV_SEARCH_NO_RESULT);
-
-  // Init search results container view.
-  search_results_container_ = new views::View();
-  search_results_container_->SetLayoutManager(
-      std::make_unique<views::FillLayout>());
-  search_results_container_->SetVisible(false);
-  AddChildView(search_results_container_);
-
-  // Init views of KeyboardShortcutItemView.
-  for (const auto& item : GetKeyboardShortcutItemList()) {
-    for (auto category : item.categories) {
-      shortcut_views_.emplace_back(
-          new KeyboardShortcutItemView(item, category));
-    }
-  }
-  std::sort(shortcut_views_.begin(), shortcut_views_.end(),
-            [](KeyboardShortcutItemView* lhs, KeyboardShortcutItemView* rhs) {
-              if (lhs->category() != rhs->category())
-                return lhs->category() < rhs->category();
-              return lhs->description_label_view()->text() <
-                     rhs->description_label_view()->text();
-            });
-
-  // Init views of TabbedPane and KeyboardShortcutItemListView.
-  tabbed_pane_ =
-      new views::TabbedPane(views::TabbedPane::Orientation::kVertical,
-                            views::TabbedPane::TabStripStyle::kHighlight);
-  ShortcutCategory current_category = ShortcutCategory::kUnknown;
-  KeyboardShortcutItemListView* item_list_view;
-  for (auto* item_view : shortcut_views_) {
-    const ShortcutCategory category = item_view->category();
-    DCHECK_NE(ShortcutCategory::kUnknown, category);
-    if (current_category != category) {
-      current_category = category;
-      item_list_view = new KeyboardShortcutItemListView();
-      views::ScrollView* const scroller = CreateScrollView();
-      scroller->SetContents(item_list_view);
-      tabbed_pane_->AddTab(GetStringForCategory(current_category), scroller);
-    }
-    if (item_list_view->has_children())
-      item_list_view->AddHorizontalSeparator();
-    item_list_view->AddChildView(item_view);
-  }
-  AddChildView(tabbed_pane_);
-}
-
-void KeyboardShortcutView::RequestFocusForActiveTab() {
-  // Get the |tab_strip_| of the |tabbed_pane_| in order to set focus on
-  // the selected tab.
-  tabbed_pane_->child_at(0)
-      ->child_at(tabbed_pane_->GetSelectedTabIndex())
-      ->RequestFocus();
-}
-
-bool KeyboardShortcutView::CanMaximize() const {
-  return false;
-}
-
-bool KeyboardShortcutView::CanMinimize() const {
+bool KeyboardShortcutView::AcceleratorPressed(
+    const ui::Accelerator& accelerator) {
+  DCHECK_EQ(ui::VKEY_W, accelerator.key_code());
+  DCHECK_EQ(ui::EF_CONTROL_DOWN, accelerator.modifiers());
+  GetWidget()->Close();
   return true;
 }
 
-bool KeyboardShortcutView::CanResize() const {
-  return false;
-}
-
-views::ClientView* KeyboardShortcutView::CreateClientView(
-    views::Widget* widget) {
-  return new views::ClientView(widget, this);
-}
-
 void KeyboardShortcutView::Layout() {
   gfx::Rect content_bounds(GetContentsBounds());
   if (content_bounds.IsEmpty())
@@ -252,9 +180,9 @@ void KeyboardShortcutView::Layout() {
   search_box_bounds.set_y(top + kSearchBoxTopPadding);
   search_box_view_->SetBoundsRect(search_box_bounds);
 
-  views::View* content_view =
-      tabbed_pane_->visible() ? tabbed_pane_ : search_results_container_;
-
+  views::View* content_view = categories_tabbed_pane_->visible()
+                                  ? categories_tabbed_pane_
+                                  : search_results_container_;
   const int search_box_used_height = search_box_bounds.height() +
                                      kSearchBoxTopPadding +
                                      kSearchBoxBottomPadding;
@@ -263,9 +191,8 @@ void KeyboardShortcutView::Layout() {
                           content_bounds.height() - search_box_used_height);
 }
 
-void KeyboardShortcutView::BackButtonPressed() {
-  search_box_view_->ClearSearch();
-  search_box_view_->SetSearchBoxActive(false);
+gfx::Size KeyboardShortcutView::CalculatePreferredSize() const {
+  return gfx::Size(800, 512);
 }
 
 void KeyboardShortcutView::QueryChanged(search_box::SearchBoxViewBase* sender) {
@@ -275,19 +202,170 @@ void KeyboardShortcutView::QueryChanged(search_box::SearchBoxViewBase* sender) {
     UpdateViewsLayout(/*is_search_box_active=*/true);
   }
 
+  debounce_timer_.Stop();
   // If search box is empty, do not show |search_results_container_|.
   if (query_empty)
     return;
 
+  // TODO(wutao): This timeout value is chosen based on subjective search
+  // latency tests on Minnie. Objective method or UMA is desired.
+  constexpr base::TimeDelta kTimeOut(base::TimeDelta::FromMilliseconds(250));
+  debounce_timer_.Start(
+      FROM_HERE, kTimeOut,
+      base::Bind(&KeyboardShortcutView::ShowSearchResults,
+                 base::Unretained(this), sender->search_box()->text()));
+}
+
+void KeyboardShortcutView::BackButtonPressed() {
+  search_box_view_->ClearSearch();
+  search_box_view_->SetSearchBoxActive(false);
+}
+
+void KeyboardShortcutView::ActiveChanged(
+    search_box::SearchBoxViewBase* sender) {
+  const bool is_search_box_active = sender->is_search_box_active();
+  is_search_box_empty_ = sender->IsSearchBoxTrimmedQueryEmpty();
+  sender->ShowBackOrGoogleIcon(is_search_box_active);
+  if (is_search_box_active) {
+    base::RecordAction(
+        base::UserMetricsAction("KeyboardShortcutViewer.Search"));
+  }
+  UpdateViewsLayout(is_search_box_active);
+}
+
+KeyboardShortcutView::KeyboardShortcutView() {
+  DCHECK_EQ(g_ksv_view, nullptr);
+  g_ksv_view = this;
+
+  // Default background is transparent.
+  SetBackground(views::CreateSolidBackground(SK_ColorWHITE));
+  InitViews();
+}
+
+void KeyboardShortcutView::InitViews() {
+  TRACE_EVENT0("shortcut_viewer", "InitViews");
+  // Init search box view.
+  search_box_view_ = std::make_unique<KSVSearchBoxView>(this);
+  search_box_view_->Init();
+  AddChildView(search_box_view_.get());
+
+  // Init no search result illustration view.
+  search_no_result_view_ = std::make_unique<views::View>();
+  SetupSearchIllustrationView(search_no_result_view_.get(),
+                              kKsvSearchNoResultIcon, IDS_KSV_SEARCH_NO_RESULT);
+
+  // Init search results container view.
+  search_results_container_ = new views::View();
+  search_results_container_->SetLayoutManager(
+      std::make_unique<views::FillLayout>());
+  search_results_container_->SetVisible(false);
+  AddChildView(search_results_container_);
+
+  // Init views of KeyboardShortcutItemView.
+  // TODO(https://crbug.com/843394): Observe changes in keyboard layout and
+  // clear the cache.
+  KeyboardShortcutItemView::ClearKeycodeToString16Cache();
+  for (const auto& item : GetKeyboardShortcutItemList()) {
+    for (auto category : item.categories) {
+      shortcut_views_.emplace_back(
+          std::make_unique<KeyboardShortcutItemView>(item, category));
+      shortcut_views_.back()->set_owned_by_client();
+    }
+  }
+  std::sort(shortcut_views_.begin(), shortcut_views_.end(),
+            [](const auto& lhs, const auto& rhs) {
+              if (lhs->category() != rhs->category())
+                return lhs->category() < rhs->category();
+              return lhs->description_label_view()->text() <
+                     rhs->description_label_view()->text();
+            });
+
+  // Init views of |categories_tabbed_pane_| and KeyboardShortcutItemListViews.
+  categories_tabbed_pane_ =
+      new views::TabbedPane(views::TabbedPane::Orientation::kVertical,
+                            views::TabbedPane::TabStripStyle::kHighlight);
+  AddChildView(categories_tabbed_pane_);
+  InitCategoriesTabbedPane();
+}
+
+void KeyboardShortcutView::InitCategoriesTabbedPane() {
+  // If the tab count is 0, |GetSelectedTabIndex()| will return -1, which we do
+  // not want to cache.
+  active_tab_index_ =
+      std::max(0, categories_tabbed_pane_->GetSelectedTabIndex());
+  // Although we remove all child views, when the KeyboardShortcutItemView is
+  // added back to the |categories_tabbed_pane_|, because there is no width
+  // changes, it will not layout the KeyboardShortcutItemView again due to the
+  // |MaybeCalculateAndDoLayout()| optimization in KeyboardShortcutItemView.
+  // Cannot remove |tab_strip_| and |contents_|, child views of the
+  // |categories_tabbed_pane_|, because they are added in the ctor of
+  // TabbedPane.
+  categories_tabbed_pane_->child_at(0)->RemoveAllChildViews(true);
+  categories_tabbed_pane_->child_at(1)->RemoveAllChildViews(true);
+
+  ShortcutCategory current_category = ShortcutCategory::kUnknown;
+  KeyboardShortcutItemListView* item_list_view;
+  for (const auto& item_view : shortcut_views_) {
+    const ShortcutCategory category = item_view->category();
+    DCHECK_NE(ShortcutCategory::kUnknown, category);
+    if (current_category != category) {
+      current_category = category;
+      item_list_view = new KeyboardShortcutItemListView();
+      views::ScrollView* const scroller = CreateScrollView();
+      scroller->SetContents(item_list_view);
+      categories_tabbed_pane_->AddTab(GetStringForCategory(current_category),
+                                      scroller);
+    }
+    if (item_list_view->has_children())
+      item_list_view->AddHorizontalSeparator();
+    views::StyledLabel* description_label_view =
+        item_view->description_label_view();
+    // Clear any styles used to highlight matched search query in search mode.
+    description_label_view->ClearStyleRanges();
+    item_list_view->AddChildView(item_view.get());
+    // Remove the search query highlight.
+    description_label_view->Layout();
+  }
+}
+
+void KeyboardShortcutView::UpdateViewsLayout(bool is_search_box_active) {
+  // 1. Search box is not active: show |categories_tabbed_pane_| and focus on
+  //    active tab.
+  // 2. Search box is active and empty: show |categories_tabbed_pane_| but focus
+  //    on search box.
+  // 3. Search box is not empty, show |search_results_container_|. Focus is on
+  //    search box.
+  const bool should_show_search_results =
+      is_search_box_active && !is_search_box_empty_;
+  if (!should_show_search_results) {
+    // Remove all child views, including horizontal separator lines, to prepare
+    // for showing search results next time.
+    search_results_container_->RemoveAllChildViews(true);
+    if (!categories_tabbed_pane_->visible()) {
+      // Repopulate |categories_tabbed_pane_| child views, which were removed
+      // when they were added to |search_results_container_|.
+      InitCategoriesTabbedPane();
+      // Select the category that was active before entering search mode.
+      categories_tabbed_pane_->SelectTabAt(active_tab_index_);
+    }
+  }
+  categories_tabbed_pane_->SetVisible(!should_show_search_results);
+  search_results_container_->SetVisible(should_show_search_results);
+  Layout();
+  SchedulePaint();
+}
+
+void KeyboardShortcutView::ShowSearchResults(
+    const base::string16& search_query) {
   search_results_container_->RemoveAllChildViews(true);
   auto* search_container_content_view = search_no_result_view_.get();
   auto found_items_list_view = std::make_unique<KeyboardShortcutItemListView>();
-  const base::string16& new_contents = sender->search_box()->text();
   base::i18n::FixedPatternStringSearchIgnoringCaseAndAccents finder(
-      new_contents);
+      search_query);
   ShortcutCategory current_category = ShortcutCategory::kUnknown;
   bool has_category_item = false;
-  for (auto* item_view : shortcut_views_) {
+  int number_search_results = 0;
+  for (const auto& item_view : shortcut_views_) {
     base::string16 description_text =
         item_view->description_label_view()->text();
     base::string16 shortcut_text = item_view->shortcut_label_view()->text();
@@ -310,24 +388,31 @@ void KeyboardShortcutView::QueryChanged(search_box::SearchBoxViewBase* sender) {
         found_items_list_view->AddHorizontalSeparator();
       else
         has_category_item = true;
-      auto* matched_item_view =
-          new KeyboardShortcutItemView(*item_view->shortcut_item(), category);
       // Highlight matched query in |description_label_view_|.
       if (match_length > 0) {
         views::StyledLabel::RangeStyleInfo style;
         views::StyledLabel* description_label_view =
-            matched_item_view->description_label_view();
+            item_view->description_label_view();
+        // Clear previous styles.
+        description_label_view->ClearStyleRanges();
         style.custom_font = description_label_view->GetDefaultFontList().Derive(
             0, gfx::Font::FontStyle::NORMAL, gfx::Font::Weight::BOLD);
         description_label_view->AddStyleRange(
             gfx::Range(match_index, match_index + match_length), style);
+        // Apply new styles to highlight matched search query.
+        description_label_view->Layout();
       }
 
-      found_items_list_view->AddChildView(matched_item_view);
+      found_items_list_view->AddChildView(item_view.get());
+      ++number_search_results;
     }
   }
 
+  std::vector<base::string16> replacement_strings;
   if (found_items_list_view->has_children()) {
+    replacement_strings.emplace_back(
+        base::NumberToString16(number_search_results));
+
     // To offset the padding between the bottom of the |search_box_view_| and
     // the top of the |search_results_container_|.
     constexpr int kTopPadding = -16;
@@ -338,39 +423,32 @@ void KeyboardShortcutView::QueryChanged(search_box::SearchBoxViewBase* sender) {
     scroller->SetContents(found_items_list_view.release());
     search_container_content_view = scroller;
   }
+  replacement_strings.emplace_back(search_query);
+  search_box_view_->SetAccessibleValue(l10n_util::GetStringFUTF16(
+      number_search_results == 0
+          ? IDS_KSV_SEARCH_BOX_ACCESSIBILITY_VALUE_WITHOUT_RESULTS
+          : IDS_KSV_SEARCH_BOX_ACCESSIBILITY_VALUE_WITH_RESULTS,
+      replacement_strings, nullptr));
   search_results_container_->AddChildView(search_container_content_view);
   Layout();
   SchedulePaint();
 }
 
-void KeyboardShortcutView::ActiveChanged(
-    search_box::SearchBoxViewBase* sender) {
-  const bool is_search_box_active = sender->is_search_box_active();
-  is_search_box_empty_ = sender->IsSearchBoxTrimmedQueryEmpty();
-  sender->ShowBackOrGoogleIcon(is_search_box_active);
-  if (is_search_box_active) {
-    base::RecordAction(
-        base::UserMetricsAction("KeyboardShortcutViewer.Search"));
-  }
-  UpdateViewsLayout(is_search_box_active);
+bool KeyboardShortcutView::CanMaximize() const {
+  return false;
 }
 
-void KeyboardShortcutView::UpdateViewsLayout(bool is_search_box_active) {
-  // 1. Search box is not active: show |tabbed_pane_| and focus on active tab.
-  // 2. Search box is active and empty: show |tabbed_pane_| but focus on search
-  //    box.
-  // 3. Search box is not empty, show |search_results_container_|. Focus is on
-  //    search box.
-  const bool should_show_search_results =
-      is_search_box_active && !is_search_box_empty_;
-  search_results_container_->SetVisible(should_show_search_results);
-  tabbed_pane_->SetVisible(!should_show_search_results);
-  if (!is_search_box_active) {
-    search_results_container_->RemoveAllChildViews(true);
-    RequestFocusForActiveTab();
-  }
-  Layout();
-  SchedulePaint();
+bool KeyboardShortcutView::CanMinimize() const {
+  return true;
+}
+
+bool KeyboardShortcutView::CanResize() const {
+  return false;
+}
+
+views::ClientView* KeyboardShortcutView::CreateClientView(
+    views::Widget* widget) {
+  return new views::ClientView(widget, this);
 }
 
 KeyboardShortcutView* KeyboardShortcutView::GetInstanceForTesting() {
@@ -378,7 +456,16 @@ KeyboardShortcutView* KeyboardShortcutView::GetInstanceForTesting() {
 }
 
 int KeyboardShortcutView::GetTabCountForTesting() const {
-  return tabbed_pane_->GetTabCount();
+  return categories_tabbed_pane_->GetTabCount();
+}
+
+const std::vector<std::unique_ptr<KeyboardShortcutItemView>>&
+KeyboardShortcutView::GetShortcutViewsForTesting() const {
+  return shortcut_views_;
+}
+
+KSVSearchBoxView* KeyboardShortcutView::GetSearchBoxViewForTesting() {
+  return search_box_view_.get();
 }
 
 }  // namespace keyboard_shortcut_viewer
