@@ -6,10 +6,13 @@
 
 #include <memory>
 
-#include "ash/fast_ink/fast_ink_points.h"
+#include "ash/components/fast_ink/fast_ink_points.h"
 #include "ash/highlighter/highlighter_controller_test_api.h"
 #include "ash/public/cpp/config.h"
 #include "ash/shell.h"
+#include "ash/system/palette/mock_palette_tool_delegate.h"
+#include "ash/system/palette/palette_tool.h"
+#include "ash/system/palette/tools/metalayer_mode.h"
 #include "ash/test/ash_test_base.h"
 #include "base/strings/stringprintf.h"
 #include "ui/aura/window_tree_host.h"
@@ -19,6 +22,36 @@
 namespace ash {
 namespace {
 
+class TestHighlighterObserver : public HighlighterController::Observer {
+ public:
+  TestHighlighterObserver() = default;
+  ~TestHighlighterObserver() override = default;
+
+  // HighlighterController::Observer:
+  void OnHighlighterEnabledChanged(HighlighterEnabledState state) override {
+    if (state == HighlighterEnabledState::kEnabled) {
+      ++enabled_count_;
+    } else if (state == HighlighterEnabledState::kDisabledByUser) {
+      ++disabled_by_user_count_;
+    } else {
+      DCHECK_EQ(HighlighterEnabledState::kDisabledBySessionEnd, state);
+      ++disabled_by_session_end_;
+    }
+  }
+
+  void OnHighlighterSelectionRecognized(const gfx::Rect& rect) override {
+    last_recognized_rect_ = rect;
+  }
+
+  int enabled_count_ = 0;
+  int disabled_by_user_count_ = 0;
+  int disabled_by_session_end_ = 0;
+  gfx::Rect last_recognized_rect_;
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(TestHighlighterObserver);
+};
+
 class HighlighterControllerTest : public AshTestBase {
  public:
   HighlighterControllerTest() = default;
@@ -26,11 +59,16 @@ class HighlighterControllerTest : public AshTestBase {
 
   void SetUp() override {
     AshTestBase::SetUp();
-    controller_test_api_ = std::make_unique<HighlighterControllerTestApi>(
-        Shell::Get()->highlighter_controller());
+    controller_ = Shell::Get()->highlighter_controller();
+    controller_test_api_ =
+        std::make_unique<HighlighterControllerTestApi>(controller_);
+
+    palette_tool_delegate_ = std::make_unique<MockPaletteToolDelegate>();
+    tool_ = std::make_unique<MetalayerMode>(palette_tool_delegate_.get());
   }
 
   void TearDown() override {
+    tool_.reset();
     // This needs to be called first to reset the controller state before the
     // shell instance gets torn down.
     controller_test_api_.reset();
@@ -61,6 +99,10 @@ class HighlighterControllerTest : public AshTestBase {
   }
 
   std::unique_ptr<HighlighterControllerTestApi> controller_test_api_;
+  std::unique_ptr<MockPaletteToolDelegate> palette_tool_delegate_;
+  std::unique_ptr<PaletteTool> tool_;
+
+  HighlighterController* controller_ = nullptr;  // Not owned.
 
  private:
   DISALLOW_COPY_AND_ASSIGN(HighlighterControllerTest);
@@ -174,6 +216,9 @@ TEST_F(HighlighterControllerTest, HighlighterGestures) {
   controller_test_api_->SetEnabled(true);
   GetEventGenerator().EnterPenPointerMode();
 
+  TestHighlighterObserver observer;
+  controller_->AddObserver(&observer);
+
   // A non-horizontal stroke is not recognized
   controller_test_api_->ResetSelection();
   GetEventGenerator().MoveTouch(gfx::Point(100, 100));
@@ -194,7 +239,9 @@ TEST_F(HighlighterControllerTest, HighlighterGestures) {
   //   have the same horizontal center line as the stroke bounding box,
   //   be 4dp wider than the stroke bounding box,
   //   be exactly 14dp high.
-  EXPECT_EQ("98,94 204x14", controller_test_api_->selection().ToString());
+  gfx::Rect expected_rect(98, 94, 204, 14);
+  EXPECT_EQ(expected_rect, controller_test_api_->selection());
+  EXPECT_EQ(expected_rect, observer.last_recognized_rect_);
 
   // An insufficiently closed C-like shape is not recognized
   controller_test_api_->ResetSelection();
@@ -216,7 +263,9 @@ TEST_F(HighlighterControllerTest, HighlighterGestures) {
   GetEventGenerator().MoveTouch(gfx::Point(200, 20));
   GetEventGenerator().ReleaseTouch();
   EXPECT_TRUE(controller_test_api_->HandleSelectionCalled());
-  EXPECT_EQ("0,0 200x100", controller_test_api_->selection().ToString());
+  expected_rect = gfx::Rect(0, 0, 200, 100);
+  EXPECT_EQ(expected_rect, controller_test_api_->selection());
+  EXPECT_EQ(expected_rect, observer.last_recognized_rect_);
 
   // A closed diamond shape is recognized
   controller_test_api_->ResetSelection();
@@ -228,7 +277,11 @@ TEST_F(HighlighterControllerTest, HighlighterGestures) {
   GetEventGenerator().MoveTouch(gfx::Point(100, 50));
   GetEventGenerator().ReleaseTouch();
   EXPECT_TRUE(controller_test_api_->HandleSelectionCalled());
-  EXPECT_EQ("0,50 200x200", controller_test_api_->selection().ToString());
+  expected_rect = gfx::Rect(0, 50, 200, 200);
+  EXPECT_EQ(expected_rect, controller_test_api_->selection());
+  EXPECT_EQ(expected_rect, observer.last_recognized_rect_);
+
+  controller_->RemoveObserver(&observer);
 }
 
 TEST_F(HighlighterControllerTest, HighlighterGesturesScaled) {
@@ -461,4 +514,36 @@ TEST_F(HighlighterControllerTest, DetachedClient) {
   EXPECT_TRUE(controller_test_api_->HandleSelectionCalled());
 }
 
+// Test enabling/disabling metalayer mode by selecting/deselecting on palette
+// tool and calling UpdateEnabledState notify observers properly.
+TEST_F(HighlighterControllerTest, UpdateEnabledState) {
+  TestHighlighterObserver observer;
+  controller_->AddObserver(&observer);
+
+  ASSERT_EQ(0, observer.enabled_count_);
+  ASSERT_EQ(0, observer.disabled_by_user_count_);
+  ASSERT_EQ(0, observer.disabled_by_session_end_);
+  tool_->OnEnable();
+  EXPECT_EQ(1, observer.enabled_count_);
+  EXPECT_EQ(0, observer.disabled_by_user_count_);
+  EXPECT_EQ(0, observer.disabled_by_session_end_);
+
+  tool_->OnDisable();
+  EXPECT_EQ(1, observer.enabled_count_);
+  EXPECT_EQ(1, observer.disabled_by_user_count_);
+  EXPECT_EQ(0, observer.disabled_by_session_end_);
+
+  tool_->OnEnable();
+  EXPECT_EQ(2, observer.enabled_count_);
+  EXPECT_EQ(1, observer.disabled_by_user_count_);
+  EXPECT_EQ(0, observer.disabled_by_session_end_);
+  controller_->UpdateEnabledState(
+      HighlighterEnabledState::kDisabledBySessionEnd);
+  EXPECT_EQ(2, observer.enabled_count_);
+  EXPECT_EQ(1, observer.disabled_by_user_count_);
+  EXPECT_EQ(1, observer.disabled_by_session_end_);
+
+  controller_->RemoveObserver(&observer);
+}
+
 }  // namespace ash
