@@ -10,11 +10,14 @@
 #include "ash/host/transformer_helper.h"
 #include "ash/shell.h"
 #include "ash/shell_delegate.h"
+#include "base/feature_list.h"
 #include "base/trace_event/trace_event.h"
 #include "services/ui/public/cpp/input_devices/input_device_controller_client.h"
 #include "services/ui/public/interfaces/window_manager.mojom.h"
+#include "ui/aura/mus/input_method_mus.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_tree_host_platform.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/events/event_sink.h"
 #include "ui/events/null_event_targeter.h"
 #include "ui/events/ozone/chromeos/cursor_controller.h"
@@ -22,20 +25,27 @@
 #include "ui/gfx/geometry/rect_conversions.h"
 #include "ui/gfx/geometry/rect_f.h"
 #include "ui/gfx/transform.h"
+#include "ui/platform_window/mojo/ime_type_converters.h"
+#include "ui/platform_window/platform_ime_controller.h"
 #include "ui/platform_window/platform_window.h"
+#include "ui/platform_window/platform_window_init_properties.h"
+#include "ui/platform_window/text_input_state.h"
 
 namespace ash {
 
 AshWindowTreeHostPlatform::AshWindowTreeHostPlatform(
-    const gfx::Rect& initial_bounds)
-    : aura::WindowTreeHostPlatform(initial_bounds), transformer_helper_(this) {
+    ui::PlatformWindowInitProperties properties)
+    : aura::WindowTreeHostPlatform(std::move(properties)),
+      transformer_helper_(this) {
   transformer_helper_.Init();
+  InitInputMethodIfNecessary();
 }
 
 AshWindowTreeHostPlatform::AshWindowTreeHostPlatform()
     : transformer_helper_(this) {
   CreateCompositor();
   transformer_helper_.Init();
+  InitInputMethodIfNecessary();
 }
 
 AshWindowTreeHostPlatform::~AshWindowTreeHostPlatform() = default;
@@ -85,6 +95,17 @@ void AshWindowTreeHostPlatform::ClearCursorConfig() {
       GetAcceleratedWidget());
 }
 
+void AshWindowTreeHostPlatform::UpdateTextInputState(
+    ui::mojom::TextInputStatePtr state) {
+  SetTextInputState(std::move(state));
+}
+
+void AshWindowTreeHostPlatform::UpdateImeVisibility(
+    bool visible,
+    ui::mojom::TextInputStatePtr state) {
+  SetImeVisibility(visible, std::move(state));
+}
+
 void AshWindowTreeHostPlatform::SetRootWindowTransformer(
     std::unique_ptr<RootWindowTransformer> transformer) {
   transformer_helper_.SetRootWindowTransformer(std::move(transformer));
@@ -125,17 +146,19 @@ gfx::Transform AshWindowTreeHostPlatform::GetInverseRootTransform() const {
   return transformer_helper_.GetInverseTransform();
 }
 
-void AshWindowTreeHostPlatform::UpdateRootWindowSizeInPixels(
-    const gfx::Size& host_size_in_pixels) {
-  transformer_helper_.UpdateWindowSize(host_size_in_pixels);
+gfx::Rect AshWindowTreeHostPlatform::GetTransformedRootWindowBoundsInPixels(
+    const gfx::Size& host_size_in_pixels) const {
+  return transformer_helper_.GetTransformedWindowBounds(host_size_in_pixels);
 }
 
 void AshWindowTreeHostPlatform::OnCursorVisibilityChangedNative(bool show) {
   SetTapToClickPaused(!show);
 }
 
-void AshWindowTreeHostPlatform::SetBoundsInPixels(const gfx::Rect& bounds) {
-  WindowTreeHostPlatform::SetBoundsInPixels(bounds);
+void AshWindowTreeHostPlatform::SetBoundsInPixels(
+    const gfx::Rect& bounds,
+    const viz::LocalSurfaceId& local_surface_id) {
+  WindowTreeHostPlatform::SetBoundsInPixels(bounds, local_surface_id);
   ConfineCursorToRootWindow();
 }
 
@@ -146,6 +169,15 @@ void AshWindowTreeHostPlatform::DispatchEvent(ui::Event* event) {
   SendEventToSink(event);
 }
 
+void AshWindowTreeHostPlatform::InitInputMethodIfNecessary() {
+  if (!base::FeatureList::IsEnabled(features::kOopAsh))
+    return;
+
+  input_method_ = std::make_unique<aura::InputMethodMus>(this, this);
+  input_method_->Init(Shell::Get()->connector());
+  SetSharedInputMethod(input_method_.get());
+}
+
 void AshWindowTreeHostPlatform::SetTapToClickPaused(bool state) {
   ui::InputDeviceControllerClient* input_device_controller_client =
       Shell::Get()->shell_delegate()->GetInputDeviceControllerClient();
@@ -156,4 +188,24 @@ void AshWindowTreeHostPlatform::SetTapToClickPaused(bool state) {
   input_device_controller_client->SetTapToClickPaused(state);
 }
 
+void AshWindowTreeHostPlatform::SetTextInputState(
+    ui::mojom::TextInputStatePtr state) {
+  ui::PlatformImeController* ime =
+      platform_window()->GetPlatformImeController();
+  if (ime)
+    ime->UpdateTextInputState(state.To<ui::TextInputState>());
+}
+
+void AshWindowTreeHostPlatform::SetImeVisibility(
+    bool visible,
+    ui::mojom::TextInputStatePtr state) {
+  if (!state.is_null())
+    SetTextInputState(std::move(state));
+
+  ui::PlatformImeController* ime =
+      platform_window()->GetPlatformImeController();
+  if (ime)
+    ime->SetImeVisibility(visible);
+}
+
 }  // namespace ash
