@@ -6,6 +6,8 @@
 #define ASH_LOGIN_LOGIN_SCREEN_CONTROLLER_H_
 
 #include "ash/ash_export.h"
+#include "ash/login/login_screen_controller_observer.h"
+#include "ash/public/interfaces/kiosk_app_info.mojom.h"
 #include "ash/public/interfaces/login_screen.mojom.h"
 #include "base/macros.h"
 #include "base/observer_list.h"
@@ -16,7 +18,6 @@ class PrefRegistrySimple;
 
 namespace ash {
 
-class LockScreenAppsFocusObserver;
 class LoginDataDispatcher;
 
 // LoginScreenController implements mojom::LoginScreen and wraps the
@@ -26,6 +27,14 @@ class LoginDataDispatcher;
 // LoginScreenClient through mojo.
 class ASH_EXPORT LoginScreenController : public mojom::LoginScreen {
  public:
+  // The current authentication stage. Used to get more verbose logging.
+  enum class AuthenticationStage {
+    kIdle,
+    kGetSystemSalt,
+    kDoAuthenticate,
+    kUserCallback,
+  };
+
   using OnShownCallback = base::OnceCallback<void(bool did_show)>;
   // Callback for authentication checks. |success| is nullopt if an
   // authentication check did not run, otherwise it is true/false if auth
@@ -41,6 +50,50 @@ class ASH_EXPORT LoginScreenController : public mojom::LoginScreen {
   // Binds the mojom::LoginScreen interface to this object.
   void BindRequest(mojom::LoginScreenRequest request);
 
+  // Check to see if an authentication attempt is in-progress.
+  bool IsAuthenticating() const;
+
+  // Hash the password and send AuthenticateUser request to LoginScreenClient.
+  // LoginScreenClient (in the chrome process) will do the authentication and
+  // request to show error messages in the screen if auth fails, or request to
+  // clear errors if auth succeeds.
+  void AuthenticateUser(const AccountId& account_id,
+                        const std::string& password,
+                        bool authenticated_by_pin,
+                        OnAuthenticateCallback callback);
+  void AttemptUnlock(const AccountId& account_id);
+  void HardlockPod(const AccountId& account_id);
+  void RecordClickOnLockIcon(const AccountId& account_id);
+  void OnFocusPod(const AccountId& account_id);
+  void OnNoPodFocused();
+  void LoadWallpaper(const AccountId& account_id);
+  void SignOutUser();
+  void CancelAddUser();
+  void LoginAsGuest();
+  void OnMaxIncorrectPasswordAttempted(const AccountId& account_id);
+  void FocusLockScreenApps(bool reverse);
+  void ShowGaiaSignin(bool can_close,
+                      const base::Optional<AccountId>& prefilled_account);
+  void OnRemoveUserWarningShown();
+  void RemoveUser(const AccountId& account_id);
+  void LaunchPublicSession(const AccountId& account_id,
+                           const std::string& locale,
+                           const std::string& input_method);
+  void RequestPublicSessionKeyboardLayouts(const AccountId& account_id,
+                                           const std::string& locale);
+  void ShowFeedback();
+  void LaunchKioskApp(const std::string& app_id);
+
+  // Add or remove an observer.
+  void AddObserver(LoginScreenControllerObserver* observer);
+  void RemoveObserver(LoginScreenControllerObserver* observer);
+
+  // Enable or disable authentication for the debug overlay.
+  enum class ForceFailAuth { kOff, kImmediate, kDelayed };
+  void set_force_fail_auth_for_debug_overlay(ForceFailAuth force_fail) {
+    force_fail_auth_for_debug_overlay_ = force_fail;
+  }
+
   // mojom::LoginScreen:
   void SetClient(mojom::LoginScreenClientPtr client) override;
   void ShowLockScreen(ShowLockScreenCallback on_shown) override;
@@ -60,47 +113,35 @@ class ASH_EXPORT LoginScreenController : public mojom::LoginScreen {
                  bool show_guest) override;
   void SetPinEnabledForUser(const AccountId& account_id,
                             bool is_enabled) override;
+  void SetAvatarForUser(const AccountId& account_id,
+                        mojom::UserAvatarPtr avatar) override;
+  void SetAuthEnabledForUser(
+      const AccountId& account_id,
+      bool is_enabled,
+      base::Optional<base::Time> auth_reenabled_time) override;
   void HandleFocusLeavingLockScreenApps(bool reverse) override;
   void SetDevChannelInfo(const std::string& os_version_label_text,
                          const std::string& enterprise_info_text,
                          const std::string& bluetooth_name) override;
   void IsReadyForPassword(IsReadyForPasswordCallback callback) override;
-
-  // Wrappers around the mojom::LoginScreenClient interface. Hash the password
-  // and send AuthenticateUser request to LoginScreenClient.
-  // LoginScreenClient(chrome) will do the authentication and request to show
-  // error messages in the screen if auth fails, or request to clear errors if
-  // auth succeeds.
-  void AuthenticateUser(const AccountId& account_id,
-                        const std::string& password,
-                        bool authenticated_by_pin,
-                        OnAuthenticateCallback callback);
-  void AttemptUnlock(const AccountId& account_id);
-  void HardlockPod(const AccountId& account_id);
-  void RecordClickOnLockIcon(const AccountId& account_id);
-  void OnFocusPod(const AccountId& account_id);
-  void OnNoPodFocused();
-  void LoadWallpaper(const AccountId& account_id);
-  void SignOutUser();
-  void CancelAddUser();
-  void LoginAsGuest();
-  void OnMaxIncorrectPasswordAttempted(const AccountId& account_id);
-  void FocusLockScreenApps(bool reverse);
-  void ShowGaiaSignin();
-
-  // Methods to manage lock screen apps focus observers.
-  // The observers will be notified when lock screen apps focus changes are
-  // reported via lock screen mojo interface.
-  void AddLockScreenAppsFocusObserver(LockScreenAppsFocusObserver* observer);
-  void RemoveLockScreenAppsFocusObserver(LockScreenAppsFocusObserver* observer);
-
+  void SetPublicSessionDisplayName(const AccountId& account_id,
+                                   const std::string& display_name) override;
+  void SetPublicSessionLocales(const AccountId& account_id,
+                               std::vector<mojom::LocaleItemPtr> locales,
+                               const std::string& default_locale,
+                               bool show_advanced_view) override;
+  void SetPublicSessionKeyboardLayouts(
+      const AccountId& account_id,
+      const std::string& locale,
+      std::vector<mojom::InputMethodItemPtr> keyboard_layouts) override;
+  void SetFingerprintUnlockState(const AccountId& account_id,
+                                 mojom::FingerprintUnlockState state) override;
+  void SetKioskApps(std::vector<mojom::KioskAppInfoPtr> kiosk_apps) override;
   // Flushes the mojo pipes - to be used in tests.
   void FlushForTesting();
 
-  // Enable or disable authentication for the debug overlay.
-  enum class ForceFailAuth { kOff, kImmediate, kDelayed };
-  void set_force_fail_auth_for_debug_overlay(ForceFailAuth force_fail) {
-    force_fail_auth_for_debug_overlay_ = force_fail;
+  AuthenticationStage authentication_stage() const {
+    return authentication_stage_;
   }
 
  private:
@@ -114,9 +155,6 @@ class ASH_EXPORT LoginScreenController : public mojom::LoginScreen {
                           const std::string& system_salt);
   void OnAuthenticateComplete(OnAuthenticateCallback callback, bool success);
 
-  void OnGetSystemSalt(PendingDoAuthenticateUser then,
-                       const std::string& system_salt);
-
   // Returns the active data dispatcher or nullptr if there is no lock screen.
   LoginDataDispatcher* DataDispatcher() const;
 
@@ -129,11 +167,9 @@ class ASH_EXPORT LoginScreenController : public mojom::LoginScreen {
   // Bindings for users of the LockScreen interface.
   mojo::BindingSet<mojom::LoginScreen> bindings_;
 
-  // True iff we are currently authentication.
-  bool is_authenticating_ = false;
+  AuthenticationStage authentication_stage_ = AuthenticationStage::kIdle;
 
-  base::ObserverList<LockScreenAppsFocusObserver>
-      lock_screen_apps_focus_observers_;
+  base::ObserverList<LoginScreenControllerObserver> observers_;
 
   // If set to false, all auth requests will forcibly fail.
   ForceFailAuth force_fail_auth_for_debug_overlay_ = ForceFailAuth::kOff;
