@@ -6,15 +6,20 @@
 
 #include "ash/login/login_screen_controller.h"
 #include "ash/login/mock_login_screen_client.h"
+#include "ash/login/ui/fake_login_detachable_base_model.h"
 #include "ash/login/ui/lock_contents_view.h"
+#include "ash/login/ui/login_auth_user_view.h"
+#include "ash/login/ui/login_big_user_view.h"
+#include "ash/login/ui/login_bubble.h"
 #include "ash/login/ui/login_test_base.h"
 #include "ash/login/ui/login_test_utils.h"
 #include "ash/public/cpp/config.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/test_session_controller_client.h"
+#include "ash/shelf/shelf.h"
 #include "ash/shelf/shelf_widget.h"
 #include "ash/shell.h"
-#include "ash/system/tray/system_tray.h"
+#include "ash/system/status_area_widget.h"
 #include "base/run_loop.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/events/test/event_generator.h"
@@ -102,8 +107,10 @@ testing::AssertionResult VerifyNotFocused(views::View* view) {
 // Verifies that the password input box has focus.
 TEST_F(LockScreenSanityTest, PasswordIsInitiallyFocused) {
   // Build lock screen.
-  auto* contents = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                        data_dispatcher());
+  auto* contents = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
 
   // The lock screen requires at least one user.
   SetUserCount(1);
@@ -111,15 +118,18 @@ TEST_F(LockScreenSanityTest, PasswordIsInitiallyFocused) {
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(contents);
 
   // Textfield should have focus.
-  EXPECT_EQ(MakeLoginPasswordTestApi(contents).textfield(),
-            contents->GetFocusManager()->GetFocusedView());
+  EXPECT_EQ(
+      MakeLoginPasswordTestApi(contents, AuthTarget::kPrimary).textfield(),
+      contents->GetFocusManager()->GetFocusedView());
 }
 
 // Verifies submitting the password invokes mojo lock screen client.
 TEST_F(LockScreenSanityTest, PasswordSubmitCallsLoginScreenClient) {
   // Build lock screen.
-  auto* contents = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                        data_dispatcher());
+  auto* contents = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
 
   // The lock screen requires at least one user.
   SetUserCount(1);
@@ -129,9 +139,9 @@ TEST_F(LockScreenSanityTest, PasswordSubmitCallsLoginScreenClient) {
   // Password submit runs mojo.
   std::unique_ptr<MockLoginScreenClient> client = BindMockLoginScreenClient();
   client->set_authenticate_user_callback_result(false);
-  EXPECT_CALL(*client,
-              AuthenticateUser_(users()[0]->basic_user_info->account_id, _, _,
-                                false, _));
+  EXPECT_CALL(
+      *client,
+      AuthenticateUser_(users()[0]->basic_user_info->account_id, _, false, _));
   ui::test::EventGenerator& generator = GetEventGenerator();
   generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
   generator.PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
@@ -144,19 +154,21 @@ TEST_F(LockScreenSanityTest,
        PasswordSubmitClearsPasswordAfterFailedAuthentication) {
   std::unique_ptr<MockLoginScreenClient> client = BindMockLoginScreenClient();
 
-  auto* contents = new LockContentsView(mojom::TrayActionState::kAvailable,
-                                        data_dispatcher());
+  auto* contents = new LockContentsView(
+      mojom::TrayActionState::kAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
   SetUserCount(1);
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(contents);
   LoginPasswordView::TestApi password_test_api =
-      MakeLoginPasswordTestApi(contents);
+      MakeLoginPasswordTestApi(contents, AuthTarget::kPrimary);
 
   MockLoginScreenClient::AuthenticateUserCallback callback;
   auto submit_password = [&]() {
     // Capture the authentication callback.
     client->set_authenticate_user_callback_storage(&callback);
     EXPECT_CALL(*client, AuthenticateUser_(testing::_, testing::_, testing::_,
-                                           testing::_, testing::_));
+                                           testing::_));
 
     // Submit password with content 'a'. This creates a browser-process
     // authentication request stored in |callback|.
@@ -197,8 +209,10 @@ TEST_F(LockScreenSanityTest, TabGoesFromLockToShelfAndBackToLock) {
       session_manager::SessionState::LOCKED);
 
   // Create lock screen.
-  auto* lock = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                    data_dispatcher());
+  auto* lock = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
   SetUserCount(1);
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(lock);
   views::View* shelf = Shelf::ForWindow(lock->GetWidget()->GetNativeWindow())
@@ -228,14 +242,15 @@ TEST_F(LockScreenSanityTest, ShiftTabGoesFromLockToStatusAreaAndBackToLock) {
   GetSessionControllerClient()->SetSessionState(
       session_manager::SessionState::LOCKED);
 
-  auto* lock = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                    data_dispatcher());
+  auto* lock = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
   SetUserCount(1);
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(lock);
   views::View* status_area =
       RootWindowController::ForWindow(lock->GetWidget()->GetNativeWindow())
-          ->GetSystemTray()
-          ->GetWidget()
+          ->GetStatusAreaWidget()
           ->GetContentsView();
 
   // Lock screen has focus.
@@ -258,8 +273,10 @@ TEST_F(LockScreenSanityTest, TabWithLockScreenAppActive) {
   GetSessionControllerClient()->SetSessionState(
       session_manager::SessionState::LOCKED);
 
-  auto* lock = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                    data_dispatcher());
+  auto* lock = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
   SetUserCount(1);
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(lock);
 
@@ -269,8 +286,7 @@ TEST_F(LockScreenSanityTest, TabWithLockScreenAppActive) {
 
   views::View* status_area =
       RootWindowController::ForWindow(lock->GetWidget()->GetNativeWindow())
-          ->GetSystemTray()
-          ->GetWidget()
+          ->GetStatusAreaWidget()
           ->GetContentsView();
 
   LoginScreenController* controller = Shell::Get()->login_screen_controller();
@@ -329,8 +345,10 @@ TEST_F(LockScreenSanityTest, FocusLockScreenWhenLockScreenAppExit) {
   // Set up lock screen.
   GetSessionControllerClient()->SetSessionState(
       session_manager::SessionState::LOCKED);
-  auto* lock = new LockContentsView(mojom::TrayActionState::kNotAvailable,
-                                    data_dispatcher());
+  auto* lock = new LockContentsView(
+      mojom::TrayActionState::kNotAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
   SetUserCount(1);
   std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(lock);
 
@@ -362,4 +380,73 @@ TEST_F(LockScreenSanityTest, FocusLockScreenWhenLockScreenAppExit) {
   EXPECT_TRUE(VerifyFocused(shelf));
 }
 
+TEST_F(LockScreenSanityTest, RemoveUser) {
+  std::unique_ptr<MockLoginScreenClient> client = BindMockLoginScreenClient();
+  LoginScreenController* controller =
+      ash::Shell::Get()->login_screen_controller();
+
+  auto* contents = new LockContentsView(
+      mojom::TrayActionState::kAvailable, LockScreen::ScreenType::kLock,
+      data_dispatcher(),
+      std::make_unique<FakeLoginDetachableBaseModel>(data_dispatcher()));
+
+  // Add two users, the first of which can be removed.
+  users().push_back(CreateUser("test1@test"));
+  users()[0]->can_remove = true;
+  users().push_back(CreateUser("test2@test"));
+  data_dispatcher()->NotifyUsers(users());
+
+  std::unique_ptr<views::Widget> widget = CreateWidgetWithContent(contents);
+
+  auto primary = [&]() {
+    return LoginUserView::TestApi(
+        MakeLoginAuthTestApi(contents, AuthTarget::kPrimary).user_view());
+  };
+  auto secondary = [&]() {
+    return LoginUserView::TestApi(
+        MakeLoginAuthTestApi(contents, AuthTarget::kSecondary).user_view());
+  };
+
+  // Fires a return and validates that mock expectations have been satisfied.
+  auto submit = [&]() {
+    GetEventGenerator().PressKey(ui::VKEY_RETURN, 0);
+    controller->FlushForTesting();
+    testing::Mock::VerifyAndClearExpectations(client.get());
+  };
+  auto focus_and_submit = [&](views::View* view) {
+    view->RequestFocus();
+    DCHECK(view->HasFocus());
+    submit();
+  };
+
+  // The secondary user is not removable (as configured above) so showing the
+  // dropdown does not result in an interactive/focusable view.
+  focus_and_submit(secondary().dropdown());
+  EXPECT_TRUE(secondary().menu());
+  EXPECT_FALSE(HasFocusInAnyChildView(secondary().menu()->bubble_view()));
+  // TODO(jdufault): Run submit() and then EXPECT_FALSE(secondary().menu()); to
+  // verify that double-enter closes the bubble.
+
+  // The primary user is removable, so the menu is interactive. Submitting the
+  // first time shows the remove user warning, submitting the second time
+  // actually removes the user. Removing the user triggers a mojo API call as
+  // well as removes the user from the UI.
+  focus_and_submit(primary().dropdown());
+  EXPECT_TRUE(primary().menu());
+  EXPECT_TRUE(HasFocusInAnyChildView(primary().menu()->bubble_view()));
+  EXPECT_CALL(*client, OnRemoveUserWarningShown()).Times(1);
+  submit();
+  EXPECT_CALL(*client, RemoveUser(users()[0]->basic_user_info->account_id))
+      .Times(1);
+  submit();
+
+  // Secondary auth should be gone because it is now the primary auth.
+  EXPECT_FALSE(MakeLockContentsViewTestApi(contents).opt_secondary_big_view());
+  EXPECT_TRUE(MakeLockContentsViewTestApi(contents)
+                  .primary_big_view()
+                  ->GetCurrentUser()
+                  ->basic_user_info->account_id ==
+              users()[1]->basic_user_info->account_id);
+}
+
 }  // namespace ash
