@@ -4,18 +4,23 @@
 
 #include "ash/login/ui/login_bubble.h"
 
-#include "ash/ash_constants.h"
+#include <memory>
+#include <utility>
+
 #include "ash/focus_cycler.h"
 #include "ash/login/ui/layout_util.h"
 #include "ash/login/ui/lock_screen.h"
 #include "ash/login/ui/lock_window.h"
 #include "ash/login/ui/login_button.h"
+#include "ash/login/ui/login_menu_view.h"
 #include "ash/login/ui/non_accessible_view.h"
+#include "ash/public/cpp/ash_constants.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/shell.h"
 #include "ash/strings/grit/ash_strings.h"
 #include "base/strings/utf_string_conversions.h"
 #include "ui/accessibility/ax_node_data.h"
+#include "ui/aura/client/focus_client.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/compositor/layer_animator.h"
 #include "ui/compositor/scoped_layer_animation_settings.h"
@@ -78,6 +83,7 @@ views::Label* CreateLabel(const base::string16& message, SkColor color) {
   label->SetAutoColorReadabilityEnabled(false);
   label->SetHorizontalAlignment(gfx::ALIGN_LEFT);
   label->SetEnabledColor(color);
+  label->SetSubpixelRenderingEnabled(false);
   const gfx::FontList& base_font_list = views::Label::GetDefaultFontList();
   label->SetFontList(base_font_list.Derive(0, gfx::Font::FontStyle::NORMAL,
                                            gfx::Font::Weight::NORMAL));
@@ -86,7 +92,7 @@ views::Label* CreateLabel(const base::string16& message, SkColor color) {
 
 class LoginErrorBubbleView : public LoginBaseBubbleView {
  public:
-  LoginErrorBubbleView(views::StyledLabel* label, views::View* anchor_view)
+  LoginErrorBubbleView(views::View* content, views::View* anchor_view)
       : LoginBaseBubbleView(anchor_view) {
     SetLayoutManager(std::make_unique<views::BoxLayout>(
         views::BoxLayout::kVertical, gfx::Insets(),
@@ -104,8 +110,7 @@ class LoginErrorBubbleView : public LoginBaseBubbleView {
     alert_view->AddChildView(alert_icon);
     AddChildView(alert_view);
 
-    label->set_auto_color_readability_enabled(false);
-    AddChildView(label);
+    AddChildView(content);
   }
 
   ~LoginErrorBubbleView() override = default;
@@ -153,10 +158,12 @@ class LoginUserMenuView : public LoginBaseBubbleView,
                     bool is_owner,
                     views::View* anchor_view,
                     bool show_remove_user,
-                    base::OnceClosure do_remove_user)
+                    base::OnceClosure on_remove_user_warning_shown,
+                    base::OnceClosure on_remove_user_requested)
       : LoginBaseBubbleView(anchor_view),
         bubble_(bubble),
-        do_remove_user_(std::move(do_remove_user)) {
+        on_remove_user_warning_shown_(std::move(on_remove_user_warning_shown)),
+        on_remove_user_requested_(std::move(on_remove_user_requested)) {
     // This view has content the user can interact with if the remove user
     // button is displayed.
     set_can_activate(show_remove_user);
@@ -306,17 +313,23 @@ class LoginUserMenuView : public LoginBaseBubbleView,
       SizeToContents();
       GetWidget()->SetSize(size());
       Layout();
+      if (on_remove_user_warning_shown_)
+        std::move(on_remove_user_warning_shown_).Run();
       return;
     }
 
-    if (do_remove_user_)
-      std::move(do_remove_user_).Run();
+    // Close the bubble before calling |on_remove_user_requested_|. |bubble_| is
+    // an unowned reference; |on_remove_user_requested_| may delete it.
     bubble_->Close();
+
+    if (on_remove_user_requested_)
+      std::move(on_remove_user_requested_).Run();
   }
 
  private:
   LoginBubble* bubble_ = nullptr;
-  base::OnceClosure do_remove_user_;
+  base::OnceClosure on_remove_user_warning_shown_;
+  base::OnceClosure on_remove_user_requested_;
   views::View* remove_user_confirm_data_ = nullptr;
   views::Label* remove_user_label_ = nullptr;
   ButtonWithContent* remove_user_button_ = nullptr;
@@ -355,18 +368,19 @@ LoginBubble::LoginBubble() {
 
 LoginBubble::~LoginBubble() {
   Shell::Get()->RemovePreTargetHandler(this);
-  if (bubble_view_) {
-    bubble_view_->GetWidget()->RemoveObserver(this);
+  if (bubble_view_)
     CloseImmediately();
-  }
 }
 
-void LoginBubble::ShowErrorBubble(views::StyledLabel* label,
-                                  views::View* anchor_view) {
+void LoginBubble::ShowErrorBubble(views::View* content,
+                                  views::View* anchor_view,
+                                  uint32_t flags) {
   if (bubble_view_)
     CloseImmediately();
 
-  bubble_view_ = new LoginErrorBubbleView(label, anchor_view);
+  flags_ = flags;
+  bubble_view_ = new LoginErrorBubbleView(content, anchor_view);
+
   Show();
 }
 
@@ -377,20 +391,21 @@ void LoginBubble::ShowUserMenu(const base::string16& username,
                                views::View* anchor_view,
                                LoginButton* bubble_opener,
                                bool show_remove_user,
-                               base::OnceClosure do_remove_user) {
+                               base::OnceClosure on_remove_user_warning_shown,
+                               base::OnceClosure on_remove_user_requested) {
   if (bubble_view_)
     CloseImmediately();
 
+  flags_ = kFlagsNone;
   bubble_opener_ = bubble_opener;
-  bubble_view_ =
-      new LoginUserMenuView(this, username, email, type, is_owner, anchor_view,
-                            show_remove_user, std::move(do_remove_user));
+  bubble_view_ = new LoginUserMenuView(this, username, email, type, is_owner,
+                                       anchor_view, show_remove_user,
+                                       std::move(on_remove_user_warning_shown),
+                                       std::move(on_remove_user_requested));
   bool had_focus = bubble_opener_->HasFocus();
-
   Show();
-
   if (had_focus) {
-    // Try to focus the bubble view only if the tooltip was focused.
+    // Try to focus the bubble view only if the bubble opener was focused.
     bubble_view_->RequestFocus();
   }
 }
@@ -400,22 +415,46 @@ void LoginBubble::ShowTooltip(const base::string16& message,
   if (bubble_view_)
     CloseImmediately();
 
+  flags_ = kFlagsNone;
   bubble_view_ = new LoginTooltipView(message, anchor_view);
   Show();
 }
 
+void LoginBubble::ShowSelectionMenu(LoginMenuView* menu,
+                                    LoginButton* bubble_opener) {
+  if (bubble_view_)
+    CloseImmediately();
+
+  flags_ = kFlagsNone;
+  bubble_opener_ = bubble_opener;
+  const bool had_focus = bubble_opener_->HasFocus();
+
+  // Transfer the ownership of |menu| to bubble widget.
+  bubble_view_ = menu;
+  Show();
+
+  if (had_focus) {
+    // Try to focus the bubble view only if the bubble opener was focused.
+    bubble_view_->RequestFocus();
+  }
+}
+
 void LoginBubble::Close() {
   ScheduleAnimation(false /*visible*/);
 }
 
+void LoginBubble::CloseImmediately() {
+  DCHECK(bubble_view_);
+  Reset(false /*widget_already_closing*/);
+}
+
 bool LoginBubble::IsVisible() {
   return bubble_view_ && bubble_view_->GetWidget()->IsVisible();
 }
 
 void LoginBubble::OnWidgetClosing(views::Widget* widget) {
-  bubble_opener_ = nullptr;
-  bubble_view_ = nullptr;
-  widget->RemoveObserver(this);
+  DCHECK_EQ(bubble_view_->GetWidget(), widget);
+  Reset(true /*widget_already_closing*/);
 }
 
 void LoginBubble::OnWidgetDestroying(views::Widget* widget) {
@@ -447,7 +486,9 @@ void LoginBubble::OnKeyEvent(ui::KeyEvent* event) {
   if (bubble_view_->GetWidget()->IsActive())
     return;
 
-  Close();
+  if (!(flags_ & kFlagPersistent)) {
+    Close();
+  }
 }
 
 void LoginBubble::OnLayerAnimationEnded(ui::LayerAnimationSequence* sequence) {
@@ -456,14 +497,32 @@ void LoginBubble::OnLayerAnimationEnded(ui::LayerAnimationSequence* sequence) {
 
   bubble_view_->layer()->GetAnimator()->RemoveObserver(this);
   if (!is_visible_)
-    bubble_view_->GetWidget()->Close();
+    CloseImmediately();
+}
+
+void LoginBubble::OnWindowFocused(aura::Window* gained_focus,
+                                  aura::Window* lost_focus) {
+  if (!bubble_view_ || !IsVisible())
+    return;
+
+  aura::Window* bubble_window = bubble_view_->GetWidget()->GetNativeView();
+  // Bubble window has the focus, do nothing.
+  if (gained_focus && bubble_window->Contains(gained_focus))
+    return;
+
+  if (!(flags_ & kFlagPersistent))
+    Close();
 }
 
 void LoginBubble::Show() {
   DCHECK(bubble_view_);
-  views::BubbleDialogDelegateView::CreateBubble(bubble_view_)->Show();
+  views::Widget* widget =
+      views::BubbleDialogDelegateView::CreateBubble(bubble_view_);
+  widget->ShowInactive();
+  widget->AddObserver(this);
+  widget->StackAtTop();
+  aura::client::GetFocusClient(widget->GetNativeView())->AddObserver(this);
   bubble_view_->SetAlignment(views::BubbleBorder::ALIGN_EDGE_TO_ANCHOR_EDGE);
-  bubble_view_->GetWidget()->AddObserver(this);
 
   ScheduleAnimation(true /*visible*/);
 
@@ -471,13 +530,6 @@ void LoginBubble::Show() {
   bubble_view_->NotifyAccessibilityEvent(ax::mojom::Event::kAlert, true);
 }
 
-void LoginBubble::CloseImmediately() {
-  DCHECK(bubble_view_);
-  bubble_view_->layer()->GetAnimator()->RemoveObserver(this);
-  bubble_view_->GetWidget()->Close();
-  is_visible_ = false;
-}
-
 void LoginBubble::ProcessPressedEvent(const ui::LocatedEvent* event) {
   if (!bubble_view_)
     return;
@@ -498,7 +550,8 @@ void LoginBubble::ProcessPressedEvent(const ui::LocatedEvent* event) {
       return;
   }
 
-  Close();
+  if (!(flags_ & kFlagPersistent))
+    Close();
 }
 
 void LoginBubble::ScheduleAnimation(bool visible) {
@@ -534,4 +587,19 @@ void LoginBubble::ScheduleAnimation(bool visible) {
   }
 }
 
+void LoginBubble::Reset(bool widget_already_closing) {
+  DCHECK(bubble_view_);
+  aura::client::GetFocusClient(bubble_view_->GetWidget()->GetNativeView())
+      ->RemoveObserver(this);
+  bubble_view_->GetWidget()->RemoveObserver(this);
+  bubble_view_->layer()->GetAnimator()->RemoveObserver(this);
+
+  if (!widget_already_closing)
+    bubble_view_->GetWidget()->Close();
+  is_visible_ = false;
+  bubble_opener_ = nullptr;
+  bubble_view_ = nullptr;
+  flags_ = kFlagsNone;
+}
+
 }  // namespace ash
