@@ -6,27 +6,31 @@
 
 #include <memory>
 
-#include "ash/ash_constants.h"
 #include "ash/login/ui/animated_rounded_image_view.h"
 #include "ash/login/ui/hover_notifier.h"
 #include "ash/login/ui/image_parser.h"
+#include "ash/login/ui/layout_util.h"
 #include "ash/login/ui/login_bubble.h"
 #include "ash/login/ui/login_button.h"
 #include "ash/login/ui/non_accessible_view.h"
 #include "ash/login/ui/user_switch_flip_animation.h"
+#include "ash/public/cpp/ash_constants.h"
 #include "ash/public/cpp/login_constants.h"
+#include "ash/public/interfaces/user_info.mojom.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/strings/grit/ash_strings.h"
 #include "ash/system/user/rounded_image_view.h"
 #include "base/memory/weak_ptr.h"
 #include "base/strings/utf_string_conversions.h"
-#include "mojo/common/values_struct_traits.h"
+#include "components/user_manager/user_type.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/compositor/layer_animation_sequence.h"
 #include "ui/compositor/layer_animator.h"
 #include "ui/compositor/scoped_layer_animation_settings.h"
 #include "ui/gfx/paint_vector_icon.h"
+#include "ui/gfx/text_elider.h"
 #include "ui/views/controls/button/image_button.h"
+#include "ui/views/controls/image_view.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/layout/box_layout.h"
 #include "ui/views/layout/fill_layout.h"
@@ -40,8 +44,6 @@ namespace {
 constexpr int kVerticalSpacingBetweenEntriesDp = 32;
 // Horizontal spacing between username label and the dropdown icon.
 constexpr int kDistanceBetweenUsernameAndDropdownDp = 8;
-// Distance from the top of the user view to the user icon.
-constexpr int kDistanceFromTopOfBigUserViewToUserIconDp = 54;
 // Distance between user icon and the user label in small/extra-small layouts.
 constexpr int kSmallManyDistanceFromUserIconToUserLabelDp = 16;
 
@@ -63,9 +65,16 @@ constexpr float kOpaqueUserViewOpacity = 1.f;
 constexpr float kTransparentUserViewOpacity = 0.63f;
 constexpr float kUserFadeAnimationDurationMs = 180;
 
-constexpr const char kUserViewClassName[] = "UserView";
-constexpr const char kLoginUserImageClassName[] = "LoginUserImage";
-constexpr const char kLoginUserLabelClassName[] = "LoginUserLabel";
+constexpr char kUserViewClassName[] = "UserView";
+constexpr char kLoginUserImageClassName[] = "LoginUserImage";
+constexpr char kLoginUserLabelClassName[] = "LoginUserLabel";
+constexpr char kLoginUserDomainClassName[] = "LoginUserDomain";
+
+// Color of the user domain text.
+constexpr SkColor kDomainTextColor = SkColorSetARGB(0xAB, 0xFF, 0xFF, 0xFF);
+constexpr int kEnterpriseIconSizeDp = 12;
+constexpr int kBetweenEnterpriseIconAndDomainDp = 8;
+constexpr int kVerticalSpacingBetweenUserNameAndDomainDp = 14;
 
 int GetImageSize(LoginDisplayStyle style) {
   switch (style) {
@@ -105,13 +114,13 @@ class LoginUserView::UserImage : public NonAccessibleView {
     // Set the initial image from |avatar| since we already have it available.
     // Then, decode the bytes via blink's PNG decoder and play any animated
     // frames if they are available.
-    if (!user->basic_user_info->avatar.isNull())
-      image_->SetImage(user->basic_user_info->avatar);
+    if (!user->basic_user_info->avatar->image.isNull())
+      image_->SetImage(user->basic_user_info->avatar->image);
 
     // Decode the avatar using blink, as blink's PNG decoder supports APNG,
     // which is the format used for the animated avators.
-    if (!user->basic_user_info->avatar_bytes.empty()) {
-      DecodeAnimation(user->basic_user_info->avatar_bytes,
+    if (!user->basic_user_info->avatar->bytes.empty()) {
+      DecodeAnimation(user->basic_user_info->avatar->bytes,
                       base::Bind(&LoginUserView::UserImage::OnImageDecoded,
                                  weak_factory_.GetWeakPtr()));
     }
@@ -141,8 +150,8 @@ class LoginUserView::UserImage : public NonAccessibleView {
 // Shows the user's name.
 class LoginUserView::UserLabel : public NonAccessibleView {
  public:
-  UserLabel(LoginDisplayStyle style)
-      : NonAccessibleView(kLoginUserLabelClassName) {
+  UserLabel(LoginDisplayStyle style, int label_width)
+      : NonAccessibleView(kLoginUserLabelClassName), label_width_(label_width) {
     SetLayoutManager(std::make_unique<views::FillLayout>());
 
     user_name_ = new views::Label();
@@ -178,13 +187,17 @@ class LoginUserView::UserLabel : public NonAccessibleView {
     // display_name can be empty in debug builds with stub users.
     if (display_name.empty())
       display_name = user->basic_user_info->display_email;
-    user_name_->SetText(base::UTF8ToUTF16(display_name));
+
+    user_name_->SetText(gfx::ElideText(base::UTF8ToUTF16(display_name),
+                                       user_name_->font_list(), label_width_,
+                                       gfx::ElideBehavior::ELIDE_TAIL));
   }
 
   const base::string16& displayed_name() const { return user_name_->text(); }
 
  private:
   views::Label* user_name_ = nullptr;
+  const int label_width_;
 
   DISALLOW_COPY_AND_ASSIGN(UserLabel);
 };
@@ -215,6 +228,52 @@ class LoginUserView::TapButton : public views::Button {
   DISALLOW_COPY_AND_ASSIGN(TapButton);
 };
 
+class LoginUserView::UserDomainInfoView : public NonAccessibleView {
+ public:
+  UserDomainInfoView() : NonAccessibleView(kLoginUserDomainClassName) {
+    auto layout =
+        std::make_unique<views::BoxLayout>(views::BoxLayout::kHorizontal);
+    layout->set_main_axis_alignment(
+        views::BoxLayout::MAIN_AXIS_ALIGNMENT_CENTER);
+    SetLayoutManager(std::move(layout));
+
+    views::ImageView* image = new views::ImageView();
+    image->SetImage(
+        gfx::CreateVectorIcon(kLoginScreenEnterpriseIcon, kDomainTextColor));
+    image->SetPreferredSize(
+        gfx::Size(kEnterpriseIconSizeDp, kEnterpriseIconSizeDp));
+    AddChildView(image);
+
+    auto* spacer = new NonAccessibleView();
+    spacer->SetPreferredSize(gfx::Size(kBetweenEnterpriseIconAndDomainDp, 0));
+    AddChildView(spacer);
+
+    label_ = new views::Label();
+    label_->SetEnabledColor(kDomainTextColor);
+    label_->SetSubpixelRenderingEnabled(false);
+    label_->SetAutoColorReadabilityEnabled(false);
+    label_->SetFontList(views::Label::GetDefaultFontList().Derive(
+        0, gfx::Font::FontStyle::NORMAL, gfx::Font::Weight::NORMAL));
+    AddChildView(label_);
+  }
+
+  ~UserDomainInfoView() override = default;
+
+  // views::View:
+  gfx::Size CalculatePreferredSize() const override {
+    gfx::Size size = views::View::CalculatePreferredSize();
+    size.set_width(kLargeUserViewWidthDp);
+    return size;
+  }
+
+  void SetText(const base::string16& text) { label_->SetText(text); }
+
+ private:
+  views::Label* label_ = nullptr;
+
+  DISALLOW_COPY_AND_ASSIGN(UserDomainInfoView);
+};
+
 // LoginUserView is defined after LoginUserView::UserLabel so it can access the
 // class members.
 
@@ -238,6 +297,14 @@ views::View* LoginUserView::TestApi::tap_button() const {
   return view_->tap_button_;
 }
 
+views::View* LoginUserView::TestApi::dropdown() const {
+  return view_->user_dropdown_;
+}
+
+LoginBubble* LoginUserView::TestApi::menu() const {
+  return view_->user_menu_.get();
+}
+
 bool LoginUserView::TestApi::is_opaque() const {
   return view_->is_opaque_;
 }
@@ -257,16 +324,31 @@ int LoginUserView::WidthForLayoutStyle(LoginDisplayStyle style) {
   return 0;
 }
 
-LoginUserView::LoginUserView(LoginDisplayStyle style,
-                             bool show_dropdown,
-                             const OnTap& on_tap)
-    : on_tap_(on_tap), display_style_(style) {
+LoginUserView::LoginUserView(
+    LoginDisplayStyle style,
+    bool show_dropdown,
+    bool show_domain,
+    const OnTap& on_tap,
+    const OnRemoveWarningShown& on_remove_warning_shown,
+    const OnRemove& on_remove)
+    : on_tap_(on_tap),
+      on_remove_warning_shown_(on_remove_warning_shown),
+      on_remove_(on_remove),
+      display_style_(style) {
   // show_dropdown can only be true when the user view is rendering in large
   // mode.
   DCHECK(!show_dropdown || style == LoginDisplayStyle::kLarge);
+  DCHECK(!show_domain || style == LoginDisplayStyle::kLarge);
+  // |on_remove_warning_shown| and |on_remove| is only available iff
+  // |show_dropdown| is true.
+  DCHECK(show_dropdown == !!on_remove_warning_shown);
+  DCHECK(show_dropdown == !!on_remove);
 
   user_image_ = new UserImage(GetImageSize(style));
-  user_label_ = new UserLabel(style);
+  int label_width =
+      WidthForLayoutStyle(style) -
+      2 * (kDistanceBetweenUsernameAndDropdownDp + kDropdownIconSizeDp);
+  user_label_ = new UserLabel(style, label_width);
   if (show_dropdown) {
     user_dropdown_ = new LoginButton(this);
     user_dropdown_->set_has_ink_drop_action_on_click(false);
@@ -277,6 +359,8 @@ LoginUserView::LoginUserView(LoginDisplayStyle style,
         gfx::CreateVectorIcon(kLockScreenDropdownIcon, SK_ColorWHITE));
     user_dropdown_->SetFocusBehavior(FocusBehavior::ALWAYS);
   }
+  if (show_domain)
+    user_domain_ = new UserDomainInfoView();
   tap_button_ = new TapButton(this);
   SetTapEnabled(true);
 
@@ -305,6 +389,9 @@ LoginUserView::LoginUserView(LoginDisplayStyle style,
   if (user_dropdown_)
     setup_layer(user_dropdown_);
 
+  if (user_domain_)
+    setup_layer(user_domain_);
+
   hover_notifier_ = std::make_unique<HoverNotifier>(
       this, base::Bind(&LoginUserView::OnHover, base::Unretained(this)));
   user_menu_ = std::make_unique<LoginBubble>();
@@ -360,6 +447,10 @@ void LoginUserView::UpdateForUser(const mojom::LoginUserInfoPtr& user,
       user_dropdown_->layer()->GetAnimator()->StartAnimation(
           make_opacity_sequence());
     }
+    if (user_domain_) {
+      user_domain_->layer()->GetAnimator()->StartAnimation(
+          make_opacity_sequence());
+    }
   } else {
     // Do not animate, so directly update to the current user.
     UpdateCurrentUserState();
@@ -399,6 +490,10 @@ void LoginUserView::Layout() {
   tap_button_->SetBoundsRect(GetLocalBounds());
 }
 
+void LoginUserView::RequestFocus() {
+  tap_button_->RequestFocus();
+}
+
 void LoginUserView::ButtonPressed(views::Button* sender,
                                   const ui::Event& event) {
   // Handle click on the dropdown arrow.
@@ -411,7 +506,7 @@ void LoginUserView::ButtonPressed(views::Button* sender,
           current_user_->basic_user_info->type, current_user_->is_device_owner,
           user_dropdown_ /*anchor_view*/, user_dropdown_ /*bubble_opener*/,
           current_user_->can_remove /*show_remove_user*/,
-          base::OnceClosure() /*do_remove_user*/);
+          on_remove_warning_shown_, on_remove_);
     } else {
       user_menu_->Close();
     }
@@ -435,6 +530,17 @@ void LoginUserView::UpdateCurrentUserState() {
         IDS_ASH_LOGIN_POD_MENU_BUTTON_ACCESSIBLE_NAME, email));
   }
 
+  if (user_domain_) {
+    DCHECK(current_user_->public_account_info);
+    const base::Optional<std::string>& enterprise_domain =
+        current_user_->public_account_info->enterprise_domain;
+    if (enterprise_domain) {
+      user_domain_->SetText(l10n_util::GetStringFUTF16(
+          IDS_ASH_LOGIN_PUBLIC_ACCOUNT_INFO_FORMAT,
+          base::UTF8ToUTF16(enterprise_domain.value())));
+    }
+  }
+
   user_image_->UpdateForUser(current_user_);
   user_label_->UpdateForUser(current_user_);
   Layout();
@@ -448,7 +554,8 @@ void LoginUserView::UpdateOpacity() {
     return;
 
   // Animate to new opacity.
-  auto build_settings = [](views::View* view) {
+  auto build_settings = [](views::View* view)
+      -> std::unique_ptr<ui::ScopedLayerAnimationSettings> {
     auto settings = std::make_unique<ui::ScopedLayerAnimationSettings>(
         view->layer()->GetAnimator());
     settings->SetTransitionDuration(
@@ -456,17 +563,26 @@ void LoginUserView::UpdateOpacity() {
     settings->SetTweenType(gfx::Tween::Type::EASE_IN_OUT);
     return settings;
   };
-  auto user_image_settings = build_settings(user_image_);
-  auto user_label_settings = build_settings(user_label_);
+  std::unique_ptr<ui::ScopedLayerAnimationSettings> user_image_settings =
+      build_settings(user_image_);
+  std::unique_ptr<ui::ScopedLayerAnimationSettings> user_label_settings =
+      build_settings(user_label_);
   float target_opacity =
       is_opaque_ ? kOpaqueUserViewOpacity : kTransparentUserViewOpacity;
   user_image_->layer()->SetOpacity(target_opacity);
   user_label_->layer()->SetOpacity(target_opacity);
   if (user_dropdown_) {
-    auto user_dropdown_settings = build_settings(user_dropdown_);
+    std::unique_ptr<ui::ScopedLayerAnimationSettings> user_dropdown_settings =
+        build_settings(user_dropdown_);
     user_dropdown_->layer()->SetOpacity(target_opacity);
   }
 
+  if (user_domain_) {
+    std::unique_ptr<ui::ScopedLayerAnimationSettings> user_domain_settings =
+        build_settings(user_domain_);
+    user_domain_->layer()->SetOpacity(target_opacity);
+  }
+
   // Animate avatar only if we are opaque.
   user_image_->SetAnimationEnabled(is_opaque_);
 }
@@ -478,6 +594,8 @@ void LoginUserView::SetLargeLayout() {
   AddChildView(tap_button_);
   if (user_dropdown_)
     AddChildView(user_dropdown_);
+  if (user_domain_)
+    AddChildView(user_domain_);
 
   // Use views::GridLayout instead of views::BoxLayout because views::BoxLayout
   // lays out children according to the view->children order.
@@ -486,6 +604,8 @@ void LoginUserView::SetLargeLayout() {
 
   constexpr int kImageColumnId = 0;
   constexpr int kLabelDropdownColumnId = 1;
+  constexpr int kLabelDomainColumnId = 2;
+
   {
     views::ColumnSet* image = layout->AddColumnSet(kImageColumnId);
     image->AddColumn(views::GridLayout::CENTER, views::GridLayout::CENTER,
@@ -517,13 +637,19 @@ void LoginUserView::SetLargeLayout() {
     label_dropdown->AddPaddingColumn(1.0f /*resize_percent*/, 0 /*width*/);
   }
 
+  {
+    views::ColumnSet* label_domain = layout->AddColumnSet(kLabelDomainColumnId);
+    label_domain->AddColumn(views::GridLayout::CENTER,
+                            views::GridLayout::CENTER, 1 /*resize_percent*/,
+                            views::GridLayout::USE_PREF, 0 /*fixed_width*/,
+                            0 /*min_width*/);
+  }
+
   auto add_padding = [&](int amount) {
     layout->AddPaddingRow(0 /*vertical_resize*/, amount /*size*/);
   };
 
   // Add views in rendering order.
-  add_padding(kDistanceFromTopOfBigUserViewToUserIconDp);
-
   // Image
   layout->StartRow(0 /*vertical_resize*/, kImageColumnId);
   layout->AddView(user_image_);
@@ -535,6 +661,12 @@ void LoginUserView::SetLargeLayout() {
   layout->AddView(user_label_);
   if (user_dropdown_)
     layout->AddView(user_dropdown_);
+
+  if (user_domain_) {
+    add_padding(kVerticalSpacingBetweenUserNameAndDomainDp);
+    layout->StartRow(0 /*vertical_resize*/, kLabelDomainColumnId);
+    layout->AddView(user_domain_);
+  }
 }
 
 void LoginUserView::SetSmallishLayout() {
