@@ -12,9 +12,6 @@ enum class Config {
   // Classic mode does not use mus.
   CLASSIC,
 
-  // Aura is backed by mus, but chrome and ash are still in the same process.
-  MUS,
-
   // Aura is backed by mus and chrome and ash are in separate processes. In this
   // mode chrome code can only use ash code in ash/public/cpp.
   MASH,
