@@ -57,6 +57,18 @@ enum ShellWindowId {
   // The container for the app list.
   kShellWindowId_AppListContainer,
 
+  // The container for the app list in tablet mode.
+  kShellWindowId_AppListTabletModeContainer,
+
+  // The parent container that holds the ARC IME container and windows created
+  // by ARC IME other than the virtual keyboard window.
+  // This container window is to ensure that the ARC IME window is stacked above
+  // top-level windows and the app list window but below the shelf.
+  kShellWindowId_ArcImeWindowParentContainer,
+
+  // The container for Virtual Keyboard from ARC IMEs.
+  kShellWindowId_ArcVirtualKeyboardContainer,
+
   // The container for the shelf.
   kShellWindowId_ShelfContainer,
 
@@ -104,12 +116,17 @@ enum ShellWindowId {
   // The container for drag/drop images and tooltips.
   kShellWindowId_DragImageAndTooltipContainer,
 
+  // The container for the fullscreen power button menu.
+  kShellWindowId_PowerMenuContainer,
+
   // The container for bubbles briefly overlaid onscreen to show settings
   // changes (volume, brightness, input method bubbles, etc.).
   kShellWindowId_SettingBubbleContainer,
 
-  // Contains the ChromeVox spoken feedback window.
-  kShellWindowId_ChromeVoxContainer,
+  // Contains special accessibility windows that can inset the display work area
+  // (e.g. the ChromeVox spoken feedback window).
+  // TODO(jamescook): Consolidate this with DockedMagnifierContainer.
+  kShellWindowId_AccessibilityPanelContainer,
 
   // The container for special components overlaid onscreen, such as the
   // region selector for partial screenshots.
@@ -147,6 +164,9 @@ const int32_t kAllShellContainerIds[] = {
     kShellWindowId_DefaultContainer,
     kShellWindowId_AlwaysOnTopContainer,
     kShellWindowId_AppListContainer,
+    kShellWindowId_AppListTabletModeContainer,
+    kShellWindowId_ArcImeWindowParentContainer,
+    kShellWindowId_ArcVirtualKeyboardContainer,
     kShellWindowId_ShelfContainer,
     kShellWindowId_ShelfBubbleContainer,
     kShellWindowId_PanelContainer,
@@ -159,8 +179,9 @@ const int32_t kAllShellContainerIds[] = {
     kShellWindowId_ImeWindowParentContainer,
     kShellWindowId_MenuContainer,
     kShellWindowId_DragImageAndTooltipContainer,
+    kShellWindowId_PowerMenuContainer,
     kShellWindowId_SettingBubbleContainer,
-    kShellWindowId_ChromeVoxContainer,
+    kShellWindowId_AccessibilityPanelContainer,
     kShellWindowId_OverlayContainer,
     kShellWindowId_DockedMagnifierContainer,
     kShellWindowId_MouseCursorContainer,
