@@ -9,15 +9,17 @@
 #include <queue>
 #include <vector>
 
-#include "ash/accessibility/chromevox_layout_manager.h"
+#include "ash/accessibility/accessibility_panel_layout_manager.h"
 #include "ash/accessibility/touch_exploration_controller.h"
-#include "ash/ash_constants.h"
-#include "ash/ash_touch_exploration_manager_chromeos.h"
+#include "ash/accessibility/touch_exploration_manager.h"
 #include "ash/focus_cycler.h"
 #include "ash/high_contrast/high_contrast_controller.h"
 #include "ash/host/ash_window_tree_host.h"
+#include "ash/keyboard/virtual_keyboard_container_layout_manager.h"
 #include "ash/lock_screen_action/lock_screen_action_background_controller.h"
 #include "ash/login_status.h"
+#include "ash/public/cpp/ash_constants.h"
+#include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/ash_switches.h"
 #include "ash/public/cpp/config.h"
 #include "ash/public/cpp/shelf_types.h"
@@ -32,12 +34,12 @@
 #include "ash/shelf/shelf_window_targeter.h"
 #include "ash/shell.h"
 #include "ash/shell_port.h"
+#include "ash/shell_state.h"
 #include "ash/system/status_area_layout_manager.h"
 #include "ash/system/status_area_widget.h"
 #include "ash/touch/touch_hud_debug.h"
 #include "ash/touch/touch_hud_projection.h"
 #include "ash/touch/touch_observer_hud.h"
-#include "ash/virtual_keyboard_container_layout_manager.h"
 #include "ash/wallpaper/wallpaper_widget_controller.h"
 #include "ash/wm/always_on_top_controller.h"
 #include "ash/wm/container_finder.h"
@@ -73,10 +75,12 @@
 #include "ui/aura/window_observer.h"
 #include "ui/aura/window_tracker.h"
 #include "ui/base/models/menu_model.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/compositor/layer.h"
 #include "ui/display/types/display_constants.h"
 #include "ui/events/event_utils.h"
 #include "ui/keyboard/keyboard_controller.h"
+#include "ui/keyboard/keyboard_layout_manager.h"
 #include "ui/keyboard/keyboard_util.h"
 #include "ui/views/controls/menu/menu_model_adapter.h"
 #include "ui/views/controls/menu/menu_runner.h"
@@ -85,6 +89,7 @@
 #include "ui/wm/core/capture_controller.h"
 #include "ui/wm/core/coordinate_conversion.h"
 #include "ui/wm/core/visibility_controller.h"
+#include "ui/wm/core/window_properties.h"
 #include "ui/wm/core/window_util.h"
 #include "ui/wm/public/tooltip_client.h"
 
@@ -471,19 +476,19 @@ void RootWindowController::Shutdown() {
 }
 
 void RootWindowController::CloseChildWindows() {
-  // NOTE: this may be called multiple times.
-  // TODO: Only call this once. The multiple calls cause complexity in the
-  // dependent Shelf and ShelfWidget code.
+  // Child windows can be closed by secondary monitor disconnection, Shell
+  // shutdown, or both. Avoid running the related cleanup code twice.
+  if (did_close_child_windows_)
+    return;
+  did_close_child_windows_ = true;
 
   // Deactivate keyboard container before closing child windows and shutting
   // down associated layout managers.
-  DeactivateKeyboard(keyboard::KeyboardController::GetInstance());
+  DeactivateKeyboard(keyboard::KeyboardController::Get());
 
   // |panel_layout_manager_| needs to be shut down before windows are destroyed.
-  if (panel_layout_manager_) {
-    panel_layout_manager_->Shutdown();
-    panel_layout_manager_ = nullptr;
-  }
+  panel_layout_manager_->Shutdown();
+  panel_layout_manager_ = nullptr;
 
   shelf_->ShutdownShelfWidget();
 
@@ -522,7 +527,6 @@ void RootWindowController::CloseChildWindows() {
   // ShelfLayoutManager because ShelfLayoutManager holds a pointer to it.
   shelf_->DestroyShelfWidget();
 
-  aura::client::SetDragDropClient(GetRootWindow(), nullptr);
   ::wm::SetTooltipClient(GetRootWindow(), nullptr);
 }
 
@@ -537,17 +541,16 @@ void RootWindowController::UpdateShelfVisibility() {
 }
 
 void RootWindowController::InitTouchHuds() {
-  if (Shell::GetAshConfig() == Config::MASH)
-    return;
-
   // Enable touch debugging features when each display is initialized.
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   if (command_line->HasSwitch(switches::kAshTouchHud))
     set_touch_hud_debug(new TouchHudDebug(GetRootWindow()));
 
   // TouchHudProjection manages its own lifetime.
-  if (command_line->HasSwitch(switches::kShowTaps))
+  if (command_line->HasSwitch(switches::kShowTaps) &&
+      !base::FeatureList::IsEnabled(features::kTapVisualizerApp)) {
     touch_hud_projection_ = new TouchHudProjection(GetRootWindow());
+  }
 }
 
 aura::Window* RootWindowController::GetWindowForFullscreenMode() {
@@ -556,50 +559,32 @@ aura::Window* RootWindowController::GetWindowForFullscreenMode() {
 
 void RootWindowController::ActivateKeyboard(
     keyboard::KeyboardController* keyboard_controller) {
-  if (!keyboard::IsKeyboardEnabled() || !keyboard_controller)
+  DCHECK(keyboard_controller);
+  if (!keyboard::IsKeyboardEnabled() || !keyboard_controller->enabled())
     return;
 
-  // keyboard window is already initialized and it's under the root window of
-  // this controller.
-  if (keyboard_controller->keyboard_container_initialized() &&
-      keyboard_controller->GetContainerWindow()->GetRootWindow() ==
-          GetRootWindow()) {
+  // If the keyboard is already activated, ensure that it is activated in this
+  // root window.
+  if (keyboard_controller->GetRootWindow() == GetRootWindow())
     return;
-  }
 
-  aura::Window* keyboard_window = keyboard_controller->GetContainerWindow();
-  DCHECK(keyboard_window->parent() == nullptr);
-
-  Shell::Get()->NotifyVirtualKeyboardActivated(true, GetRootWindow());
   aura::Window* vk_container =
       GetContainer(kShellWindowId_VirtualKeyboardContainer);
   DCHECK(vk_container);
-  vk_container->AddChild(keyboard_window);
+  keyboard_controller->ActivateKeyboardInContainer(vk_container);
 
   keyboard_controller->LoadKeyboardUiInBackground();
 }
 
 void RootWindowController::DeactivateKeyboard(
     keyboard::KeyboardController* keyboard_controller) {
-  if (!keyboard_controller ||
-      !keyboard_controller->keyboard_container_initialized()) {
+  DCHECK(keyboard_controller);
+  if (!keyboard_controller->enabled())
     return;
-  }
 
-  aura::Window* keyboard_window = keyboard_controller->GetContainerWindow();
   // If the VK is under the root window of this controller.
-  if (keyboard_window->GetRootWindow() == GetRootWindow()) {
-    // Virtual keyboard may be deactivated while still showing, hide the
-    // keyboard before removing it from view hierarchy.
-    keyboard_controller->HideKeyboard(
-        keyboard::KeyboardController::HIDE_REASON_AUTOMATIC);
-    aura::Window* vk_container =
-        GetContainer(kShellWindowId_VirtualKeyboardContainer);
-    DCHECK(vk_container);
-    DCHECK_EQ(vk_container, keyboard_window->parent());
-    vk_container->RemoveChild(keyboard_window);
-    Shell::Get()->NotifyVirtualKeyboardActivated(false, GetRootWindow());
-  }
+  if (keyboard_controller->GetRootWindow() == GetRootWindow())
+    keyboard_controller->DeactivateKeyboard();
 }
 
 void RootWindowController::SetTouchAccessibilityAnchorPoint(
@@ -625,13 +610,20 @@ void RootWindowController::ShowContextMenu(const gfx::Point& location_in_screen,
   UMA_HISTOGRAM_ENUMERATION("Apps.ContextMenuShowSource.Desktop", source_type,
                             ui::MENU_SOURCE_TYPE_LAST);
 
+  int run_types = views::MenuRunner::CONTEXT_MENU;
+  views::MenuAnchorPosition anchor_position = views::MENU_ANCHOR_TOPLEFT;
+  if (::features::IsTouchableAppContextMenuEnabled()) {
+    run_types |= views::MenuRunner::USE_TOUCHABLE_LAYOUT |
+                 views::MenuRunner::FIXED_ANCHOR;
+    anchor_position = views::MENU_ANCHOR_BUBBLE_TOUCHABLE_ABOVE;
+  }
   menu_runner_ = std::make_unique<views::MenuRunner>(
-      menu_model_.get(), views::MenuRunner::CONTEXT_MENU,
+      menu_model_.get(), run_types,
       base::Bind(&RootWindowController::OnMenuClosed, base::Unretained(this),
                  base::TimeTicks::Now()));
   menu_runner_->RunMenuAt(wallpaper_widget_controller()->GetWidget(), nullptr,
                           gfx::Rect(location_in_screen, gfx::Size()),
-                          views::MENU_ANCHOR_TOPLEFT, source_type);
+                          anchor_position, source_type);
 }
 
 void RootWindowController::HideContextMenu() {
@@ -639,6 +631,10 @@ void RootWindowController::HideContextMenu() {
     menu_runner_->Cancel();
 }
 
+bool RootWindowController::IsContextMenuShown() const {
+  return menu_runner_ && menu_runner_->IsRunning();
+}
+
 void RootWindowController::UpdateAfterLoginStatusChange(LoginStatus status) {
   StatusAreaWidget* status_area_widget =
       shelf_->shelf_widget()->status_area_widget();
@@ -702,7 +698,7 @@ void RootWindowController::Init(RootWindowType root_window_type) {
   root_window_layout_manager_->OnWindowResized();
   if (root_window_type == RootWindowType::PRIMARY) {
     if (Shell::GetAshConfig() != Config::MASH)
-      shell->CreateKeyboard();
+      shell->EnableKeyboard();
   } else {
     window_tree_host_->Show();
 
@@ -710,17 +706,19 @@ void RootWindowController::Init(RootWindowType root_window_type) {
     shell->OnRootWindowAdded(root_window);
   }
 
-  // TODO: AshTouchExplorationManager doesn't work with mus.
+  // TODO: TouchExplorationManager doesn't work with mash.
   // http://crbug.com/679782
   if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
           switches::kAshDisableTouchExplorationMode) &&
       Shell::GetAshConfig() != Config::MASH) {
-    touch_exploration_manager_.reset(new AshTouchExplorationManager(this));
+    touch_exploration_manager_ =
+        std::make_unique<TouchExplorationManager>(this);
   }
 }
 
 void RootWindowController::InitLayoutManagers() {
-  // Create the shelf and status area widgets.
+  // Create the shelf and status area widgets. Creates the ShelfLayoutManager
+  // as a side-effect.
   DCHECK(!shelf_->shelf_widget());
   aura::Window* root = GetRootWindow();
   shelf_->CreateShelfWidget(root);
@@ -842,6 +840,13 @@ void RootWindowController::CreateContainers() {
       kShellWindowId_LockScreenRelatedContainersContainer,
       "LockScreenRelatedContainersContainer", screen_rotation_container);
 
+  aura::Window* app_list_tablet_mode_container =
+      CreateContainer(kShellWindowId_AppListTabletModeContainer,
+                      "AppListTabletModeContainer", non_lock_screen_containers);
+  wm::SetSnapsChildrenToPhysicalPixelBoundary(app_list_tablet_mode_container);
+  app_list_tablet_mode_container->SetProperty(::wm::kUsesScreenCoordinatesKey,
+                                              true);
+
   CreateContainer(kShellWindowId_UnparentedControlContainer,
                   "UnparentedControlContainer", non_lock_screen_containers);
 
@@ -850,7 +855,7 @@ void RootWindowController::CreateContainers() {
                       non_lock_screen_containers);
   ::wm::SetChildWindowVisibilityChangesAnimated(default_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(default_container);
-  default_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  default_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   wm::SetChildrenUseExtendedHitRegionForWindow(default_container);
 
   aura::Window* always_on_top_container =
@@ -858,32 +863,43 @@ void RootWindowController::CreateContainers() {
                       "AlwaysOnTopContainer", non_lock_screen_containers);
   ::wm::SetChildWindowVisibilityChangesAnimated(always_on_top_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(always_on_top_container);
-  always_on_top_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  always_on_top_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* app_list_container =
       CreateContainer(kShellWindowId_AppListContainer, "AppListContainer",
                       non_lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(app_list_container);
-  app_list_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  app_list_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
+
+  aura::Window* arc_ime_parent_container = CreateContainer(
+      kShellWindowId_ArcImeWindowParentContainer, "ArcImeWindowParentContainer",
+      non_lock_screen_containers);
+  wm::SetSnapsChildrenToPhysicalPixelBoundary(arc_ime_parent_container);
+  arc_ime_parent_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
+  aura::Window* arc_vk_container =
+      CreateContainer(kShellWindowId_ArcVirtualKeyboardContainer,
+                      "ArcVirtualKeyboardContainer", arc_ime_parent_container);
+  wm::SetSnapsChildrenToPhysicalPixelBoundary(arc_vk_container);
+  arc_vk_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* shelf_container_parent = lock_screen_related_containers;
   aura::Window* shelf_container = CreateContainer(
       kShellWindowId_ShelfContainer, "ShelfContainer", shelf_container_parent);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(shelf_container);
-  shelf_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  shelf_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   shelf_container->SetProperty(kLockedToRootKey, true);
 
   aura::Window* panel_container =
       CreateContainer(kShellWindowId_PanelContainer, "PanelContainer",
                       non_lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(panel_container);
-  panel_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  panel_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* shelf_bubble_container =
       CreateContainer(kShellWindowId_ShelfBubbleContainer,
                       "ShelfBubbleContainer", non_lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(shelf_bubble_container);
-  shelf_bubble_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  shelf_bubble_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   shelf_bubble_container->SetProperty(kLockedToRootKey, true);
 
   aura::Window* modal_container =
@@ -891,60 +907,69 @@ void RootWindowController::CreateContainers() {
                       "SystemModalContainer", non_lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(modal_container);
   ::wm::SetChildWindowVisibilityChangesAnimated(modal_container);
-  modal_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  modal_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   wm::SetChildrenUseExtendedHitRegionForWindow(modal_container);
 
   aura::Window* lock_container =
       CreateContainer(kShellWindowId_LockScreenContainer, "LockScreenContainer",
                       lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(lock_container);
-  lock_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  lock_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* lock_action_handler_container =
       CreateContainer(kShellWindowId_LockActionHandlerContainer,
                       "LockActionHandlerContainer", lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(lock_action_handler_container);
   ::wm::SetChildWindowVisibilityChangesAnimated(lock_action_handler_container);
-  lock_action_handler_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  lock_action_handler_container->SetProperty(::wm::kUsesScreenCoordinatesKey,
+                                             true);
 
   aura::Window* lock_modal_container =
       CreateContainer(kShellWindowId_LockSystemModalContainer,
                       "LockSystemModalContainer", lock_screen_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(lock_modal_container);
   ::wm::SetChildWindowVisibilityChangesAnimated(lock_modal_container);
-  lock_modal_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  lock_modal_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   wm::SetChildrenUseExtendedHitRegionForWindow(lock_modal_container);
 
   aura::Window* status_container =
       CreateContainer(kShellWindowId_StatusContainer, "StatusContainer",
                       lock_screen_related_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(status_container);
-  status_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  status_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   status_container->SetProperty(kLockedToRootKey, true);
 
+  aura::Window* power_menu_container =
+      CreateContainer(kShellWindowId_PowerMenuContainer, "PowerMenuContainer",
+                      lock_screen_related_containers);
+  wm::SetSnapsChildrenToPhysicalPixelBoundary(power_menu_container);
+  power_menu_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
+
   aura::Window* settings_bubble_container =
       CreateContainer(kShellWindowId_SettingBubbleContainer,
                       "SettingBubbleContainer", lock_screen_related_containers);
   ::wm::SetChildWindowVisibilityChangesAnimated(settings_bubble_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(settings_bubble_container);
-  settings_bubble_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  settings_bubble_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
   settings_bubble_container->SetProperty(kLockedToRootKey, true);
 
-  aura::Window* chromevox_container =
-      CreateContainer(kShellWindowId_ChromeVoxContainer, "ChromeVoxContainer",
-                      lock_screen_related_containers);
-  ::wm::SetChildWindowVisibilityChangesAnimated(chromevox_container);
-  chromevox_container->SetProperty(kUsesScreenCoordinatesKey, true);
-  chromevox_container->SetProperty(kLockedToRootKey, true);
-  chromevox_container->SetLayoutManager(new ChromeVoxLayoutManager());
+  aura::Window* accessibility_panel_container = CreateContainer(
+      kShellWindowId_AccessibilityPanelContainer, "AccessibilityPanelContainer",
+      lock_screen_related_containers);
+  ::wm::SetChildWindowVisibilityChangesAnimated(accessibility_panel_container);
+  accessibility_panel_container->SetProperty(::wm::kUsesScreenCoordinatesKey,
+                                             true);
+  accessibility_panel_container->SetProperty(kLockedToRootKey, true);
+  accessibility_panel_container->SetLayoutManager(
+      new AccessibilityPanelLayoutManager());
 
   aura::Window* virtual_keyboard_parent_container = CreateContainer(
       kShellWindowId_ImeWindowParentContainer, "VirtualKeyboardParentContainer",
       lock_screen_related_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(
       virtual_keyboard_parent_container);
-  virtual_keyboard_parent_container->SetProperty(kUsesScreenCoordinatesKey,
-                                                 true);
+  virtual_keyboard_parent_container->SetProperty(
+      ::wm::kUsesScreenCoordinatesKey, true);
   virtual_keyboard_parent_container->SetLayoutManager(
       new VirtualKeyboardContainerLayoutManager(
           virtual_keyboard_parent_container));
@@ -952,27 +977,30 @@ void RootWindowController::CreateContainers() {
       kShellWindowId_VirtualKeyboardContainer, "VirtualKeyboardContainer",
       virtual_keyboard_parent_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(virtual_keyboard_container);
-  virtual_keyboard_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  virtual_keyboard_container->SetProperty(::wm::kUsesScreenCoordinatesKey,
+                                          true);
+  virtual_keyboard_container->SetLayoutManager(
+      new keyboard::KeyboardLayoutManager(keyboard::KeyboardController::Get()));
 
   aura::Window* menu_container =
       CreateContainer(kShellWindowId_MenuContainer, "MenuContainer",
                       lock_screen_related_containers);
   ::wm::SetChildWindowVisibilityChangesAnimated(menu_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(menu_container);
-  menu_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  menu_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* drag_drop_container = CreateContainer(
       kShellWindowId_DragImageAndTooltipContainer,
       "DragImageAndTooltipContainer", lock_screen_related_containers);
   ::wm::SetChildWindowVisibilityChangesAnimated(drag_drop_container);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(drag_drop_container);
-  drag_drop_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  drag_drop_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   aura::Window* overlay_container =
       CreateContainer(kShellWindowId_OverlayContainer, "OverlayContainer",
                       lock_screen_related_containers);
   wm::SetSnapsChildrenToPhysicalPixelBoundary(overlay_container);
-  overlay_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  overlay_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   CreateContainer(kShellWindowId_DockedMagnifierContainer,
                   "DockedMagnifierContainer", lock_screen_related_containers);
@@ -980,7 +1008,7 @@ void RootWindowController::CreateContainers() {
   aura::Window* mouse_cursor_container =
       CreateContainer(kShellWindowId_MouseCursorContainer,
                       "MouseCursorContainer", screen_rotation_container);
-  mouse_cursor_container->SetProperty(kUsesScreenCoordinatesKey, true);
+  mouse_cursor_container->SetProperty(::wm::kUsesScreenCoordinatesKey, true);
 
   CreateContainer(kShellWindowId_PowerButtonAnimationContainer,
                   "PowerButtonAnimationContainer", screen_rotation_container);
@@ -1012,7 +1040,7 @@ void RootWindowController::ResetRootForNewWindowsIfNecessary() {
     // The root window for new windows is being destroyed. Switch to the primary
     // root window if possible.
     aura::Window* primary_root = Shell::GetPrimaryRootWindow();
-    Shell::Get()->set_root_window_for_new_windows(
+    Shell::Get()->shell_state()->SetRootWindowForNewWindows(
         primary_root == root ? nullptr : primary_root);
   }
 }
