@@ -35,9 +35,9 @@
 #include "ui/events/test/test_event_handler.h"
 #include "ui/keyboard/keyboard_controller.h"
 #include "ui/keyboard/keyboard_switches.h"
-#include "ui/keyboard/keyboard_test_util.h"
 #include "ui/keyboard/keyboard_ui.h"
 #include "ui/keyboard/keyboard_util.h"
+#include "ui/keyboard/test/keyboard_test_util.h"
 #include "ui/views/controls/menu/menu_controller.h"
 #include "ui/views/widget/widget.h"
 #include "ui/views/widget/widget_delegate.h"
@@ -103,6 +103,11 @@ class RootWindowControllerTest : public AshTestBase {
   views::Widget* CreateTestWidget(const gfx::Rect& bounds) {
     views::Widget* widget = views::Widget::CreateWindowWithContextAndBounds(
         NULL, CurrentContext(), bounds);
+    // The initial bounds will be constrained to the screen work area or the
+    // parent. See Widget::InitialBounds() & Widget::SetBoundsConstrained().
+    // Explicitly setting the bounds here will allow the view to be positioned
+    // such that it can extend outside the screen work area.
+    widget->SetBounds(bounds);
     widget->Show();
     return widget;
   }
@@ -110,6 +115,8 @@ class RootWindowControllerTest : public AshTestBase {
   views::Widget* CreateModalWidget(const gfx::Rect& bounds) {
     views::Widget* widget = views::Widget::CreateWindowWithContextAndBounds(
         new TestDelegate(true), CurrentContext(), bounds);
+    // See the above comment.
+    widget->SetBounds(bounds);
     widget->Show();
     return widget;
   }
@@ -118,6 +125,8 @@ class RootWindowControllerTest : public AshTestBase {
                                              aura::Window* parent) {
     views::Widget* widget = views::Widget::CreateWindowWithParentAndBounds(
         new TestDelegate(true), parent, bounds);
+    // See the above comment.
+    widget->SetBounds(bounds);
     widget->Show();
     return widget;
   }
@@ -168,7 +177,6 @@ TEST_F(RootWindowControllerTest, MoveWindows_Basic) {
   views::Widget* unparented_control = new Widget;
   Widget::InitParams params;
   params.bounds = gfx::Rect(650, 10, 100, 100);
-  params.context = CurrentContext();
   params.type = Widget::InitParams::TYPE_CONTROL;
   unparented_control->Init(params);
   EXPECT_EQ(root_windows[1],
@@ -678,7 +686,7 @@ class VirtualKeyboardRootWindowControllerTest
         keyboard::switches::kEnableVirtualKeyboard);
     AshTestBase::SetUp();
     keyboard::SetTouchKeyboardEnabled(true);
-    Shell::Get()->CreateKeyboard();
+    Shell::Get()->EnableKeyboard();
   }
 
   void TearDown() override {
@@ -721,182 +729,6 @@ class TargetHitTestEventHandler : public ui::test::TestEventHandler {
   DISALLOW_COPY_AND_ASSIGN(TargetHitTestEventHandler);
 };
 
-// Test for http://crbug.com/297858. Virtual keyboard container should only show
-// on primary root window if no window has touch capability.
-TEST_F(VirtualKeyboardRootWindowControllerTest,
-       VirtualKeyboardOnPrimaryRootWindowDefault) {
-  UpdateDisplay("500x500,500x500");
-
-  aura::Window::Windows root_windows = Shell::GetAllRootWindows();
-  aura::Window* primary_root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* secondary_root_window = root_windows[0] == primary_root_window
-                                            ? root_windows[1]
-                                            : root_windows[0];
-
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-  ASSERT_TRUE(controller);
-  aura::Window* vk_container_in_primary = Shell::GetContainer(
-      primary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  aura::Window* vk_container_in_secondary = Shell::GetContainer(
-      secondary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  aura::Window* vk_window =
-      controller->GetContainerWindowWithoutCreationForTest();
-  EXPECT_TRUE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_FALSE(vk_container_in_secondary->Contains(vk_window));
-}
-
-// Test for http://crbug.com/303429. Virtual keyboard container should show on
-// a display which has touch capability.
-TEST_F(VirtualKeyboardRootWindowControllerTest,
-       VirtualKeyboardOnTouchableDisplayOnly) {
-  UpdateDisplay("500x500,500x500");
-  display::Display secondary_display =
-      Shell::Get()->display_manager()->GetSecondaryDisplay();
-  display::test::DisplayManagerTestApi(Shell::Get()->display_manager())
-      .SetTouchSupport(secondary_display.id(),
-                       display::Display::TouchSupport::AVAILABLE);
-
-  // The primary display doesn't have touch capability and the secondary display
-  // does.
-  ASSERT_NE(display::Display::TouchSupport::AVAILABLE,
-            display::Screen::GetScreen()->GetPrimaryDisplay().touch_support());
-  ASSERT_EQ(
-      display::Display::TouchSupport::AVAILABLE,
-      Shell::Get()->display_manager()->GetSecondaryDisplay().touch_support());
-
-  aura::Window::Windows root_windows = Shell::GetAllRootWindows();
-  aura::Window* primary_root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* secondary_root_window = root_windows[0] == primary_root_window
-                                            ? root_windows[1]
-                                            : root_windows[0];
-  aura::Window* vk_container_in_primary = Shell::GetContainer(
-      primary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  aura::Window* vk_container_in_secondary = Shell::GetContainer(
-      secondary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-
-  controller->ShowKeyboard(false);
-  aura::Window* vk_window =
-      controller->GetContainerWindowWithoutCreationForTest();
-  EXPECT_FALSE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_TRUE(vk_container_in_secondary->Contains(vk_window));
-
-  // Move the focus to the primary display.
-  aura::Window* focusable_window_in_primary_display =
-      CreateTestWindowInShellWithBounds(
-          primary_root_window->GetBoundsInScreen());
-  ASSERT_EQ(primary_root_window,
-            focusable_window_in_primary_display->GetRootWindow());
-  focusable_window_in_primary_display->Focus();
-  ASSERT_TRUE(focusable_window_in_primary_display->HasFocus());
-
-  // Virtual keyboard shows up on the secondary display even if a window in the
-  // primary screen has the focus.
-  controller->ShowKeyboard(false);
-  EXPECT_FALSE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_TRUE(vk_container_in_secondary->Contains(vk_window));
-}
-
-// Test for http://crbug.com/303429. If both of displays have touch capability,
-// virtual keyboard follows the input focus.
-TEST_F(VirtualKeyboardRootWindowControllerTest, FollowInputFocus) {
-  UpdateDisplay("500x500,500x500");
-  const int64_t primary_display_id =
-      display::Screen::GetScreen()->GetPrimaryDisplay().id();
-  display::test::DisplayManagerTestApi(Shell::Get()->display_manager())
-      .SetTouchSupport(primary_display_id,
-                       display::Display::TouchSupport::AVAILABLE);
-  const int64_t secondary_display_id =
-      Shell::Get()->display_manager()->GetSecondaryDisplay().id();
-  display::test::DisplayManagerTestApi(Shell::Get()->display_manager())
-      .SetTouchSupport(secondary_display_id,
-                       display::Display::TouchSupport::AVAILABLE);
-
-  // Both of displays have touch capability.
-  ASSERT_EQ(display::Display::TouchSupport::AVAILABLE,
-            display::Screen::GetScreen()->GetPrimaryDisplay().touch_support());
-  ASSERT_EQ(
-      display::Display::TouchSupport::AVAILABLE,
-      Shell::Get()->display_manager()->GetSecondaryDisplay().touch_support());
-
-  aura::Window::Windows root_windows = Shell::GetAllRootWindows();
-  aura::Window* primary_root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* secondary_root_window = root_windows[0] == primary_root_window
-                                            ? root_windows[1]
-                                            : root_windows[0];
-  aura::Window* vk_container_in_primary = Shell::GetContainer(
-      primary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  aura::Window* vk_container_in_secondary = Shell::GetContainer(
-      secondary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-  aura::Window* focusable_window_in_primary_display =
-      CreateTestWindowInShellWithBounds(
-          primary_root_window->GetBoundsInScreen());
-  ASSERT_EQ(primary_root_window,
-            focusable_window_in_primary_display->GetRootWindow());
-  aura::Window* focusable_window_in_secondary_display =
-      CreateTestWindowInShellWithBounds(
-          secondary_root_window->GetBoundsInScreen());
-  ASSERT_EQ(secondary_root_window,
-            focusable_window_in_secondary_display->GetRootWindow());
-
-  controller->ShowKeyboard(false);
-  aura::Window* vk_window =
-      controller->GetContainerWindowWithoutCreationForTest();
-  EXPECT_TRUE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_FALSE(vk_container_in_secondary->Contains(vk_window));
-
-  // Move the focus to the secondary display.
-  focusable_window_in_secondary_display->Focus();
-  ASSERT_TRUE(focusable_window_in_secondary_display->HasFocus());
-
-  controller->ShowKeyboard(false);
-  EXPECT_FALSE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_TRUE(vk_container_in_secondary->Contains(vk_window));
-
-  // Move back focus to the primary display.
-  focusable_window_in_primary_display->Focus();
-  ASSERT_TRUE(focusable_window_in_primary_display->HasFocus());
-
-  controller->ShowKeyboard(false);
-  EXPECT_TRUE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_FALSE(vk_container_in_secondary->Contains(vk_window));
-}
-
-// Test for http://crbug.com/303429. Even if both of display don't have touch
-// capability, the virtual keyboard shows up on the specified display.
-TEST_F(VirtualKeyboardRootWindowControllerTest,
-       VirtualKeyboardShowOnSpecifiedDisplay) {
-  UpdateDisplay("500x500,500x500");
-  display::Display secondary_display = GetSecondaryDisplay();
-
-  aura::Window::Windows root_windows = Shell::GetAllRootWindows();
-  aura::Window* primary_root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* secondary_root_window = root_windows[0] == primary_root_window
-                                            ? root_windows[1]
-                                            : root_windows[0];
-  aura::Window* vk_container_in_primary = Shell::GetContainer(
-      primary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  aura::Window* vk_container_in_secondary = Shell::GetContainer(
-      secondary_root_window, kShellWindowId_VirtualKeyboardContainer);
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-  aura::Window* vk_window =
-      controller->GetContainerWindowWithoutCreationForTest();
-
-  ASSERT_TRUE(vk_container_in_primary->Contains(vk_window));
-  ASSERT_FALSE(vk_container_in_secondary->Contains(vk_window));
-
-  const int64_t secondary_display_id = secondary_display.id();
-  controller->ShowKeyboardInDisplay(secondary_display_id);
-
-  EXPECT_FALSE(vk_container_in_primary->Contains(vk_window));
-  EXPECT_TRUE(vk_container_in_secondary->Contains(vk_window));
-}
-
 // Test for http://crbug.com/263599. Virtual keyboard should be able to receive
 // events at blocked user session.
 TEST_F(VirtualKeyboardRootWindowControllerTest,
@@ -908,7 +740,7 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
   keyboard_container->Show();
 
   aura::Window* contents_window =
-      keyboard::KeyboardController::GetInstance()->ui()->GetContentsWindow();
+      keyboard::KeyboardController::Get()->GetContentsWindow();
   contents_window->SetBounds(gfx::Rect());
   contents_window->Show();
 
@@ -934,23 +766,21 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
   root_window->RemovePreTargetHandler(&handler);
 }
 
-// Test for http://crbug.com/299787. RootWindowController should delete
-// the old container since the keyboard controller creates a new window in
-// GetWindowContainer().
+// Test for http://crbug.com/299787. RootWindowController should remove
+// the old keyboard window when we activate it elsewhere.
+// TODO(https://crbug.com/849995): This test no longer belongs here, but in
+// KeyboardController.
 TEST_F(VirtualKeyboardRootWindowControllerTest,
        DeleteOldContainerOnVirtualKeyboardInit) {
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-  aura::Window* keyboard_container =
-      controller->GetContainerWindowWithoutCreationForTest();
-  ASSERT_TRUE(keyboard_container);
-  // Track the keyboard container window.
+  auto* controller = keyboard::KeyboardController::Get();
+  aura::Window* keyboard_window = controller->GetContentsWindow();
+  // Track the keyboard contents window.
   aura::WindowTracker tracker;
-  tracker.Add(keyboard_container);
+  tracker.Add(keyboard_window);
   // Reinitialize the keyboard.
-  Shell::Get()->CreateKeyboard();
-  // keyboard_container should no longer be present.
-  EXPECT_FALSE(tracker.Contains(keyboard_container));
+  Shell::Get()->EnableKeyboard();
+  // keyboard_window should no longer be present.
+  EXPECT_FALSE(tracker.Contains(keyboard_window));
 }
 
 // Test for crbug.com/342524. After user login, the work space should restore to
@@ -960,9 +790,8 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, RestoreWorkspaceAfterLogin) {
   aura::Window* keyboard_container =
       Shell::GetContainer(root_window, kShellWindowId_VirtualKeyboardContainer);
   keyboard_container->Show();
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
-  aura::Window* contents_window = controller->ui()->GetContentsWindow();
+  auto* controller = keyboard::KeyboardController::Get();
+  aura::Window* contents_window = controller->GetContentsWindow();
   contents_window->SetBounds(
       keyboard::KeyboardBoundsFromRootBounds(root_window->bounds(), 100));
   contents_window->Show();
@@ -990,18 +819,15 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, RestoreWorkspaceAfterLogin) {
 // Ensure that system modal dialogs do not block events targeted at the virtual
 // keyboard.
 TEST_F(VirtualKeyboardRootWindowControllerTest, ClickWithActiveModalDialog) {
-  keyboard::KeyboardController* controller =
-      keyboard::KeyboardController::GetInstance();
+  auto* controller = keyboard::KeyboardController::Get();
   aura::Window* root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* container_window = controller->GetContainerWindow();
-  ASSERT_TRUE(container_window);
-  ASSERT_TRUE(root_window->Contains(container_window));
-  container_window->Show();
+  ASSERT_EQ(root_window, controller->GetRootWindow());
 
-  aura::Window* contents_window = controller->ui()->GetContentsWindow();
+  aura::Window* contents_window = controller->GetContentsWindow();
   contents_window->SetName("KeyboardContentsWindow");
   contents_window->SetBounds(
       keyboard::KeyboardBoundsFromRootBounds(root_window->bounds(), 100));
+  contents_window->Show();
 
   ui::test::TestEventHandler handler;
   root_window->AddPreTargetHandler(&handler);
@@ -1031,8 +857,7 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, ClickWithActiveModalDialog) {
 // Ensure that the visible area for scrolling the text caret excludes the
 // region occluded by the on-screen keyboard.
 TEST_F(VirtualKeyboardRootWindowControllerTest, EnsureCaretInWorkArea) {
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
   keyboard::KeyboardUI* ui = keyboard_controller->ui();
 
   MockTextInputClient text_input_client;
@@ -1041,8 +866,6 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, EnsureCaretInWorkArea) {
   input_method->SetFocusedTextInputClient(&text_input_client);
 
   aura::Window* root_window = Shell::GetPrimaryRootWindow();
-  ASSERT_TRUE(keyboard_controller->GetContainerWindowWithoutCreationForTest());
-  keyboard_controller->GetContainerWindow()->Show();
 
   const int keyboard_height = 100;
   aura::Window* contents_window = ui->GetContentsWindow();
@@ -1050,7 +873,7 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, EnsureCaretInWorkArea) {
       root_window->bounds(), keyboard_height));
   contents_window->Show();
 
-  ui->EnsureCaretInWorkArea();
+  ui->EnsureCaretInWorkArea(contents_window->GetBoundsInScreen());
   ASSERT_EQ(root_window->bounds().width(),
             text_input_client.caret_exclude_rect().width());
   ASSERT_EQ(keyboard_height, text_input_client.caret_exclude_rect().height());
@@ -1071,8 +894,7 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
   aura::Window* primary_root_window = root_windows[0];
   aura::Window* secondary_root_window = root_windows[1];
 
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
   keyboard::KeyboardUI* ui = keyboard_controller->ui();
 
   MockTextInputClient text_input_client;
@@ -1083,15 +905,12 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
   const int keyboard_height = 100;
   // Check that the keyboard on the primary screen doesn't cover the window on
   // the secondary screen.
-  aura::Window* keyboard_container = keyboard_controller->GetContainerWindow();
-  ASSERT_TRUE(keyboard_container);
-  keyboard_container->Show();
   aura::Window* contents_window = ui->GetContentsWindow();
   contents_window->SetBounds(keyboard::KeyboardBoundsFromRootBounds(
       primary_root_window->bounds(), keyboard_height));
   contents_window->Show();
 
-  ui->EnsureCaretInWorkArea();
+  ui->EnsureCaretInWorkArea(contents_window->GetBoundsInScreen());
   EXPECT_TRUE(primary_root_window->GetBoundsInScreen().Contains(
       text_input_client.caret_exclude_rect()));
   EXPECT_EQ(primary_root_window->GetBoundsInScreen().width(),
@@ -1101,11 +920,11 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
 
   // Move the keyboard into the secondary display and check that the keyboard
   // doesn't cover the window on the primary screen.
-  keyboard_controller->ShowKeyboardInDisplay(secondary_display_id);
+  keyboard_controller->ShowKeyboardInDisplay(GetSecondaryDisplay());
   contents_window->SetBounds(keyboard::KeyboardBoundsFromRootBounds(
       secondary_root_window->bounds(), keyboard_height));
 
-  ui->EnsureCaretInWorkArea();
+  ui->EnsureCaretInWorkArea(contents_window->GetBoundsInScreen());
   EXPECT_FALSE(primary_root_window->GetBoundsInScreen().Contains(
       text_input_client.caret_exclude_rect()));
   EXPECT_TRUE(secondary_root_window->GetBoundsInScreen().Contains(
@@ -1121,17 +940,12 @@ TEST_F(VirtualKeyboardRootWindowControllerTest,
 // crbug/377180.
 TEST_F(VirtualKeyboardRootWindowControllerTest, ZOrderTest) {
   UpdateDisplay("800x600");
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
-  keyboard::KeyboardUI* ui = keyboard_controller->ui();
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
 
   aura::Window* root_window = Shell::GetPrimaryRootWindow();
-  aura::Window* keyboard_container = keyboard_controller->GetContainerWindow();
-  ASSERT_TRUE(keyboard_container);
-  keyboard_container->Show();
 
   const int keyboard_height = 200;
-  aura::Window* contents_window = ui->GetContentsWindow();
+  aura::Window* contents_window = keyboard_controller->GetContentsWindow();
   gfx::Rect keyboard_bounds = keyboard::KeyboardBoundsFromRootBounds(
       root_window->bounds(), keyboard_height);
   contents_window->SetBounds(keyboard_bounds);
@@ -1201,16 +1015,82 @@ TEST_F(VirtualKeyboardRootWindowControllerTest, ZOrderTest) {
 // orientation. See crbug/417612.
 TEST_F(VirtualKeyboardRootWindowControllerTest, DisplayRotation) {
   UpdateDisplay("800x600");
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
-  aura::Window* keyboard_container = keyboard_controller->GetContainerWindow();
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
   keyboard_controller->ShowKeyboard(false);
-  keyboard_controller->ui()->GetContentsWindow()->SetBounds(
-      gfx::Rect(0, 400, 800, 200));
-  EXPECT_EQ("0,400 800x200", keyboard_container->bounds().ToString());
+
+  aura::Window* keyboard_window = keyboard_controller->GetContentsWindow();
+
+  keyboard_window->SetBounds(gfx::Rect(0, 400, 800, 200));
+  EXPECT_EQ("0,400 800x200", keyboard_window->bounds().ToString());
 
   UpdateDisplay("600x800");
-  EXPECT_EQ("0,600 600x200", keyboard_container->bounds().ToString());
+  EXPECT_EQ("0,600 600x200", keyboard_window->bounds().ToString());
+}
+
+// Keeps a count of all the events a window receives.
+class EventObserver : public ui::EventHandler {
+ public:
+  EventObserver() = default;
+  ~EventObserver() override = default;
+
+  int GetEventCount(ui::EventType type) { return event_counts_[type]; }
+  void ResetAllEventCounts() { event_counts_.clear(); }
+
+ private:
+  // Overridden from ui::EventHandler:
+  void OnEvent(ui::Event* event) override {
+    ui::EventHandler::OnEvent(event);
+    event_counts_[event->type()]++;
+  }
+
+  std::map<ui::EventType, int> event_counts_;
+
+  DISALLOW_COPY_AND_ASSIGN(EventObserver);
+};
+
+// Tests that tapping/clicking inside the keyboard does not give it focus.
+TEST_F(VirtualKeyboardRootWindowControllerTest, ClickDoesNotFocusKeyboard) {
+  aura::Window* root_window = Shell::GetPrimaryRootWindow();
+
+  // Create a test window in the background with the same size as the screen.
+  aura::test::EventCountDelegate delegate;
+  std::unique_ptr<aura::Window> background_window(
+      CreateTestWindowInShellWithDelegate(&delegate, 0, root_window->bounds()));
+  background_window->Focus();
+  EXPECT_TRUE(background_window->IsVisible());
+  EXPECT_TRUE(background_window->HasFocus());
+
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
+  keyboard_controller->ShowKeyboard(false);
+  keyboard_controller->NotifyContentsLoaded();
+
+  aura::Window* keyboard_window = keyboard_controller->GetContentsWindow();
+  keyboard_window->SetBounds(gfx::Rect(100, 100, 100, 100));
+  EXPECT_TRUE(keyboard_window->IsVisible());
+  EXPECT_FALSE(keyboard_window->HasFocus());
+
+  // Click on the keyboard. Make sure the keyboard receives the event, but does
+  // not get focus.
+  EventObserver observer;
+  keyboard_window->AddPreTargetHandler(&observer);
+
+  ui::test::EventGenerator generator(root_window);
+  generator.MoveMouseTo(keyboard_window->bounds().CenterPoint());
+  generator.ClickLeftButton();
+  EXPECT_TRUE(background_window->HasFocus());
+  EXPECT_FALSE(keyboard_window->HasFocus());
+  EXPECT_EQ("0 0", delegate.GetMouseButtonCountsAndReset());
+  EXPECT_EQ(1, observer.GetEventCount(ui::ET_MOUSE_PRESSED));
+  EXPECT_EQ(1, observer.GetEventCount(ui::ET_MOUSE_RELEASED));
+
+  // Click outside of the keyboard. It should reach the window behind.
+  observer.ResetAllEventCounts();
+  generator.MoveMouseTo(gfx::Point());
+  generator.ClickLeftButton();
+  EXPECT_EQ("1 1", delegate.GetMouseButtonCountsAndReset());
+  EXPECT_EQ(0, observer.GetEventCount(ui::ET_MOUSE_PRESSED));
+  EXPECT_EQ(0, observer.GetEventCount(ui::ET_MOUSE_RELEASED));
+  keyboard_window->RemovePreTargetHandler(&observer);
 }
 
 }  // namespace ash
