@@ -15,9 +15,9 @@
 #include "ash/session/session_observer.h"
 #include "ash/session/test_session_controller_client.h"
 #include "ash/shell.h"
+#include "ash/system/message_center/notification_tray.h"
 #include "ash/system/screen_security/screen_tray_item.h"
-#include "ash/system/tray/system_tray.h"
-#include "ash/system/web_notification/web_notification_tray.h"
+#include "ash/system/tray/system_tray_notifier.h"
 #include "ash/test/ash_test_base.h"
 #include "ash/wm/overview/window_selector_controller.h"
 #include "ash/wm/window_util.h"
@@ -445,7 +445,8 @@ TEST_F(SessionControllerPrefsTest, Observer) {
   EXPECT_EQ(nullptr, observer.last_user_pref_service());
 
   auto pref_service = std::make_unique<TestingPrefServiceSimple>();
-  Shell::RegisterProfilePrefs(pref_service->registry(), true /* for_test */);
+  Shell::RegisterUserProfilePrefs(pref_service->registry(),
+                                  true /* for_test */);
   controller->ProvideUserPrefServiceForTest(kUserAccount1,
                                             std::move(pref_service));
   EXPECT_EQ(controller->GetUserPrefServiceForUser(kUserAccount1),
@@ -472,7 +473,8 @@ TEST_F(SessionControllerPrefsTest, Observer) {
   // becoming initialized.
   observer.clear_last_user_pref_service();
   pref_service = std::make_unique<TestingPrefServiceSimple>();
-  Shell::RegisterProfilePrefs(pref_service->registry(), true /* for_test */);
+  Shell::RegisterUserProfilePrefs(pref_service->registry(),
+                                  true /* for_test */);
   controller->ProvideUserPrefServiceForTest(kUserAccount2,
                                             std::move(pref_service));
   EXPECT_EQ(nullptr, observer.last_user_pref_service());
@@ -557,29 +559,28 @@ class CanSwitchUserTest : public AshTestBase {
 
   void SetUp() override {
     AshTestBase::SetUp();
-    SystemTray* system_tray = GetPrimarySystemTray();
-    share_item_ = system_tray->GetScreenShareItem();
-    capture_item_ = system_tray->GetScreenCaptureItem();
-    EXPECT_TRUE(share_item_);
-    EXPECT_TRUE(capture_item_);
-    WebNotificationTray::DisableAnimationsForTest(true);
+    NotificationTray::DisableAnimationsForTest(true);
   }
 
   void TearDown() override {
     RunAllPendingInMessageLoop();
-    WebNotificationTray::DisableAnimationsForTest(false);
+    NotificationTray::DisableAnimationsForTest(false);
     AshTestBase::TearDown();
   }
 
   // Accessing the capture session functionality.
   // Simulates a screen capture session start.
   void StartCaptureSession() {
-    capture_item_->Start(base::Bind(&CanSwitchUserTest::StopCaptureCallback,
-                                    base::Unretained(this)));
+    Shell::Get()->system_tray_notifier()->NotifyScreenCaptureStart(
+        base::BindRepeating(&CanSwitchUserTest::StopCaptureCallback,
+                            base::Unretained(this)),
+        base::EmptyString16());
   }
 
   // The callback which gets called when the screen capture gets stopped.
-  void StopCaptureSession() { capture_item_->Stop(); }
+  void StopCaptureSession() {
+    Shell::Get()->system_tray_notifier()->NotifyScreenCaptureStop();
+  }
 
   // Simulates a screen capture session stop.
   void StopCaptureCallback() { stop_capture_callback_hit_count_++; }
@@ -587,12 +588,16 @@ class CanSwitchUserTest : public AshTestBase {
   // Accessing the share session functionality.
   // Simulate a Screen share session start.
   void StartShareSession() {
-    share_item_->Start(base::Bind(&CanSwitchUserTest::StopShareCallback,
-                                  base::Unretained(this)));
+    Shell::Get()->system_tray_notifier()->NotifyScreenShareStart(
+        base::BindRepeating(&CanSwitchUserTest::StopShareCallback,
+                            base::Unretained(this)),
+        base::EmptyString16());
   }
 
   // Simulates a screen share session stop.
-  void StopShareSession() { share_item_->Stop(); }
+  void StopShareSession() {
+    Shell::Get()->system_tray_notifier()->NotifyScreenShareStop();
+  }
 
   // The callback which gets called when the screen share gets stopped.
   void StopShareCallback() { stop_share_callback_hit_count_++; }
@@ -655,11 +660,6 @@ class CanSwitchUserTest : public AshTestBase {
     }
   }
 
-  // The two items from the SystemTray for the screen capture / share
-  // functionality.
-  ScreenTrayItem* capture_item_ = nullptr;
-  ScreenTrayItem* share_item_ = nullptr;
-
   // Various counters to query for.
   int stop_capture_callback_hit_count_ = 0;
   int stop_share_callback_hit_count_ = 0;
