@@ -6,6 +6,7 @@
 
 #include <memory>
 
+#include "ash/app_list/app_list_controller_impl.h"
 #include "ash/public/cpp/config.h"
 #include "ash/public/cpp/shelf_item_delegate.h"
 #include "ash/public/cpp/shelf_model.h"
@@ -19,7 +20,6 @@
 #include "ash/shell.h"
 #include "ash/system/status_area_widget.h"
 #include "base/logging.h"
-#include "ui/app_list/presenter/app_list.h"
 #include "ui/display/types/display_constants.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/keyboard/keyboard_controller_observer.h"
@@ -76,6 +76,7 @@ void Shelf::CreateShelfWidget(aura::Window* root) {
   aura::Window* shelf_container =
       root->GetChildById(kShellWindowId_ShelfContainer);
   shelf_widget_.reset(new ShelfWidget(shelf_container, this));
+  shelf_widget_->Initialize();
 
   DCHECK(!shelf_layout_manager_);
   shelf_layout_manager_ = shelf_widget_->shelf_layout_manager();
@@ -90,13 +91,11 @@ void Shelf::CreateShelfWidget(aura::Window* root) {
 }
 
 void Shelf::ShutdownShelfWidget() {
-  // May be called multiple times during shutdown.
-  if (shelf_widget_)
-    shelf_widget_->Shutdown();
+  shelf_widget_->Shutdown();
 }
 
 void Shelf::DestroyShelfWidget() {
-  // May be called multiple times during shutdown.
+  DCHECK(shelf_widget_);
   shelf_widget_.reset();
 }
 
@@ -190,8 +189,14 @@ ShelfVisibilityState Shelf::GetVisibilityState() const {
 }
 
 int Shelf::GetAccessibilityPanelHeight() const {
-  return shelf_layout_manager_ ? shelf_layout_manager_->chromevox_panel_height()
-                               : 0;
+  return shelf_layout_manager_
+             ? shelf_layout_manager_->accessibility_panel_height()
+             : 0;
+}
+
+void Shelf::SetAccessibilityPanelHeight(int height) {
+  if (shelf_layout_manager_)
+    shelf_layout_manager_->SetAccessibilityPanelHeight(height);
 }
 
 int Shelf::GetDockedMagnifierHeight() const {
@@ -269,7 +274,7 @@ bool Shelf::ProcessGestureEvent(const ui::GestureEvent& event) {
 }
 
 void Shelf::ProcessMouseWheelEvent(const ui::MouseWheelEvent& event) {
-  Shell::Get()->app_list()->ProcessMouseWheelEvent(event);
+  Shell::Get()->app_list_controller()->ProcessMouseWheelEvent(event);
 }
 
 void Shelf::AddObserver(ShelfObserver* observer) {
@@ -302,12 +307,12 @@ bool Shelf::ShouldHideOnSecondaryDisplay(session_manager::SessionState state) {
 
 void Shelf::SetVirtualKeyboardBoundsForTesting(const gfx::Rect& bounds) {
   keyboard::KeyboardStateDescriptor state;
-  state.is_available = !bounds.IsEmpty();
+  state.is_visible = !bounds.IsEmpty();
   state.is_locked = false;
   state.visual_bounds = bounds;
   state.occluded_bounds = bounds;
   state.displaced_bounds = gfx::Rect();
-  shelf_layout_manager_->OnKeyboardAvailabilityChanged(state.is_available);
+  shelf_layout_manager_->OnKeyboardVisibilityStateChanged(state.is_visible);
   shelf_layout_manager_->OnKeyboardVisibleBoundsChanged(state.visual_bounds);
   shelf_layout_manager_->OnKeyboardWorkspaceOccludedBoundsChanged(
       state.occluded_bounds);
@@ -329,11 +334,6 @@ LoginShelfView* Shelf::GetLoginShelfViewForTesting() {
 }
 
 void Shelf::WillDeleteShelfLayoutManager() {
-  if (Shell::GetAshConfig() == Config::MASH) {
-    // TODO(sky): this should be removed once Shell is used everywhere.
-    ShutdownShelfWidget();
-  }
-
   // Clear event handlers that might forward events to the destroyed instance.
   auto_hide_event_handler_.reset();
   bezel_event_handler_.reset();
