@@ -9,7 +9,11 @@
 #include <vector>
 
 #include "ash/animation/animation_change_type.h"
-#include "ash/keyboard/keyboard_observer_register.h"
+#include "ash/app_list/app_list_controller_impl.h"
+#include "ash/app_list/model/app_list_view_state.h"
+#include "ash/app_list/views/app_list_view.h"
+#include "ash/public/cpp/app_list/app_list_constants.h"
+#include "ash/public/cpp/app_list/app_list_features.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
 #include "ash/screen_util.h"
@@ -30,10 +34,6 @@
 #include "base/auto_reset.h"
 #include "base/command_line.h"
 #include "base/i18n/rtl.h"
-#include "ui/app_list/app_list_constants.h"
-#include "ui/app_list/app_list_features.h"
-#include "ui/app_list/presenter/app_list.h"
-#include "ui/app_list/views/app_list_view.h"
 #include "ui/base/ui_base_switches.h"
 #include "ui/compositor/layer.h"
 #include "ui/compositor/layer_animation_observer.h"
@@ -86,6 +86,12 @@ bool IsAppListWindow(const aura::Window* window) {
   return parent && parent->id() == kShellWindowId_AppListContainer;
 }
 
+bool IsTabletModeEnabled() {
+  return Shell::Get()
+      ->tablet_mode_controller()
+      ->IsTabletModeWindowManagerEnabled();
+}
+
 }  // namespace
 
 // ShelfLayoutManager::UpdateShelfObserver -------------------------------------
@@ -134,6 +140,10 @@ bool ShelfLayoutManager::State::IsScreenLocked() const {
   return session_state == session_manager::SessionState::LOCKED;
 }
 
+bool ShelfLayoutManager::State::IsActiveSessionState() const {
+  return session_state == session_manager::SessionState::ACTIVE;
+}
+
 bool ShelfLayoutManager::State::Equals(const State& other) const {
   return other.visibility_state == visibility_state &&
          (visibility_state != SHELF_AUTO_HIDE ||
@@ -152,7 +162,6 @@ ShelfLayoutManager::ShelfLayoutManager(ShelfWidget* shelf_widget, Shelf* shelf)
       shelf_(shelf),
       is_background_blur_enabled_(
           app_list::features::IsBackgroundBlurEnabled()),
-      keyboard_observer_(this),
       scoped_session_observer_(this) {
   DCHECK(shelf_widget_);
   DCHECK(shelf_);
@@ -160,6 +169,7 @@ ShelfLayoutManager::ShelfLayoutManager(ShelfWidget* shelf_widget, Shelf* shelf)
   ShellPort::Get()->AddLockStateObserver(this);
   Shell::Get()->activation_client()->AddObserver(this);
   state_.session_state = Shell::Get()->session_controller()->GetSessionState();
+  keyboard::KeyboardController::Get()->AddObserver(this);
 }
 
 ShelfLayoutManager::~ShelfLayoutManager() {
@@ -168,6 +178,7 @@ ShelfLayoutManager::~ShelfLayoutManager() {
 
   for (auto& observer : observers_)
     observer.WillDeleteShelfLayoutManager();
+  keyboard::KeyboardController::Get()->RemoveObserver(this);
   Shell::Get()->RemoveShellObserver(this);
   ShellPort::Get()->RemoveLockStateObserver(this);
 }
@@ -214,6 +225,11 @@ void ShelfLayoutManager::LayoutShelfAndUpdateBounds(bool change_work_area) {
 }
 
 void ShelfLayoutManager::LayoutShelf() {
+  // The ShelfWidget may be partially closed (no native widget) during shutdown
+  // so skip layout.
+  if (in_shutdown_)
+    return;
+
   LayoutShelfAndUpdateBounds(true);
 }
 
@@ -230,12 +246,6 @@ ShelfVisibilityState ShelfLayoutManager::CalculateShelfVisibility() {
 }
 
 void ShelfLayoutManager::UpdateVisibilityState() {
-  // Shelf is not available before login.
-  // TODO(crbug.com/701157): Remove this when the login webui fake-shelf is
-  // replaced with views.
-  if (!Shell::Get()->session_controller()->IsActiveUserSessionStarted())
-    return;
-
   // Bail out early after shelf is destroyed.
   aura::Window* shelf_window = shelf_widget_->GetNativeWindow();
   if (in_shutdown_ || !shelf_window)
@@ -244,8 +254,8 @@ void ShelfLayoutManager::UpdateVisibilityState() {
   if (shelf_->ShouldHideOnSecondaryDisplay(state_.session_state)) {
     // Needed to hide system tray on secondary display.
     SetState(SHELF_HIDDEN);
-  } else if ((state_.IsScreenLocked() || state_.IsAddingSecondaryUser())) {
-    // Needed to show system tray on web UI lock screen.
+  } else if (!state_.IsActiveSessionState()) {
+    // Needed to show system tray in non active session state.
     SetState(SHELF_VISIBLE);
   } else if (Shell::Get()->screen_pinning_controller()->IsPinned()) {
     SetState(SHELF_HIDDEN);
@@ -429,16 +439,12 @@ void ShelfLayoutManager::OnPinnedStateChanged(aura::Window* pinned_window) {
   UpdateVisibilityState();
 }
 
-void ShelfLayoutManager::OnVirtualKeyboardStateChanged(
-    bool activated,
-    aura::Window* root_window) {
-  UpdateKeyboardObserverFromStateChanged(
-      activated, root_window, shelf_widget_->GetNativeWindow()->GetRootWindow(),
-      &keyboard_observer_);
-}
-
 void ShelfLayoutManager::OnAppListVisibilityChanged(bool shown,
                                                     aura::Window* root_window) {
+  // Shell may be under destruction.
+  if (!shelf_widget_ || !shelf_widget_->GetNativeWindow())
+    return;
+
   if (shelf_widget_->GetNativeWindow()->GetRootWindow() != root_window)
     return;
 
@@ -446,6 +452,14 @@ void ShelfLayoutManager::OnAppListVisibilityChanged(bool shown,
   MaybeUpdateShelfBackground(AnimationChangeType::IMMEDIATE);
 }
 
+void ShelfLayoutManager::OnSplitViewModeStarted() {
+  MaybeUpdateShelfBackground(AnimationChangeType::ANIMATE);
+}
+
+void ShelfLayoutManager::OnSplitViewModeEnded() {
+  MaybeUpdateShelfBackground(AnimationChangeType::ANIMATE);
+}
+
 void ShelfLayoutManager::OnWindowActivated(ActivationReason reason,
                                            aura::Window* gained_active,
                                            aura::Window* lost_active) {
@@ -460,22 +474,18 @@ void ShelfLayoutManager::OnKeyboardAppearanceChanged(
   LayoutShelfAndUpdateBounds(change_work_area);
 }
 
-void ShelfLayoutManager::OnKeyboardAvailabilityChanged(
-    const bool is_available) {
+void ShelfLayoutManager::OnKeyboardVisibilityStateChanged(
+    const bool is_visible) {
   // On login screen if keyboard has been just hidden, update bounds just once
   // but ignore target_bounds.work_area_insets since shelf overlaps with login
   // window.
   if (Shell::Get()->session_controller()->IsUserSessionBlocked() &&
-      !is_available) {
+      !is_visible) {
     Shell::Get()->SetDisplayWorkAreaInsets(shelf_widget_->GetNativeWindow(),
                                            gfx::Insets());
   }
 }
 
-void ShelfLayoutManager::OnKeyboardClosed() {
-  keyboard_observer_.RemoveAll();
-}
-
 ShelfBackgroundType ShelfLayoutManager::GetShelfBackgroundType() const {
   if (state_.pre_lock_screen_animation_active)
     return SHELF_BACKGROUND_DEFAULT;
@@ -498,11 +508,15 @@ ShelfBackgroundType ShelfLayoutManager::GetShelfBackgroundType() const {
     return SHELF_BACKGROUND_OVERLAP;
   }
 
+  // If split view mode is active, make the shelf fully opapue.
+  if (Shell::Get()->IsSplitViewModeActive())
+    return SHELF_BACKGROUND_SPLIT_VIEW;
+
   return SHELF_BACKGROUND_DEFAULT;
 }
 
-void ShelfLayoutManager::SetChromeVoxPanelHeight(int height) {
-  chromevox_panel_height_ = height;
+void ShelfLayoutManager::SetAccessibilityPanelHeight(int height) {
+  accessibility_panel_height_ = height;
   LayoutShelf();
 }
 
@@ -652,10 +666,16 @@ void ShelfLayoutManager::UpdateBoundsAndOpacity(
                               &status_bounds);
     status_widget->SetBounds(status_bounds);
 
-    // For crbug.com/622431, when the shelf alignment is BOTTOM_LOCKED, we
-    // don't set display work area, as it is not real user-set alignment.
+    // Do not update the work area when the alignment changes to BOTTOM_LOCKED
+    // to prevent window movement when the screen is locked: crbug.com/622431
+    // The work area is initialized with BOTTOM_LOCKED insets to prevent window
+    // movement on async preference initialization in tests: crbug.com/834369
+    const display::Display display =
+        display::Screen::GetScreen()->GetDisplayNearestWindow(
+            shelf_widget_->GetNativeWindow());
     if (!state_.IsScreenLocked() && change_work_area &&
-        shelf_->alignment() != SHELF_ALIGNMENT_BOTTOM_LOCKED) {
+        (shelf_->alignment() != SHELF_ALIGNMENT_BOTTOM_LOCKED ||
+         display.work_area() == display.bounds())) {
       gfx::Insets insets;
       // If user session is blocked (login to new user session or add user to
       // the existing session - multi-profile) then give 100% of work area only
@@ -703,8 +723,8 @@ void ShelfLayoutManager::CalculateTargetBounds(const State& state,
   aura::Window* shelf_window = shelf_widget_->GetNativeWindow();
   gfx::Rect available_bounds =
       screen_util::GetDisplayBoundsWithShelf(shelf_window);
-  available_bounds.Inset(0, chromevox_panel_height_ + docked_magnifier_height_,
-                         0, 0);
+  available_bounds.Inset(
+      0, accessibility_panel_height_ + docked_magnifier_height_, 0, 0);
   int shelf_width = PrimaryAxisValue(available_bounds.width(), shelf_size);
   int shelf_height = PrimaryAxisValue(shelf_size, available_bounds.height());
   int bottom_shelf_vertical_offset = available_bounds.bottom() - shelf_height;
@@ -749,8 +769,8 @@ void ShelfLayoutManager::CalculateTargetBounds(const State& state,
 
   // Also push in the work area insets for both the ChromeVox panel and the
   // Docked Magnifier.
-  target_bounds->work_area_insets +=
-      gfx::Insets(chromevox_panel_height_ + docked_magnifier_height_, 0, 0, 0);
+  target_bounds->work_area_insets += gfx::Insets(
+      accessibility_panel_height_ + docked_magnifier_height_, 0, 0, 0);
 
   target_bounds->opacity = ComputeTargetOpacity(state);
   target_bounds->status_opacity =
@@ -887,8 +907,8 @@ gfx::Rect ShelfLayoutManager::GetAutoHideShowShelfRegionInScreen() const {
   else
     show_shelf_region_in_screen.set_width(kMaxAutoHideShowShelfRegionSize);
 
-  // TODO: Figure out if we need any special handling when the keyboard is
-  // visible.
+  // TODO(pkotwicz): Figure out if we need any special handling when the
+  // keyboard is visible.
   return show_shelf_region_in_screen;
 }
 
@@ -1099,12 +1119,12 @@ void ShelfLayoutManager::StartGestureDrag(
     const gfx::Rect shelf_bounds = GetIdealBounds();
     shelf_background_type_before_drag_ = shelf_background_type_;
     gesture_drag_status_ = GESTURE_DRAG_APPLIST_IN_PROGRESS;
-    Shell::Get()->app_list()->Show(
+    Shell::Get()->app_list_controller()->Show(
         display::Screen::GetScreen()
             ->GetDisplayNearestWindow(shelf_widget_->GetNativeWindow())
             .id(),
-        app_list::kSwipeFromShelf);
-    Shell::Get()->app_list()->UpdateYPositionAndOpacity(
+        app_list::kSwipeFromShelf, gesture_in_screen.time_stamp());
+    Shell::Get()->app_list_controller()->UpdateYPositionAndOpacity(
         shelf_bounds.y(), GetAppListBackgroundOpacityOnShelfOpacity());
     launcher_above_shelf_bottom_amount_ =
         shelf_bounds.bottom() - gesture_in_screen.location().y();
@@ -1128,13 +1148,13 @@ void ShelfLayoutManager::UpdateGestureDrag(
     // Dismiss the app list if the shelf changed to vertical alignment during
     // dragging.
     if (!shelf_->IsHorizontalAlignment()) {
-      Shell::Get()->app_list()->Dismiss();
+      Shell::Get()->app_list_controller()->DismissAppList();
       launcher_above_shelf_bottom_amount_ = 0.f;
       gesture_drag_status_ = GESTURE_DRAG_NONE;
       return;
     }
     const gfx::Rect shelf_bounds = GetIdealBounds();
-    Shell::Get()->app_list()->UpdateYPositionAndOpacity(
+    Shell::Get()->app_list_controller()->UpdateYPositionAndOpacity(
         std::min(gesture_in_screen.location().y(), shelf_bounds.y()),
         GetAppListBackgroundOpacityOnShelfOpacity());
     launcher_above_shelf_bottom_amount_ =
@@ -1192,7 +1212,7 @@ void ShelfLayoutManager::CompleteGestureDrag(
   // behavior if there is at least one window visible.
   gesture_drag_status_ = GESTURE_DRAG_COMPLETE_IN_PROGRESS;
   if (shelf_->auto_hide_behavior() != new_auto_hide_behavior &&
-      HasVisibleWindow()) {
+      HasVisibleWindow() && !IsTabletModeEnabled()) {
     shelf_->SetAutoHideBehavior(new_auto_hide_behavior);
   } else {
     UpdateVisibilityState();
@@ -1207,8 +1227,8 @@ void ShelfLayoutManager::CompleteAppListDrag(
   if (gesture_drag_status_ == GESTURE_DRAG_NONE)
     return;
 
-  using app_list::mojom::AppListState;
-  AppListState app_list_state = AppListState::PEEKING;
+  using app_list::AppListViewState;
+  AppListViewState app_list_state = AppListViewState::PEEKING;
   if (gesture_in_screen.type() == ui::ET_SCROLL_FLING_START &&
       fabs(gesture_in_screen.details().velocity_y()) >
           kAppListDragVelocityThreshold) {
@@ -1216,38 +1236,36 @@ void ShelfLayoutManager::CompleteAppListDrag(
     // list if the fling was fast enough and in the correct direction, otherwise
     // close it.
     app_list_state = gesture_in_screen.details().velocity_y() < 0
-                         ? AppListState::FULLSCREEN_ALL_APPS
-                         : AppListState::CLOSED;
+                         ? AppListViewState::FULLSCREEN_ALL_APPS
+                         : AppListViewState::CLOSED;
   } else {
     // Snap the app list to corresponding state according to the snapping
     // thresholds.
-    if (Shell::Get()
-            ->tablet_mode_controller()
-            ->IsTabletModeWindowManagerEnabled()) {
+    if (IsTabletModeEnabled()) {
       app_list_state = launcher_above_shelf_bottom_amount_ >
                                kAppListDragSnapToFullscreenThreshold
-                           ? AppListState::FULLSCREEN_ALL_APPS
-                           : AppListState::CLOSED;
+                           ? AppListViewState::FULLSCREEN_ALL_APPS
+                           : AppListViewState::CLOSED;
     } else {
       if (launcher_above_shelf_bottom_amount_ <=
           kAppListDragSnapToClosedThreshold)
-        app_list_state = AppListState::CLOSED;
+        app_list_state = AppListViewState::CLOSED;
       else if (launcher_above_shelf_bottom_amount_ <=
                kAppListDragSnapToPeekingThreshold)
-        app_list_state = AppListState::PEEKING;
+        app_list_state = AppListViewState::PEEKING;
       else
-        app_list_state = AppListState::FULLSCREEN_ALL_APPS;
+        app_list_state = AppListViewState::FULLSCREEN_ALL_APPS;
     }
   }
 
-  Shell::Get()->app_list()->EndDragFromShelf(app_list_state);
+  Shell::Get()->app_list_controller()->EndDragFromShelf(app_list_state);
 
   gesture_drag_status_ = GESTURE_DRAG_NONE;
 }
 
 void ShelfLayoutManager::CancelGestureDrag() {
   if (gesture_drag_status_ == GESTURE_DRAG_APPLIST_IN_PROGRESS) {
-    Shell::Get()->app_list()->Dismiss();
+    Shell::Get()->app_list_controller()->DismissAppList();
   } else {
     gesture_drag_status_ = GESTURE_DRAG_CANCEL_IN_PROGRESS;
     UpdateVisibilityState();
@@ -1274,6 +1292,15 @@ bool ShelfLayoutManager::CanStartFullscreenAppListDrag(
   if (scroll_y_hint >= 0)
     return false;
 
+  // In overview mode, app list for tablet mode is hidden temporarily and will
+  // be shown automatically after overview mode ends. So prevent opening it
+  // here.
+  if (Shell::Get()
+          ->app_list_controller()
+          ->IsHomeLauncherEnabledInTabletMode()) {
+    return false;
+  }
+
   return true;
 }
 
