@@ -10,6 +10,7 @@
 #include <utility>
 #include <vector>
 
+#include "ash/app_list/views/app_list_drag_and_drop_host.h"
 #include "ash/public/cpp/shelf_model_observer.h"
 #include "ash/public/interfaces/shelf.mojom.h"
 #include "ash/shelf/ink_drop_button_listener.h"
@@ -19,17 +20,17 @@
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "third_party/skia/include/core/SkColor.h"
-#include "ui/app_list/views/app_list_drag_and_drop_host.h"
 #include "ui/views/animation/bounds_animator_observer.h"
 #include "ui/views/animation/ink_drop_state.h"
 #include "ui/views/context_menu_controller.h"
 #include "ui/views/controls/button/button.h"
+#include "ui/views/controls/menu/menu_types.h"
 #include "ui/views/focus/focus_manager.h"
 #include "ui/views/view.h"
 #include "ui/views/view_model.h"
 
 namespace ui {
-class MenuModel;
+class SimpleMenuModel;
 }
 
 namespace views {
@@ -48,6 +49,7 @@ class Shelf;
 class ShelfButton;
 class ShelfModel;
 struct ShelfItem;
+class ShelfMenuModelAdapter;
 class ShelfWidget;
 
 enum ShelfAlignmentUmaEnumValue {
@@ -89,6 +91,10 @@ class ASH_EXPORT ShelfView : public views::View,
   // Returns true if we're showing a menu.
   bool IsShowingMenu() const;
 
+  // Returns true if we're showing a menu for |view|. |view| could be a
+  // ShelfButton or the ShelfView.
+  bool IsShowingMenuForView(views::View* view) const;
+
   // Returns true if overflow bubble is shown.
   bool IsShowingOverflowBubble() const;
 
@@ -174,6 +180,11 @@ class ASH_EXPORT ShelfView : public views::View,
   // True if the current |drag_view_| is the given |drag_view|.
   bool IsDraggedView(const ShelfButton* drag_view) const;
 
+  // Returns the list of open windows that correspond to the app represented by
+  // this shelf view.
+  const std::vector<aura::Window*> GetOpenWindowsForShelfView(
+      views::View* view);
+
   // Return the view model for test purposes.
   const views::ViewModel* view_model_for_test() const {
     return view_model_.get();
@@ -294,6 +305,20 @@ class ASH_EXPORT ShelfView : public views::View,
   // Updates the visible range of overflow items in |overflow_view|.
   void UpdateOverflowRange(ShelfView* overflow_view) const;
 
+  // Gets the menu anchor rect for menus. |source| is the view that is
+  // asking for a menu, |location| is the location of the event, |context_menu|
+  // is whether the menu is for a context or application menu.
+  gfx::Rect GetMenuAnchorRect(const views::View& source,
+                              const gfx::Point& location,
+                              bool context_menu) const;
+
+  // Gets the menu anchor position for a menu. |for_item| is true if the menu is
+  // for an item on the shelf, or false if the menu is for the shelf view
+  // itself, |context_menu| is whether the menu will be an application menu or
+  // context menu, and |touch_menu| is whether the menu was initiated by touch.
+  views::MenuAnchorPosition GetMenuAnchorPosition(bool for_item,
+                                                  bool context_menu) const;
+
   // Overridden from views::View:
   gfx::Size CalculatePreferredSize() const override;
   void OnBoundsChanged(const gfx::Rect& previous_bounds) override;
@@ -334,16 +359,6 @@ class ASH_EXPORT ShelfView : public views::View,
       ShelfAction action,
       base::Optional<std::vector<mojom::MenuItemPtr>> menu_items);
 
-  // Show a list of all running items for this shelf |item|; it only shows a
-  // menu if there are multiple running items. |source| specifies the view
-  // responsible for showing the menu, and the bubble will point towards it.
-  // The |event_flags| are the flags of the event which triggered this menu.
-  // Returns |true| if a menu is shown.
-  bool ShowListMenuForView(const ShelfItem& item,
-                           views::View* source,
-                           const ui::Event& event,
-                           views::InkDrop* ink_drop);
-
   // Overridden from views::ContextMenuController:
   void ShowContextMenuForView(views::View* source,
                               const gfx::Point& point,
@@ -353,15 +368,14 @@ class ASH_EXPORT ShelfView : public views::View,
   // If |context_menu| is set, the displayed menu is a context menu and not
   // a menu listing one or more running applications.
   // The |click_point| is only used for |context_menu|'s.
-  void ShowMenu(std::unique_ptr<ui::MenuModel> menu_model,
+  void ShowMenu(std::unique_ptr<ui::SimpleMenuModel> menu_model,
                 views::View* source,
                 const gfx::Point& click_point,
                 bool context_menu,
-                ui::MenuSourceType source_type,
-                views::InkDrop* ink_drop);
+                ui::MenuSourceType source_type);
 
   // Callback for MenuRunner.
-  void OnMenuClosed(views::InkDrop* ink_drop);
+  void OnMenuClosed(views::View* source);
 
   // Overridden from views::BoundsAnimatorObserver:
   void OnBoundsAnimatorProgressed(views::BoundsAnimator* animator) override;
@@ -372,6 +386,9 @@ class ASH_EXPORT ShelfView : public views::View,
   // item, we should ignore the call.
   bool IsRepostEvent(const ui::Event& event);
 
+  // Returns true if the given |item| is supposed to be shown to the user.
+  bool ShouldShowShelfItem(const ShelfItem& item);
+
   // Convenience accessor to model_->items().
   const ShelfItem* ShelfItemForView(const views::View* view) const;
 
@@ -381,6 +398,9 @@ class ASH_EXPORT ShelfView : public views::View,
 
   bool CanPrepareForDrag(Pointer pointer, const ui::LocatedEvent& event);
 
+  // Updates the back button opacity and focus behavior based on tablet mode.
+  void UpdateBackButton();
+
   // The model; owned by Launcher.
   ShelfModel* model_;
 
@@ -420,6 +440,10 @@ class ASH_EXPORT ShelfView : public views::View,
   // |dragging_| is set only if the mouse is dragged far enough.
   ShelfButton* drag_view_ = nullptr;
 
+  // The view showing a context menu. This can be either a ShelfView or
+  // ShelfButton.
+  views::View* menu_owner_ = nullptr;
+
   // Position of the mouse down event in |drag_view_|'s coordinates.
   gfx::Point drag_origin_;
 
@@ -431,9 +455,8 @@ class ASH_EXPORT ShelfView : public views::View,
 
   std::unique_ptr<views::FocusSearch> focus_search_;
 
-  // Manages the context menu, and the list menu.
-  std::unique_ptr<ui::MenuModel> menu_model_;
-  std::unique_ptr<views::MenuRunner> launcher_menu_runner_;
+  // Responsible for building and running all menus.
+  std::unique_ptr<ShelfMenuModelAdapter> shelf_menu_model_adapter_;
   std::unique_ptr<ScopedRootWindowForNewWindows>
       scoped_root_window_for_new_windows_;
 
@@ -447,10 +470,6 @@ class ASH_EXPORT ShelfView : public views::View,
   // The timestamp of the event which closed the last menu - or 0.
   base::TimeTicks closing_event_time_;
 
-  // The timestamp of the event which opened the last context menu on a
-  // ShelfButton. Used in metrics.
-  base::TimeTicks shelf_button_context_menu_time_;
-
   // True if a drag and drop operation created/pinned the item in the launcher
   // and it needs to be deleted/unpinned again if the operation gets cancelled.
   bool drag_and_drop_item_pinned_ = false;
