@@ -4,10 +4,14 @@
 
 #include "ash/shelf/shelf_widget.h"
 
+#include <utility>
+
 #include "ash/animation/animation_change_type.h"
 #include "ash/focus_cycler.h"
+#include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/ash_switches.h"
 #include "ash/public/cpp/window_properties.h"
+#include "ash/public/interfaces/kiosk_app_info.mojom.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
 #include "ash/shelf/app_list_button.h"
@@ -43,9 +47,13 @@ views::View* FindFirstOrLastFocusableChild(views::View* root,
   views::FocusTraversable* dummy_focus_traversable;
   views::View* dummy_focus_traversable_view;
   return search.FindNextFocusableView(
-      root, find_last_child, views::FocusSearch::DOWN,
-      false /*check_starting_view*/, &dummy_focus_traversable,
-      &dummy_focus_traversable_view);
+      root,
+      find_last_child ? views::FocusSearch::SearchDirection::kBackwards
+                      : views::FocusSearch::SearchDirection::kForwards,
+      views::FocusSearch::TraversalDirection::kDown,
+      views::FocusSearch::StartingViewPolicy::kSkipStartingView,
+      views::FocusSearch::AnchoredDialogPolicy::kCanGoIntoAnchoredDialog,
+      &dummy_focus_traversable, &dummy_focus_traversable_view);
 }
 
 }  // namespace
@@ -130,7 +138,7 @@ bool ShelfWidget::IsUsingViewsShelf() {
     case session_manager::SessionState::UNKNOWN:
     case session_manager::SessionState::LOGIN_PRIMARY:
     case session_manager::SessionState::LOGGED_IN_NOT_ACTIVE:
-      return switches::IsUsingViewsLogin();
+      return features::IsViewsLoginEnabled();
   }
 }
 
@@ -212,32 +220,28 @@ ShelfWidget::ShelfWidget(aura::Window* shelf_container, Shelf* shelf)
   // Calls back into |this| and depends on |shelf_view_|.
   background_animator_.AddObserver(this);
   background_animator_.AddObserver(delegate_view_);
-
-  // Sets initial session state to make sure the UI is properly shown.
-  OnSessionStateChanged(Shell::Get()->session_controller()->GetSessionState());
 }
 
 ShelfWidget::~ShelfWidget() {
   // Must call Shutdown() before destruction.
-  DCHECK(did_shutdown_);
+  DCHECK(!status_area_widget_);
+}
+
+void ShelfWidget::Initialize() {
+  // Sets initial session state to make sure the UI is properly shown.
+  OnSessionStateChanged(Shell::Get()->session_controller()->GetSessionState());
 }
 
 void ShelfWidget::Shutdown() {
-  if (did_shutdown_)
-    return;
-
   // Shutting down the status area widget may cause some widgets (e.g. bubbles)
   // to close, so uninstall the ShelfLayoutManager event filters first. Don't
   // reset the pointer until later because other widgets (e.g. app list) may
   // access it later in shutdown.
-  if (shelf_layout_manager_)
-    shelf_layout_manager_->PrepareForShutdown();
+  shelf_layout_manager_->PrepareForShutdown();
 
-  if (status_area_widget_) {
-    background_animator_.RemoveObserver(status_area_widget_.get());
-    Shell::Get()->focus_cycler()->RemoveWidget(status_area_widget_.get());
-    status_area_widget_.reset();
-  }
+  background_animator_.RemoveObserver(status_area_widget_.get());
+  Shell::Get()->focus_cycler()->RemoveWidget(status_area_widget_.get());
+  status_area_widget_.reset();
 
   // Don't need to update the shelf background during shutdown.
   background_animator_.RemoveObserver(delegate_view_);
@@ -247,8 +251,6 @@ void ShelfWidget::Shutdown() {
   Shell::Get()->focus_cycler()->RemoveWidget(this);
   SetFocusCycler(nullptr);
   RemoveObserver(this);
-
-  did_shutdown_ = true;
 }
 
 void ShelfWidget::CreateStatusAreaWidget(aura::Window* status_container) {
@@ -346,11 +348,20 @@ AppListButton* ShelfWidget::GetAppListButton() const {
   return shelf_view_->GetAppListButton();
 }
 
+BackButton* ShelfWidget::GetBackButton() const {
+  return shelf_view_->GetBackButton();
+}
+
 app_list::ApplicationDragAndDropHost*
 ShelfWidget::GetDragAndDropHostForAppList() {
   return shelf_view_;
 }
 
+void ShelfWidget::SetLoginKioskApps(
+    std::vector<mojom::KioskAppInfoPtr> kiosk_apps) {
+  login_shelf_view_->SetKioskApps(std::move(kiosk_apps));
+}
+
 void ShelfWidget::set_default_last_focusable_child(
     bool default_last_focusable_child) {
   delegate_view_->set_default_last_focusable_child(
