@@ -9,6 +9,7 @@
 
 #include "ash/ash_export.h"
 #include "ash/public/cpp/shelf_types.h"
+#include "ash/public/interfaces/kiosk_app_info.mojom.h"
 #include "ash/session/session_observer.h"
 #include "ash/shelf/shelf_background_animator.h"
 #include "ash/shelf/shelf_background_animator_observer.h"
@@ -24,6 +25,7 @@ class ApplicationDragAndDropHost;
 namespace ash {
 enum class AnimationChangeType;
 class AppListButton;
+class BackButton;
 class FocusCycler;
 class LoginShelfView;
 class Shelf;
@@ -43,6 +45,15 @@ class ASH_EXPORT ShelfWidget : public views::Widget,
   ShelfWidget(aura::Window* shelf_container, Shelf* shelf);
   ~ShelfWidget() override;
 
+  // Sets the initial session state and show the UI. Not part of the constructor
+  // because showing the UI triggers the accessibility checks in browser_tests,
+  // which will crash unless the constructor returns, allowing the caller
+  // to store the constructed widget.
+  void Initialize();
+
+  // Clean up prior to deletion.
+  void Shutdown();
+
   // Returns true if the views-based shelf is being shown.
   static bool IsUsingViewsShelf();
 
@@ -73,9 +84,6 @@ class ASH_EXPORT ShelfWidget : public views::Widget,
   void SetFocusCycler(FocusCycler* focus_cycler);
   FocusCycler* GetFocusCycler();
 
-  // Clean up prior to deletion.
-  void Shutdown();
-
   // See Shelf::UpdateIconPositionForPanel().
   void UpdateIconPositionForPanel(aura::Window* panel);
 
@@ -85,9 +93,15 @@ class ASH_EXPORT ShelfWidget : public views::Widget,
   // Returns the button that opens the app launcher.
   AppListButton* GetAppListButton() const;
 
+  // Returns the browser back button.
+  BackButton* GetBackButton() const;
+
   // Returns the ApplicationDragAndDropHost for this shelf.
   app_list::ApplicationDragAndDropHost* GetDragAndDropHostForAppList();
 
+  // Updates the list of kiosk apps for the login shelf view.
+  void SetLoginKioskApps(std::vector<mojom::KioskAppInfoPtr> kiosk_apps);
+
   void set_default_last_focusable_child(bool default_last_focusable_child);
 
   // Overridden from views::WidgetObserver:
@@ -143,8 +157,6 @@ class ASH_EXPORT ShelfWidget : public views::Widget,
 
   ScopedSessionObserver scoped_session_observer_;
 
-  bool did_shutdown_ = false;
-
   DISALLOW_COPY_AND_ASSIGN(ShelfWidget);
 };
 
