@@ -18,7 +18,6 @@ class Label;
 }
 
 namespace ash {
-class SystemClockObserver;
 class SystemInfoDefaultView;
 
 namespace tray {
@@ -27,14 +26,12 @@ class TimeView;
 
 // The bottom row of the system menu. The default view shows the current date
 // and power status. The tray view shows the current time.
-class ASH_EXPORT TraySystemInfo : public SystemTrayItem, public ClockObserver {
+class ASH_EXPORT TraySystemInfo : public SystemTrayItem {
  public:
   explicit TraySystemInfo(SystemTray* system_tray);
   ~TraySystemInfo() override;
 
   const tray::TimeView* GetTimeTrayForTesting() const;
-  const SystemInfoDefaultView* GetDefaultViewForTesting() const;
-  views::View* CreateDefaultViewForTesting(LoginStatus status);
 
  private:
   // SystemTrayItem:
@@ -44,20 +41,10 @@ class ASH_EXPORT TraySystemInfo : public SystemTrayItem, public ClockObserver {
   void OnDefaultViewDestroyed() override;
   void UpdateAfterShelfAlignmentChange() override;
 
-  // ClockObserver:
-  void OnDateFormatChanged() override;
-  void OnSystemClockTimeUpdated() override;
-  void OnSystemClockCanSetTimeChanged(bool can_set_time) override;
-  void Refresh() override;
-
   void SetupLabelForTimeTray(views::Label* label);
-  void UpdateTimeFormat();
 
   tray::TimeView* tray_view_;
   SystemInfoDefaultView* default_view_;
-  LoginStatus login_status_;
-
-  std::unique_ptr<SystemClockObserver> system_clock_observer_;
 
   DISALLOW_COPY_AND_ASSIGN(TraySystemInfo);
 };
