@@ -4,8 +4,8 @@
 
 #include "ash/system/enterprise/tray_enterprise.h"
 
-#include "ash/ash_view_ids.h"
 #include "ash/login_status.h"
+#include "ash/public/cpp/ash_view_ids.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/shell.h"
 #include "ash/strings/grit/ash_strings.h"
@@ -41,7 +41,8 @@ base::string16 GetEnterpriseMessage() {
 }  // namespace
 
 TrayEnterprise::TrayEnterprise(SystemTray* system_tray)
-    : SystemTrayItem(system_tray, UMA_ENTERPRISE), tray_view_(nullptr) {
+    : SystemTrayItem(system_tray, SystemTrayItemUmaType::UMA_ENTERPRISE),
+      tray_view_(nullptr) {
   Shell::Get()->system_tray_model()->enterprise_domain()->AddObserver(this);
 }
 
