@@ -6,6 +6,7 @@
 
 #include "ash/ime/ime_controller.h"
 #include "ash/ime/ime_switch_type.h"
+#include "ash/public/cpp/ash_features.h"
 #include "ash/public/interfaces/ime_info.mojom.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/shell.h"
@@ -14,7 +15,7 @@
 #include "ash/system/tray/actionable_view.h"
 #include "ash/system/tray/system_menu_button.h"
 #include "ash/system/tray/tray_constants.h"
-#include "ash/system/tray/tray_details_view.h"
+#include "ash/system/tray/tray_detailed_view.h"
 #include "ash/system/tray/tray_popup_item_style.h"
 #include "ash/system/tray/tray_popup_utils.h"
 #include "ash/system/tray/tri_view.h"
@@ -45,13 +46,13 @@ const int kMinFontSizeDelta = -10;
 // an IME property. A checkmark icon is shown in the row if selected.
 class ImeListItemView : public ActionableView {
  public:
-  ImeListItemView(SystemTrayItem* owner,
-                  ImeListView* list_view,
+  ImeListItemView(ImeListView* list_view,
                   const base::string16& id,
                   const base::string16& label,
                   bool selected,
-                  const SkColor button_color)
-      : ActionableView(owner, TrayPopupInkDropStyle::FILL_BOUNDS),
+                  const SkColor button_color,
+                  bool use_unified_theme)
+      : ActionableView(nullptr, TrayPopupInkDropStyle::FILL_BOUNDS),
         ime_list_view_(list_view),
         selected_(selected) {
     SetInkDropMode(InkDropHostView::InkDropMode::ON);
@@ -83,8 +84,8 @@ class ImeListItemView : public ActionableView {
     // The label shows the IME full name.
     auto* label_view = TrayPopupUtils::CreateDefaultLabel();
     label_view->SetText(label);
-    TrayPopupItemStyle style(
-        TrayPopupItemStyle::FontStyle::DETAILED_VIEW_LABEL);
+    TrayPopupItemStyle style(TrayPopupItemStyle::FontStyle::DETAILED_VIEW_LABEL,
+                             use_unified_theme);
     style.SetupLabel(label_view);
 
     label_view->SetHorizontalAlignment(gfx::ALIGN_LEFT);
@@ -145,7 +146,7 @@ class KeyboardStatusRow : public views::View {
   views::ToggleButton* toggle() const { return toggle_; }
   bool is_toggled() const { return toggle_->is_on(); }
 
-  void Init(views::ButtonListener* listener) {
+  void Init(views::ButtonListener* listener, bool use_unified_theme) {
     TrayPopupUtils::ConfigureAsStickyHeader(this);
     SetLayoutManager(std::make_unique<views::FillLayout>());
 
@@ -162,8 +163,8 @@ class KeyboardStatusRow : public views::View {
     auto* label = TrayPopupUtils::CreateDefaultLabel();
     label->SetText(ui::ResourceBundle::GetSharedInstance().GetLocalizedString(
         IDS_ASH_STATUS_TRAY_ACCESSIBILITY_VIRTUAL_KEYBOARD));
-    TrayPopupItemStyle style(
-        TrayPopupItemStyle::FontStyle::DETAILED_VIEW_LABEL);
+    TrayPopupItemStyle style(TrayPopupItemStyle::FontStyle::DETAILED_VIEW_LABEL,
+                             use_unified_theme);
     style.SetupLabel(label);
     tri_view->AddView(TriView::Container::CENTER, label);
 
@@ -181,11 +182,15 @@ class KeyboardStatusRow : public views::View {
   DISALLOW_COPY_AND_ASSIGN(KeyboardStatusRow);
 };
 
-ImeListView::ImeListView(SystemTrayItem* owner)
-    : TrayDetailsView(owner),
+ImeListView::ImeListView(DetailedViewDelegate* delegate)
+    : ImeListView(delegate, features::IsSystemTrayUnifiedEnabled()) {}
+
+ImeListView::ImeListView(DetailedViewDelegate* delegate, bool use_unified_theme)
+    : TrayDetailedView(delegate),
       last_item_selected_with_keyboard_(false),
       should_focus_ime_after_selection_with_keyboard_(false),
-      current_ime_view_(nullptr) {}
+      current_ime_view_(nullptr),
+      use_unified_theme_(use_unified_theme) {}
 
 ImeListView::~ImeListView() = default;
 
@@ -251,8 +256,8 @@ void ImeListView::AppendImeListAndProperties(
   for (size_t i = 0; i < list.size(); i++) {
     const bool selected = current_ime_id == list[i].id;
     views::View* ime_view =
-        new ImeListItemView(owner(), this, list[i].short_name, list[i].name,
-                            selected, gfx::kGoogleGreen700);
+        new ImeListItemView(this, list[i].short_name, list[i].name, selected,
+                            gfx::kGoogleGreen700, use_unified_theme_);
     scroll_content()->AddChildView(ime_view);
     ime_map_[ime_view] = list[i].id;
 
@@ -268,8 +273,8 @@ void ImeListView::AppendImeListAndProperties(
       // Adds the property items.
       for (size_t i = 0; i < property_list.size(); i++) {
         ImeListItemView* property_view = new ImeListItemView(
-            owner(), this, base::string16(), property_list[i].label,
-            property_list[i].checked, kMenuIconColor);
+            this, base::string16(), property_list[i].label,
+            property_list[i].checked, kMenuIconColor, use_unified_theme_);
         scroll_content()->AddChildView(property_view);
         property_map_[property_view] = property_list[i].key;
       }
@@ -286,7 +291,7 @@ void ImeListView::AppendImeListAndProperties(
 void ImeListView::PrependKeyboardStatusRow() {
   DCHECK(!keyboard_status_row_);
   keyboard_status_row_ = new KeyboardStatusRow;
-  keyboard_status_row_->Init(this);
+  keyboard_status_row_->Init(this, use_unified_theme_);
   scroll_content()->AddChildViewAt(keyboard_status_row_, 0);
 }
 
