@@ -7,8 +7,8 @@
 #include <memory>
 #include <utility>
 
-#include "ash/ash_view_ids.h"
 #include "ash/metrics/user_metrics_recorder.h"
+#include "ash/public/cpp/ash_view_ids.h"
 #include "ash/public/cpp/config.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/session/session_controller.h"
@@ -45,6 +45,7 @@
 #include "chromeos/network/network_state_handler_observer.h"
 #include "chromeos/network/proxy/ui_proxy_config_service.h"
 #include "components/device_event_log/device_event_log.h"
+#include "components/onc/onc_constants.h"
 #include "device/bluetooth/bluetooth_adapter_factory.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 #include "third_party/skia/include/core/SkColor.h"
@@ -66,9 +67,9 @@
 
 #include "ui/views/view.h"
 
+using chromeos::ManagedNetworkConfigurationHandler;
 using chromeos::NetworkHandler;
 using chromeos::NetworkStateHandler;
-using chromeos::ManagedNetworkConfigurationHandler;
 using chromeos::NetworkTypePattern;
 
 namespace ash {
@@ -79,28 +80,6 @@ const int64_t kBluetoothTimeoutDelaySeconds = 2;
 const int kMobileNetworkBatteryIconSize = 14;
 const int kPowerStatusPaddingRight = 10;
 
-bool IsProhibitedByPolicy(const chromeos::NetworkState* network) {
-  if (!NetworkTypePattern::WiFi().MatchesType(network->type()))
-    return false;
-  if (!Shell::Get()->session_controller()->IsActiveUserSessionStarted())
-    return false;
-  ManagedNetworkConfigurationHandler* managed_configuration_handler =
-      NetworkHandler::Get()->managed_network_configuration_handler();
-  const base::DictionaryValue* global_network_config =
-      managed_configuration_handler->GetGlobalConfigFromPolicy(
-          std::string() /* no username hash, device policy */);
-  bool policy_prohibites_unmanaged = false;
-  if (global_network_config) {
-    global_network_config->GetBooleanWithoutPathExpansion(
-        ::onc::global_network_config::kAllowOnlyPolicyNetworksToConnect,
-        &policy_prohibites_unmanaged);
-  }
-  if (!policy_prohibites_unmanaged)
-    return false;
-  return !managed_configuration_handler->FindPolicyByGuidAndProfile(
-      network->guid(), network->profile_path(), nullptr /* onc_source */);
-}
-
 bool IsCellularSimLocked() {
   const chromeos::DeviceState* cellular_device =
       NetworkHandler::Get()->network_state_handler()->GetDeviceStateByType(
@@ -116,6 +95,12 @@ void ShowCellularSettings() {
       cellular_network ? cellular_network->guid() : std::string());
 }
 
+bool IsSecondaryUser() {
+  SessionController* session_controller = Shell::Get()->session_controller();
+  return session_controller->IsActiveUserSessionStarted() &&
+         !session_controller->IsUserPrimary();
+}
+
 }  // namespace
 
 // A header row for sections in network detailed view which contains a title and
@@ -140,16 +125,6 @@ class NetworkListView::SectionHeaderRowView : public views::View,
 
   void SetSubtitle(int subtitle_id) {
     network_row_title_view_->SetSubtitle(subtitle_id);
-
-    // The left padding of the toggle is different depending on whether the
-    // subtitle is displayed.
-    const int toggle_left_padding = subtitle_id == 0
-                                        ? kToggleLeftPaddingWithoutSubtitle
-                                        : kToggleLeftPaddingWithSubtitle;
-    const gfx::Insets previous_insets = toggle_->border()->GetInsets();
-    toggle_->SetBorder(views::CreateEmptyBorder(
-        gfx::Insets(previous_insets.top(), toggle_left_padding,
-                    previous_insets.bottom(), previous_insets.right())));
   }
 
   virtual void SetToggleState(bool toggle_enabled, bool is_on) {
@@ -188,9 +163,6 @@ class NetworkListView::SectionHeaderRowView : public views::View,
   }
 
  private:
-  static const int kToggleLeftPaddingWithoutSubtitle = 18;
-  static const int kToggleLeftPaddingWithSubtitle = 8;
-
   void InitializeLayout() {
     TrayPopupUtils::ConfigureAsStickyHeader(this);
     SetLayoutManager(std::make_unique<views::FillLayout>());
@@ -233,7 +205,7 @@ namespace {
 class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
                             public chromeos::NetworkStateHandlerObserver {
  public:
-  MobileHeaderRowView(NetworkStateHandler* network_state_handler)
+  explicit MobileHeaderRowView(NetworkStateHandler* network_state_handler)
       : SectionHeaderRowView(IDS_ASH_STATUS_TRAY_NETWORK_MOBILE),
         network_state_handler_(network_state_handler),
         weak_ptr_factory_(this) {
@@ -316,6 +288,8 @@ class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
         network_state_handler_->GetTechnologyState(
             NetworkTypePattern::Tether());
 
+    bool default_toggle_enabled = !IsSecondaryUser();
+
     // If Cellular is available, toggle state and subtitle reflect Cellular.
     if (cellular_state != NetworkStateHandler::TECHNOLOGY_UNAVAILABLE) {
       const chromeos::DeviceState* cellular_device =
@@ -323,7 +297,7 @@ class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
               NetworkTypePattern::Cellular());
       bool cellular_enabled =
           cellular_state == NetworkStateHandler::TECHNOLOGY_ENABLED;
-      SetToggleState(true /* toggle_enabled */, cellular_enabled);
+      SetToggleState(default_toggle_enabled, cellular_enabled);
 
       int subtitle = 0;
       if (!cellular_device ||
@@ -370,7 +344,7 @@ class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
         // "Initializing...". TODO(stevenjb): Rename the string to _MOBILE.
         SetSubtitle(IDS_ASH_STATUS_TRAY_INITIALIZING_CELLULAR);
       } else {
-        SetToggleState(true /* toggle_enabled */, false /* is_on */);
+        SetToggleState(default_toggle_enabled, false /* is_on */);
         SetSubtitle(IDS_ASH_STATUS_TRAY_ENABLING_MOBILE_ENABLES_BLUETOOTH);
       }
       return;
@@ -388,7 +362,7 @@ class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
         network_state_handler_->SetTechnologyEnabled(
             NetworkTypePattern::Tether(), true /* enabled */,
             chromeos::network_handler::ErrorCallback());
-        SetToggleState(true /* toggle_enabled */, true /* is_on */);
+        SetToggleState(default_toggle_enabled, true /* is_on */);
         // "Initializing...". TODO(stevenjb): Rename the string to _MOBILE.
         SetSubtitle(IDS_ASH_STATUS_TRAY_INITIALIZING_CELLULAR);
         return;
@@ -401,7 +375,7 @@ class MobileHeaderRowView : public NetworkListView::SectionHeaderRowView,
                               NetworkTypePattern::Tether())) {
       subtitle = IDS_ASH_STATUS_TRAY_NO_MOBILE_DEVICES_FOUND;
     }
-    SetToggleState(true /* toggle_enabled */, tether_enabled /* is_on */);
+    SetToggleState(default_toggle_enabled, tether_enabled /* is_on */);
     SetSubtitle(subtitle);
   }
 
@@ -444,12 +418,12 @@ class WifiHeaderRowView : public NetworkListView::SectionHeaderRowView {
  public:
   WifiHeaderRowView()
       : SectionHeaderRowView(IDS_ASH_STATUS_TRAY_NETWORK_WIFI),
-        join_(nullptr) {}
+        join_button_(nullptr) {}
 
   ~WifiHeaderRowView() override = default;
 
   void SetToggleState(bool toggle_enabled, bool is_on) override {
-    join_->SetEnabled(is_on);
+    join_button_->SetEnabled(toggle_enabled && is_on);
     SectionHeaderRowView::SetToggleState(toggle_enabled, is_on);
   }
 
@@ -473,19 +447,19 @@ class WifiHeaderRowView : public NetworkListView::SectionHeaderRowView {
     gfx::ImageSkia disabled_image = network_icon::GetImageForNewWifiNetwork(
         SkColorSetA(prominent_color, kDisabledJoinIconAlpha),
         SkColorSetA(prominent_color, kDisabledJoinBadgeAlpha));
-    join_ = new SystemMenuButton(this, normal_image, disabled_image,
-                                 IDS_ASH_STATUS_TRAY_OTHER_WIFI);
-    join_->SetInkDropColor(prominent_color);
-    join_->SetEnabled(enabled);
-    container()->AddView(TriView::Container::END, join_);
+    join_button_ = new SystemMenuButton(this, normal_image, disabled_image,
+                                        IDS_ASH_STATUS_TRAY_OTHER_WIFI);
+    join_button_->SetInkDropColor(prominent_color);
+    join_button_->SetEnabled(enabled);
+    container()->AddView(TriView::Container::END, join_button_);
   }
 
   void ButtonPressed(views::Button* sender, const ui::Event& event) override {
-    if (sender == join_) {
+    if (sender == join_button_) {
       Shell::Get()->metrics()->RecordUserMetricsAction(
           UMA_STATUS_AREA_NETWORK_JOIN_OTHER_CLICKED);
       Shell::Get()->system_tray_controller()->ShowNetworkCreate(
-          shill::kTypeWifi);
+          ::onc::network_type::kWiFi);
       return;
     }
     SectionHeaderRowView::ButtonPressed(sender, event);
@@ -505,7 +479,7 @@ class WifiHeaderRowView : public NetworkListView::SectionHeaderRowView {
   static constexpr int kDisabledJoinIconAlpha = 0x1D;
 
   // A button to invoke "Join Wi-Fi network" dialog.
-  SystemMenuButton* join_;
+  SystemMenuButton* join_button_;
 
   DISALLOW_COPY_AND_ASSIGN(WifiHeaderRowView);
 };
@@ -514,8 +488,9 @@ class WifiHeaderRowView : public NetworkListView::SectionHeaderRowView {
 
 // NetworkListView:
 
-NetworkListView::NetworkListView(SystemTrayItem* owner, LoginStatus login)
-    : NetworkStateListDetailedView(owner, LIST_TYPE_NETWORK, login),
+NetworkListView::NetworkListView(DetailedViewDelegate* delegate,
+                                 LoginStatus login)
+    : NetworkStateListDetailedView(delegate, LIST_TYPE_NETWORK, login),
       needs_relayout_(false),
       no_wifi_networks_view_(nullptr),
       mobile_header_view_(nullptr),
@@ -585,7 +560,13 @@ void NetworkListView::UpdateNetworkIcons() {
         handler->GetNetworkStateFromGuid(info->guid);
     if (!network)
       continue;
-    bool prohibited_by_policy = IsProhibitedByPolicy(network);
+    bool prohibited_by_policy =
+        Shell::Get()->session_controller()->IsActiveUserSessionStarted() &&
+        NetworkHandler::Get()
+            ->managed_network_configuration_handler()
+            ->IsNetworkBlockedByPolicy(network->type(), network->guid(),
+                                       network->profile_path(),
+                                       network->GetHexSsid());
     info->label = network_icon::GetLabelForNetwork(
         network, network_icon::ICON_TYPE_MENU_LIST);
     info->image =
@@ -705,10 +686,12 @@ NetworkListView::UpdateNetworkListEntries() {
     return new_guids;
   }
 
+  bool should_clear_info_label = true;
   if (!handler->FirstNetworkByType(NetworkTypePattern::WiFi())) {
     UpdateInfoLabel(IDS_ASH_STATUS_TRAY_NETWORK_WIFI_ENABLED, index,
                     &no_wifi_networks_view_);
     ++index;
+    should_clear_info_label = false;
   }
 
   // Add Wi-Fi networks.
@@ -721,6 +704,9 @@ NetworkListView::UpdateNetworkListEntries() {
   if (index == 0) {
     UpdateInfoLabel(IDS_ASH_STATUS_TRAY_NO_NETWORKS, index,
                     &no_wifi_networks_view_);
+  } else if (should_clear_info_label) {
+    // Update the label to show nothing.
+    UpdateInfoLabel(0, index, &no_wifi_networks_view_);
   }
 
   return new_guids;
@@ -744,7 +730,7 @@ bool NetworkListView::ShouldMobileDataSectionBeShown() {
   // Secondary users cannot enable Bluetooth, and Tether is only UNINITIALIZED
   // if Bluetooth is disabled. Hide the section in this case.
   if (handler->IsTechnologyUninitialized(NetworkTypePattern::Tether()) &&
-      !Shell::Get()->session_controller()->IsUserPrimary()) {
+      IsSecondaryUser()) {
     return false;
   }
 
@@ -775,16 +761,17 @@ void NetworkListView::UpdateViewForNetwork(HoverHighlightView* view,
 
   // Add an additional icon to the right of the label for networks
   // that require it (e.g. Tether, controlled by extension).
-  views::View* power_icon = CreatePowerStatusView(info);
-  if (power_icon) {
-    view->AddRightView(
-        power_icon, views::CreateEmptyBorder(
-                        gfx::Insets(0 /* top */, 0 /* left */, 0 /* bottom */,
-                                    kPowerStatusPaddingRight)));
+  views::View* icon = CreatePowerStatusView(info);
+  if (icon) {
+    view->AddRightView(icon, views::CreateEmptyBorder(gfx::Insets(
+                                 0 /* top */, 0 /* left */, 0 /* bottom */,
+                                 kPowerStatusPaddingRight)));
   } else {
-    views::View* controlled_icon = CreateControlledByExtensionView(info);
-    if (controlled_icon)
-      view->AddRightView(controlled_icon);
+    icon = CreatePolicyView(info);
+    if (!icon)
+      icon = CreateControlledByExtensionView(info);
+    if (icon)
+      view->AddRightView(icon);
   }
 
   needs_relayout_ = true;
@@ -816,6 +803,27 @@ views::View* NetworkListView::CreatePowerStatusView(const NetworkInfo& info) {
   return icon;
 }
 
+views::View* NetworkListView::CreatePolicyView(const NetworkInfo& info) {
+  // Check if the network is managed by policy.
+  const chromeos::NetworkState* network =
+      NetworkHandler::Get()->network_state_handler()->GetNetworkStateFromGuid(
+          info.guid);
+  if (!network)
+    return nullptr;
+  const base::DictionaryValue* policy =
+      NetworkHandler::Get()
+          ->managed_network_configuration_handler()
+          ->FindPolicyByGuidAndProfile(network->guid(), network->profile_path(),
+                                       nullptr /* onc_source */);
+  if (!policy)
+    return nullptr;
+
+  views::ImageView* controlled_icon = TrayPopupUtils::CreateMainImageView();
+  controlled_icon->SetImage(
+      gfx::CreateVectorIcon(kSystemMenuBusinessIcon, kMenuIconColor));
+  return controlled_icon;
+}
+
 views::View* NetworkListView::CreateControlledByExtensionView(
     const NetworkInfo& info) {
   NetworkingConfigDelegate* networking_config_delegate =
@@ -923,7 +931,7 @@ int NetworkListView::UpdateSectionHeaderRow(NetworkTypePattern pattern,
   // visible when the header row is not at the top of the list.
   if (child_index > 0) {
     if (!*separator_view)
-      *separator_view = TrayPopupUtils::CreateListSubHeaderSeparator();
+      *separator_view = CreateListSubHeaderSeparator();
     PlaceViewAtIndex(*separator_view, child_index++);
   } else {
     if (*separator_view)
@@ -931,9 +939,10 @@ int NetworkListView::UpdateSectionHeaderRow(NetworkTypePattern pattern,
     *separator_view = nullptr;
   }
 
+  bool default_toggle_enabled = !IsSecondaryUser();
   // Mobile updates its toggle state independently.
   if (!pattern.MatchesPattern(NetworkTypePattern::Mobile()))
-    (*view)->SetToggleState(true /* toggle_enabled */, enabled /* is_on */);
+    (*view)->SetToggleState(default_toggle_enabled, enabled /* is_on */);
   PlaceViewAtIndex(*view, child_index++);
   return child_index;
 }
