@@ -53,6 +53,12 @@ constexpr int kBubbleMargin = 8;
 // Elevation used for the bubble shadow effect (tiny).
 constexpr int kBubbleShadowElevation = 2;
 
+bool IsSecondaryUser() {
+  SessionController* session_controller = Shell::Get()->session_controller();
+  return session_controller->IsActiveUserSessionStarted() &&
+         !session_controller->IsUserPrimary();
+}
+
 }  // namespace
 
 // A bubble which displays network info.
@@ -170,10 +176,10 @@ class InfoThrobberLayout : public views::LayoutManager {
 // NetworkStateListDetailedView
 
 NetworkStateListDetailedView::NetworkStateListDetailedView(
-    SystemTrayItem* owner,
+    DetailedViewDelegate* delegate,
     ListType list_type,
     LoginStatus login)
-    : TrayDetailsView(owner),
+    : TrayDetailedView(delegate),
       list_type_(list_type),
       login_(login),
       info_button_(nullptr),
@@ -218,8 +224,7 @@ void NetworkStateListDetailedView::HandleButtonPressed(views::Button* sender,
   if (sender == settings_button_)
     ShowSettings();
 
-  if (owner()->system_tray())
-    owner()->system_tray()->CloseBubble();
+  CloseBubble();
 }
 
 void NetworkStateListDetailedView::HandleViewClicked(views::View* view) {
@@ -233,24 +238,27 @@ void NetworkStateListDetailedView::HandleViewClicked(views::View* view) {
   const NetworkState* network =
       NetworkHandler::Get()->network_state_handler()->GetNetworkStateFromGuid(
           guid);
-  // TODO(stevenjb): Test network->connectable() here instead of
-  // IsDefaultCellular once network configuration is integrated into Settings.
-  // crbug.com/380937.
-  if (!network || network->IsConnectingOrConnected() ||
-      network->IsDefaultCellular()) {
-    Shell::Get()->metrics()->RecordUserMetricsAction(
-        list_type_ == LIST_TYPE_VPN
-            ? UMA_STATUS_AREA_SHOW_VPN_CONNECTION_DETAILS
-            : UMA_STATUS_AREA_SHOW_NETWORK_CONNECTION_DETAILS);
-    Shell::Get()->system_tray_controller()->ShowNetworkSettings(
-        network ? network->guid() : std::string());
-  } else {
+  bool can_connect = network && !network->IsConnectingOrConnected();
+  if (network->IsDefaultCellular())
+    can_connect = false;  // Default Cellular network is not connectable.
+  if (!network->connectable() && IsSecondaryUser()) {
+    // Secondary users can only connect to fully configured networks.
+    can_connect = false;
+  }
+  if (can_connect) {
     Shell::Get()->metrics()->RecordUserMetricsAction(
         list_type_ == LIST_TYPE_VPN
             ? UMA_STATUS_AREA_CONNECT_TO_VPN
             : UMA_STATUS_AREA_CONNECT_TO_CONFIGURED_NETWORK);
     chromeos::NetworkConnect::Get()->ConnectToNetworkId(network->guid());
+    return;
   }
+  Shell::Get()->metrics()->RecordUserMetricsAction(
+      list_type_ == LIST_TYPE_VPN
+          ? UMA_STATUS_AREA_SHOW_VPN_CONNECTION_DETAILS
+          : UMA_STATUS_AREA_SHOW_NETWORK_CONNECTION_DETAILS);
+  Shell::Get()->system_tray_controller()->ShowNetworkSettings(
+      network ? network->guid() : std::string());
 }
 
 void NetworkStateListDetailedView::CreateExtraTitleRowButtons() {
@@ -260,13 +268,11 @@ void NetworkStateListDetailedView::CreateExtraTitleRowButtons() {
   DCHECK(!info_button_);
   tri_view()->SetContainerVisible(TriView::Container::END, true);
 
-  info_button_ = new SystemMenuButton(this, kSystemMenuInfoIcon,
-                                      IDS_ASH_STATUS_TRAY_NETWORK_INFO);
+  info_button_ = CreateInfoButton(IDS_ASH_STATUS_TRAY_NETWORK_INFO);
   tri_view()->AddView(TriView::Container::END, info_button_);
 
   DCHECK(!settings_button_);
-  settings_button_ = new SystemMenuButton(this, kSystemMenuSettingsIcon,
-                                          IDS_ASH_STATUS_TRAY_NETWORK_SETTINGS);
+  settings_button_ = CreateSettingsButton(IDS_ASH_STATUS_TRAY_NETWORK_SETTINGS);
   tri_view()->AddView(TriView::Container::END, settings_button_);
 }
 
