@@ -5,6 +5,7 @@
 #include "ash/system/network/tray_network.h"
 
 #include "ash/login_status.h"
+#include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/config.h"
 #include "ash/shell.h"
 #include "ash/system/network/network_list.h"
@@ -12,12 +13,10 @@
 #include "ash/system/tray/system_tray_test_api.h"
 #include "ash/test/ash_test_base.h"
 #include "base/macros.h"
+#include "base/run_loop.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/network/network_handler.h"
 #include "components/prefs/testing_pref_service.h"
-#include "ui/message_center/message_center.h"
-
-using message_center::MessageCenter;
 
 namespace ash {
 namespace {
@@ -40,7 +39,7 @@ class TrayNetworkTest : public AshTestBase {
                                                               &local_state_);
     }
     // Networking stubs may have asynchronous initialization.
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
   }
 
   void TearDown() override {
@@ -62,54 +61,47 @@ class TrayNetworkTest : public AshTestBase {
 
 // Verifies that the network views can be created.
 TEST_F(TrayNetworkTest, Basics) {
+  // TODO(tetsui): Remove the test after UnifiedSystemTray launch.
+  // https://crbug.com/847104
+  if (features::IsSystemTrayUnifiedEnabled())
+    return;
+
   // Open the system tray menu.
   SystemTray* system_tray = GetPrimarySystemTray();
   system_tray->ShowDefaultView(BUBBLE_CREATE_NEW, false /* show_by_click */);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Show network details.
   TrayNetwork* tray_network = SystemTrayTestApi(system_tray).tray_network();
   const int close_delay_in_seconds = 0;
   system_tray->ShowDetailedView(tray_network, close_delay_in_seconds,
                                 BUBBLE_USE_EXISTING);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Network details view was created.
   ASSERT_TRUE(tray_network->detailed());
   EXPECT_TRUE(tray_network->detailed()->visible());
 }
 
-// Verifies that toggling Wi-Fi (usually via keyboard) shows a notification.
-TEST_F(TrayNetworkTest, ToggleWifi) {
-  TrayNetwork* tray_network =
-      SystemTrayTestApi(GetPrimarySystemTray()).tray_network();
-
-  // No notifications at startup.
-  ASSERT_EQ(0u, MessageCenter::Get()->NotificationCount());
-
-  // Simulate a user action to toggle Wi-Fi.
-  tray_network->RequestToggleWifi();
-
-  // Notification was shown.
-  EXPECT_EQ(1u, MessageCenter::Get()->NotificationCount());
-  EXPECT_TRUE(MessageCenter::Get()->HasPopupNotifications());
-  EXPECT_TRUE(MessageCenter::Get()->FindVisibleNotificationById("wifi-toggle"));
-}
-
 // Open network info bubble and close network detailed view. Confirm that it
 // doesn't crash.
 TEST_F(TrayNetworkTest, NetworkInfoBubble) {
+  // TODO(tetsui): Remove the test after UnifiedSystemTray launch.
+  // https://crbug.com/847104
+  if (features::IsSystemTrayUnifiedEnabled())
+    return;
+
   // Open the system tray menu.
   SystemTray* system_tray = GetPrimarySystemTray();
   system_tray->ShowDefaultView(BUBBLE_CREATE_NEW, true /* show_by_click */);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Show network details.
   TrayNetwork* tray_network = SystemTrayTestApi(system_tray).tray_network();
   const int close_delay_in_seconds = 0;
   system_tray->ShowDetailedView(tray_network, close_delay_in_seconds,
                                 BUBBLE_USE_EXISTING);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Show info bubble.
   tray_network->detailed()->ToggleInfoBubbleForTesting();
