@@ -95,7 +95,8 @@ class TitleView : public views::View, public views::ButtonListener {
         new views::Label(l10n_util::GetStringUTF16(IDS_ASH_STYLUS_TOOLS_TITLE));
     title_label->SetHorizontalAlignment(gfx::ALIGN_LEFT);
     AddChildView(title_label);
-    TrayPopupItemStyle style(TrayPopupItemStyle::FontStyle::TITLE);
+    TrayPopupItemStyle style(TrayPopupItemStyle::FontStyle::TITLE,
+                             false /* use_unified_theme */);
     style.SetupLabel(title_label);
     layout_ptr->SetFlexForView(title_label, 1);
     help_button_ = new SystemMenuButton(this, kSystemMenuHelpIcon,
@@ -206,8 +207,8 @@ void PaletteTray::OnActiveUserPrefServiceChanged(PrefService* pref_service) {
   pref_change_registrar_user_->Init(pref_service);
   pref_change_registrar_user_->Add(
       prefs::kEnableStylusTools,
-      base::Bind(&PaletteTray::OnPaletteEnabledPrefChanged,
-                 base::Unretained(this)));
+      base::BindRepeating(&PaletteTray::OnPaletteEnabledPrefChanged,
+                          base::Unretained(this)));
 
   // Read the initial value.
   OnPaletteEnabledPrefChanged();
@@ -253,8 +254,8 @@ void PaletteTray::OnLocalStatePrefServiceInitialized(
   pref_change_registrar_local_->Init(local_state_pref_service_);
   pref_change_registrar_local_->Add(
       prefs::kHasSeenStylus,
-      base::Bind(&PaletteTray::OnHasSeenStylusPrefChanged,
-                 base::Unretained(this)));
+      base::BindRepeating(&PaletteTray::OnHasSeenStylusPrefChanged,
+                          base::Unretained(this)));
 
   OnHasSeenStylusPrefChanged();
 }
