@@ -4,19 +4,23 @@
 
 #include "ash/system/tray/system_tray_controller.h"
 
+#include "ash/public/cpp/ash_features.h"
 #include "ash/root_window_controller.h"
 #include "ash/shell.h"
+#include "ash/system/audio/tray_audio.h"
+#include "ash/system/model/clock_model.h"
 #include "ash/system/model/enterprise_domain_model.h"
 #include "ash/system/model/system_tray_model.h"
+#include "ash/system/model/tracing_model.h"
 #include "ash/system/status_area_widget.h"
 #include "ash/system/tray/system_tray.h"
 #include "ash/system/tray/system_tray_notifier.h"
+#include "ash/system/unified/unified_system_tray.h"
 #include "ash/system/update/tray_update.h"
 
 namespace ash {
 
-SystemTrayController::SystemTrayController()
-    : hour_clock_type_(base::GetHourClockType()) {}
+SystemTrayController::SystemTrayController() {}
 
 SystemTrayController::~SystemTrayController() = default;
 
@@ -137,6 +141,11 @@ void SystemTrayController::ShowNetworkSettings(const std::string& network_id) {
     system_tray_client_->ShowNetworkSettings(network_id);
 }
 
+void SystemTrayController::ShowMultiDeviceSetup() {
+  if (system_tray_client_)
+    system_tray_client_->ShowMultiDeviceSetup();
+}
+
 void SystemTrayController::RequestRestartForUpdate() {
   if (system_tray_client_)
     system_tray_client_->RequestRestartForUpdate();
@@ -151,11 +160,6 @@ void SystemTrayController::SetClient(mojom::SystemTrayClientPtr client) {
 }
 
 void SystemTrayController::SetPrimaryTrayEnabled(bool enabled) {
-  ash::SystemTray* tray =
-      Shell::GetPrimaryRootWindowController()->GetSystemTray();
-  if (!tray)
-    return;
-
   // We disable the UI to prevent user from interacting with UI elements,
   // particularly with the system tray menu. However, in case if the system tray
   // bubble is opened at this point, it remains opened and interactive even
@@ -163,10 +167,29 @@ void SystemTrayController::SetPrimaryTrayEnabled(bool enabled) {
   // (http://crbug.com/497080). Close the menu to fix it. Calling
   // SystemTray::SetEnabled(false) guarantees, that the menu will not be opened
   // until the UI is enabled again.
-  if (!enabled && tray->HasSystemBubble())
-    tray->CloseBubble();
 
-  tray->SetEnabled(enabled);
+  if (features::IsSystemTrayUnifiedEnabled()) {
+    UnifiedSystemTray* tray = Shell::GetPrimaryRootWindowController()
+                                  ->GetStatusAreaWidget()
+                                  ->unified_system_tray();
+    if (!tray)
+      return;
+
+    if (!enabled && tray->IsBubbleShown())
+      tray->CloseBubble();
+
+    tray->SetEnabled(enabled);
+  } else {
+    ash::SystemTray* tray =
+        Shell::GetPrimaryRootWindowController()->GetSystemTray();
+    if (!tray)
+      return;
+
+    if (!enabled && tray->HasSystemBubble())
+      tray->CloseBubble();
+
+    tray->SetEnabled(enabled);
+  }
 }
 
 void SystemTrayController::SetPrimaryTrayVisible(bool visible) {
@@ -177,8 +200,7 @@ void SystemTrayController::SetPrimaryTrayVisible(bool visible) {
 }
 
 void SystemTrayController::SetUse24HourClock(bool use_24_hour) {
-  hour_clock_type_ = use_24_hour ? base::k24HourClock : base::k12HourClock;
-  Shell::Get()->system_tray_notifier()->NotifyDateFormatChanged();
+  Shell::Get()->system_tray_model()->SetUse24HourClock(use_24_hour);
 }
 
 void SystemTrayController::SetEnterpriseDisplayDomain(
@@ -186,38 +208,44 @@ void SystemTrayController::SetEnterpriseDisplayDomain(
     bool active_directory_managed) {
   Shell::Get()
       ->system_tray_model()
-      ->enterprise_domain()
       ->SetEnterpriseDisplayDomain(enterprise_display_domain,
                                    active_directory_managed);
 }
 
 void SystemTrayController::SetPerformanceTracingIconVisible(bool visible) {
-  Shell::Get()->system_tray_notifier()->NotifyTracingModeChanged(visible);
+  Shell::Get()->system_tray_model()->SetPerformanceTracingIconVisible(visible);
 }
 
 void SystemTrayController::ShowUpdateIcon(mojom::UpdateSeverity severity,
                                           bool factory_reset_required,
                                           mojom::UpdateType update_type) {
-  // Show the icon on all displays.
-  for (RootWindowController* root : Shell::GetAllRootWindowControllers()) {
-    ash::SystemTray* tray = root->GetSystemTray();
-    // External monitors might not have a tray yet.
-    if (!tray)
-      continue;
-    tray->tray_update()->ShowUpdateIcon(severity, factory_reset_required,
-                                        update_type);
-  }
+  Shell::Get()->system_tray_model()->ShowUpdateIcon(
+      severity, factory_reset_required, update_type);
 }
 
 void SystemTrayController::SetUpdateOverCellularAvailableIconVisible(
     bool visible) {
-  // Show the icon on all displays.
-  for (auto* root_window_controller : Shell::GetAllRootWindowControllers()) {
-    ash::SystemTray* tray = root_window_controller->GetSystemTray();
-    // External monitors might not have a tray yet.
-    if (!tray)
-      continue;
-    tray->tray_update()->SetUpdateOverCellularAvailableIconVisible(visible);
+  Shell::Get()->system_tray_model()->SetUpdateOverCellularAvailableIconVisible(
+      visible);
+}
+
+void SystemTrayController::ShowVolumeSliderBubble() {
+  // Show the bubble on all monitors with a system tray.
+  if (features::IsSystemTrayUnifiedEnabled()) {
+    for (RootWindowController* root : Shell::GetAllRootWindowControllers()) {
+      UnifiedSystemTray* system_tray =
+          root->GetStatusAreaWidget()->unified_system_tray();
+      if (!system_tray)
+        continue;
+      system_tray->ShowVolumeSliderBubble();
+    }
+  } else {
+    for (RootWindowController* root : Shell::GetAllRootWindowControllers()) {
+      ash::SystemTray* system_tray = root->GetSystemTray();
+      if (!system_tray)
+        continue;
+      system_tray->GetTrayAudio()->ShowPopUpVolumeView();
+    }
   }
 }
 
