@@ -71,6 +71,13 @@ extern const SkColor kHeaderBackgroundColor;
 
 extern const SkColor kHeaderTextColorNormal;
 
+// Constants for the title row.
+constexpr int kTitleRowVerticalPadding = 4;
+constexpr int kTitleRowProgressBarHeight = 2;
+constexpr int kTitleRowPaddingTop = kTitleRowVerticalPadding;
+constexpr int kTitleRowPaddingBottom =
+    kTitleRowVerticalPadding - kTitleRowProgressBarHeight;
+
 extern const SkColor kMobileNotConnectedXIconColor;
 
 // Extra padding used to adjust hitting region around tray items.
@@ -127,25 +134,73 @@ extern const int kTrayPopupSystemInfoRowHeight;
 
 // The colors used when --enable-features=SystemTrayUnified flag is enabled.
 constexpr SkColor kUnifiedMenuBackgroundColor = SkColorSetRGB(0x20, 0x21, 0x24);
-constexpr SkColor kUnifiedMenuTextColor = SkColorSetRGB(0xf1, 0xf2, 0xf3);
-constexpr SkColor kUnifiedMenuIconColor = SkColorSetRGB(0xf1, 0xf2, 0xf3);
+constexpr SkColor kUnifiedMenuBackgroundColorWithBlur =
+    SkColorSetA(kUnifiedMenuBackgroundColor, 0xB3);
+constexpr float kUnifiedMenuBackgroundBlur = 30.f;
+constexpr SkColor kUnifiedMenuTextColor = SkColorSetRGB(0xf1, 0xf3, 0xf4);
+constexpr SkColor kUnifiedMenuIconColor = SkColorSetRGB(0xf1, 0xf3, 0xf4);
+constexpr SkColor kUnifiedMenuSecondaryTextColor =
+    SkColorSetA(kUnifiedMenuIconColor, 0xa3);
 constexpr SkColor kUnifiedMenuIconColorDisabled =
     SkColorSetA(kUnifiedMenuIconColor, 0xa3);
 constexpr SkColor kUnifiedMenuButtonColor =
     SkColorSetA(kUnifiedMenuIconColor, 0x14);
+constexpr SkColor kUnifiedMenuSeparatorColor =
+    SkColorSetA(kUnifiedMenuIconColor, 0x23);
 constexpr SkColor kUnifiedMenuButtonColorActive =
     SkColorSetRGB(0x25, 0x81, 0xdf);
+constexpr SkColor kUnifiedNotificationSeparatorColor =
+    SkColorSetRGB(0xdf, 0xe0, 0xe0);
+constexpr SkColor kUnifiedFeaturePodHoverColor =
+    SkColorSetRGB(0xff, 0xff, 0xff);
 
+constexpr gfx::Insets kUnifiedMenuItemPadding(0, 16, 16, 16);
+constexpr gfx::Insets kUnifiedSliderPadding(0, 16);
+
+constexpr int kUnifiedNotificationCenterSpacing = 16;
+constexpr int kUnifiedTrayCornerRadius = 20;
 constexpr int kUnifiedTopShortcutSpacing = 16;
 constexpr gfx::Insets kUnifiedTopShortcutPadding(0, 16);
 
+constexpr int kUnifiedSystemInfoHeight = 16;
+constexpr int kUnifiedSystemInfoSpacing = 8;
+
 // Constants used in FeaturePodsView of UnifiedSystemTray.
-constexpr int kUnifiedFeaturePodIconSize = 48;
-constexpr gfx::Size kUnifiedFeaturePodSize(64, 88);
+constexpr gfx::Size kUnifiedFeaturePodIconSize(48, 48);
+constexpr gfx::Size kUnifiedFeaturePodSize(84, 88);
+constexpr gfx::Size kUnifiedFeaturePodCollapsedSize(48, 48);
+constexpr gfx::Insets kUnifiedFeaturePodIconPadding(4);
+constexpr gfx::Insets kUnifiedFeaturePodHoverPadding(2);
+constexpr int kUnifiedFeaturePodSpacing = 6;
+constexpr int kUnifiedFeaturePodHoverRadius = 4;
 constexpr int kUnifiedFeaturePodVerticalPadding = 28;
-constexpr int kUnifiedFeaturePodHorizontalSidePadding = 52;
-constexpr int kUnifiedFeaturePodHorizontalMiddlePadding = 32;
+constexpr int kUnifiedFeaturePodHorizontalSidePadding = 26;
+constexpr int kUnifiedFeaturePodHorizontalMiddlePadding = 28;
+constexpr int kUnifiedFeaturePodCollapsedVerticalPadding = 16;
+constexpr int kUnifiedFeaturePodCollapsedHorizontalPadding = 24;
+constexpr int kUnifiedFeaturePodArrowSpacing = 4;
 constexpr int kUnifiedFeaturePodItemsInRow = 3;
+constexpr int kUnifiedFeaturePodMaxItemsInCollapsed = 5;
+constexpr int kUnifiedNotificationSeparatorThickness = 1;
+
+// Separators between multiple users are shorter than the full width.
+constexpr int kUnifiedUserChooserSeparatorSideMargin = 64;
+// Additional gap above and below the longer separator between user list and
+// "Sign in another user..." button.
+constexpr int kUnifiedUserChooserLargeSeparatorVerticalSpacing = 8;
+//
+constexpr int kUnifiedUserChooserRowHeight = 64;
+constexpr int kUnifiedUserChooserAvatorIconColumnWidth = 64;
+constexpr int kUnifiedUserChooserCloseIconColumnWidth = 64;
+
+// Gap between the buttons on the top shortcut row, other than the
+// expand/collapse button.
+constexpr int kUnifiedTopShortcutButtonDefaultSpacing = 16;
+constexpr int kUnifiedTopShortcutButtonMinSpacing = 4;
+
+// Constants used in the title row of a detailed view in UnifiedSystemTray.
+constexpr gfx::Insets kUnifiedDetailedViewTitlePadding(0, 0, 0, 16);
+constexpr int kUnifiedDetailedViewTitleRowHeight = 64;
 
 }  // namespace ash
 
