@@ -5,6 +5,7 @@
 #ifndef ASH_SYSTEM_UNIFIED_QUIET_MODE_FEATURE_POD_CONTROLLER_H_
 #define ASH_SYSTEM_UNIFIED_QUIET_MODE_FEATURE_POD_CONTROLLER_H_
 
+#include "ash/ash_export.h"
 #include "ash/system/unified/feature_pod_controller_base.h"
 #include "base/macros.h"
 #include "base/strings/string16.h"
@@ -12,24 +13,31 @@
 
 namespace ash {
 
+class UnifiedSystemTrayController;
+
 // Controller of a feature pod button that toggles do-not-disturb mode.
 // If the do-not-disturb mode is enabled, the button indicates it by bright
 // background color and different icon.
-class QuietModeFeaturePodController
+class ASH_EXPORT QuietModeFeaturePodController
     : public FeaturePodControllerBase,
       public message_center::MessageCenterObserver {
  public:
-  QuietModeFeaturePodController();
+  explicit QuietModeFeaturePodController(
+      UnifiedSystemTrayController* tray_controller);
   ~QuietModeFeaturePodController() override;
 
   // FeaturePodControllerBase:
   FeaturePodButton* CreateButton() override;
-  void OnPressed() override;
+  void OnIconPressed() override;
+  void OnLabelPressed() override;
+  SystemTrayItemUmaType GetUmaType() const override;
 
   // message_center::MessageCenterObserver:
   void OnQuietModeChanged(bool in_quiet_mode) override;
 
  private:
+  UnifiedSystemTrayController* const tray_controller_;
+
   FeaturePodButton* button_ = nullptr;
 
   DISALLOW_COPY_AND_ASSIGN(QuietModeFeaturePodController);
