@@ -20,8 +20,10 @@
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
 #include "ash/session/test_session_controller_client.h"
+#include "ash/shelf/shelf.h"
 #include "ash/shell.h"
 #include "ash/shell/toplevel_window.h"
+#include "ash/system/status_area_widget.h"
 #include "ash/test/ash_test_environment.h"
 #include "ash/test/ash_test_helper.h"
 #include "ash/test_screenshot_delegate.h"
@@ -31,12 +33,18 @@
 #include "ash/window_manager_service.h"
 #include "ash/wm/top_level_window_factory.h"
 #include "ash/wm/window_positioner.h"
+#include "ash/ws/window_service_owner.h"
 #include "base/memory/ptr_util.h"
-#include "components/signin/core/account_id/account_id.h"
+#include "components/account_id/account_id.h"
 #include "components/user_manager/user_names.h"
+#include "mojo/public/cpp/bindings/map.h"
 #include "services/ui/public/cpp/property_type_converters.h"
 #include "services/ui/public/interfaces/window_manager.mojom.h"
-#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
+#include "services/ui/public/interfaces/window_tree_constants.mojom.h"
+#include "services/ui/ws2/test_window_tree_client.h"
+#include "services/ui/ws2/window_service.h"
+#include "services/ui/ws2/window_tree.h"
+#include "services/ui/ws2/window_tree_test_helper.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/screen_position_client.h"
 #include "ui/aura/client/window_parenting_client.h"
@@ -180,6 +188,11 @@ void AshTestBase::TearDown() {
   teardown_called_ = true;
   Shell::Get()->session_controller()->NotifyChromeTerminating();
 
+  // These depend upon WindowService, which is owned by Shell, so they must
+  // be destroyed before the Shell (owned by AshTestHelper).
+  window_tree_test_helper_.reset();
+  window_tree_.reset();
+
   // Flush the message loop to finish pending release tasks.
   RunAllPendingInMessageLoop();
 
@@ -201,6 +214,11 @@ SystemTray* AshTestBase::GetPrimarySystemTray() {
   return Shell::Get()->GetPrimarySystemTray();
 }
 
+// static
+UnifiedSystemTray* AshTestBase::GetPrimaryUnifiedSystemTray() {
+  return GetPrimaryShelf()->GetStatusAreaWidget()->unified_system_tray();
+}
+
 ui::test::EventGenerator& AshTestBase::GetEventGenerator() {
   if (!event_generator_) {
     event_generator_.reset(
@@ -228,7 +246,6 @@ void AshTestBase::UpdateDisplay(const std::string& display_specs) {
   ScreenOrientationControllerTestApi(
       Shell::Get()->screen_orientation_controller())
       .UpdateNaturalOrientation();
-  ash_test_helper_->NotifyClientAboutAcceleratedWidgets();
 }
 
 aura::Window* AshTestBase::CurrentContext() {
@@ -255,14 +272,7 @@ std::unique_ptr<aura::Window> AshTestBase::CreateTestWindow(
     const gfx::Rect& bounds_in_screen,
     aura::client::WindowType type,
     int shell_window_id) {
-  if (AshTestHelper::config() != Config::MASH) {
-    return base::WrapUnique<aura::Window>(
-        CreateTestWindowInShellWithDelegateAndType(
-            nullptr, type, shell_window_id, bounds_in_screen));
-  }
-
-  // For mash route creation through the window manager. This better simulates
-  // what happens when a client creates a top level window.
+  // The following simulates what happens when a client creates a window.
   std::map<std::string, std::vector<uint8_t>> properties;
   if (!bounds_in_screen.IsEmpty()) {
     properties[ui::mojom::WindowManager::kBounds_InitProperty] =
@@ -278,13 +288,23 @@ std::unique_ptr<aura::Window> AshTestBase::CreateTestWindow(
 
   const ui::mojom::WindowType mus_window_type =
       MusWindowTypeFromWindowType(type);
-  WindowManager* window_manager =
-      ash_test_helper_->window_manager_service()->window_manager();
-  aura::Window* window = CreateAndParentTopLevelWindow(
-      window_manager, mus_window_type, &properties);
+  if (AshTestHelper::config() == Config::MASH) {
+    // For mash route creation through the window manager. This better simulates
+    // what happens when a client creates a top level window.
+    return CreateTestWindowMash(mus_window_type, shell_window_id, &properties);
+  }
+
+  properties[ui::mojom::WindowManager::kWindowType_InitProperty] =
+      mojo::ConvertTo<std::vector<uint8_t>>(
+          static_cast<int32_t>(mus_window_type));
+
+  // WindowTreeTestHelper maps 0 to a unique id.
+  std::unique_ptr<aura::Window> window(
+      GetWindowTreeTestHelper()->NewTopLevelWindow(
+          mojo::MapToFlatMap(std::move(properties))));
   window->set_id(shell_window_id);
   window->Show();
-  return base::WrapUnique<aura::Window>(window);
+  return window;
 }
 
 std::unique_ptr<aura::Window> AshTestBase::CreateToplevelTestWindow(
@@ -380,7 +400,7 @@ void AshTestBase::ParentWindowInPrimaryRootWindow(aura::Window* window) {
 }
 
 void AshTestBase::RunAllPendingInMessageLoop() {
-  ash_test_helper_->RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 }
 
 TestScreenshotDelegate* AshTestBase::GetScreenshotDelegate() {
@@ -409,6 +429,18 @@ void AshTestBase::SimulateUserLogin(const std::string& user_email) {
   session_controller_client->SetSessionState(SessionState::ACTIVE);
 }
 
+void AshTestBase::SimulateNewUserFirstLogin(const std::string& user_email) {
+  TestSessionControllerClient* const session_controller_client =
+      GetSessionControllerClient();
+  session_controller_client->AddUserSession(
+      user_email, user_manager::USER_TYPE_REGULAR, true /* enable_settings */,
+      true /* provide_pref_service */, true /* is_new_profile */);
+  session_controller_client->SwitchActiveUser(
+      AccountId::FromUserEmail(user_email));
+  session_controller_client->SetSessionState(
+      session_manager::SessionState::ACTIVE);
+}
+
 void AshTestBase::SimulateGuestLogin() {
   const std::string guest = user_manager::kGuestUserName;
   TestSessionControllerClient* session = GetSessionControllerClient();
@@ -511,6 +543,49 @@ display::Display AshTestBase::GetSecondaryDisplay() {
   return ash_test_helper_->GetSecondaryDisplay();
 }
 
+ui::ws2::WindowTreeTestHelper* AshTestBase::GetWindowTreeTestHelper() {
+  CreateWindowTreeIfNecessary();
+  return window_tree_test_helper_.get();
+}
+
+ui::ws2::TestWindowTreeClient* AshTestBase::GetTestWindowTreeClient() {
+  CreateWindowTreeIfNecessary();
+  return window_tree_client_.get();
+}
+
+ui::ws2::WindowTree* AshTestBase::GetWindowTree() {
+  CreateWindowTreeIfNecessary();
+  return window_tree_.get();
+}
+
+std::unique_ptr<aura::Window> AshTestBase::CreateTestWindowMash(
+    ui::mojom::WindowType window_type,
+    int shell_window_id,
+    std::map<std::string, std::vector<uint8_t>>* properties) {
+  WindowManager* window_manager =
+      ash_test_helper_->window_manager_service()->window_manager();
+  aura::Window* window = CreateAndParentTopLevelWindow(
+      window_manager, window_type, window_manager->property_converter(),
+      properties);
+  window->set_id(shell_window_id);
+  window->Show();
+  return base::WrapUnique<aura::Window>(window);
+}
+
+void AshTestBase::CreateWindowTreeIfNecessary() {
+  if (window_tree_client_)
+    return;
+
+  // Lazily create a single client.
+  window_tree_client_ = std::make_unique<ui::ws2::TestWindowTreeClient>();
+  window_tree_ =
+      Shell::Get()->window_service_owner()->window_service()->CreateWindowTree(
+          window_tree_client_.get());
+  window_tree_->InitFromFactory();
+  window_tree_test_helper_ =
+      std::make_unique<ui::ws2::WindowTreeTestHelper>(window_tree_.get());
+}
+
 void AshTestBase::OnWindowInitialized(aura::Window* window) {}
 
 void AshTestBase::OnHostInitialized(aura::WindowTreeHost* host) {
