@@ -12,7 +12,6 @@
 
 #include "ash/public/cpp/shell_window_ids.h"
 #include "base/macros.h"
-#include "base/message_loop/message_loop.h"
 #include "base/threading/thread.h"
 #include "components/user_manager/user_type.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -41,9 +40,17 @@ class Rect;
 }
 
 namespace ui {
+namespace mojom {
+enum class WindowType;
+}
 namespace test {
 class EventGenerator;
 }
+namespace ws2 {
+class TestWindowTreeClient;
+class WindowTree;
+class WindowTreeTestHelper;
+}  // namespace ws2
 }
 
 namespace views {
@@ -60,6 +67,7 @@ class Shelf;
 class SystemTray;
 class TestScreenshotDelegate;
 class TestSessionControllerClient;
+class UnifiedSystemTray;
 
 class AshTestBase : public testing::Test,
                     public aura::EnvObserver,
@@ -78,6 +86,9 @@ class AshTestBase : public testing::Test,
   // Returns the system tray on the primary display.
   static SystemTray* GetPrimarySystemTray();
 
+  // Returns the unified system tray on the primary display.
+  static UnifiedSystemTray* GetPrimaryUnifiedSystemTray();
+
   // Update the display configuration as given in |display_specs|.
   // See ash::DisplayManagerTestApi::UpdateDisplay for more details.
   void UpdateDisplay(const std::string& display_specs);
@@ -99,15 +110,19 @@ class AshTestBase : public testing::Test,
   // window, otherwise the window is added to the display matching
   // |bounds_in_screen|. |shell_window_id| is the shell window id to give to
   // the new window.
+  //
+  // In classic mode this simulates creating a window as a client of Ash would.
+  // That is, it goes through the WindowService.
+  //
   // TODO(sky): convert existing CreateTestWindow() functions into this one.
   std::unique_ptr<aura::Window> CreateTestWindow(
       const gfx::Rect& bounds_in_screen = gfx::Rect(),
       aura::client::WindowType type = aura::client::WINDOW_TYPE_NORMAL,
       int shell_window_id = kShellWindowId_Invalid);
 
-  // Creates a visible top-level window. For Config::CLASSIC and Config::MUS
-  // this creates a Window with a delegate. For Config::MASH this creates a
-  // window as if the client requested a top-level window.
+  // Creates a visible top-level window. For Config::CLASSIC this creates a
+  // Window with a delegate. For Config::MASH this creates a window as if the
+  // client requested a top-level window.
   std::unique_ptr<aura::Window> CreateToplevelTestWindow(
       const gfx::Rect& bounds_in_screen = gfx::Rect(),
       int shell_window_id = kShellWindowId_Invalid);
@@ -173,6 +188,8 @@ class AshTestBase : public testing::Test,
 
   AshTestHelper* ash_test_helper() { return ash_test_helper_.get(); }
 
+  // Deprecated. Using this triggers a preprocessor warning, use
+  // base::RunLoop().RunUntilIdle() instead.
   void RunAllPendingInMessageLoop();
 
   TestScreenshotDelegate* GetScreenshotDelegate();
@@ -189,6 +206,9 @@ class AshTestBase : public testing::Test,
   // existing user sessions and makes it the active user session.
   void SimulateUserLogin(const std::string& user_email);
 
+  // Simular to SimulateUserLogin but for a newly created user first ever login.
+  void SimulateNewUserFirstLogin(const std::string& user_email);
+
   // Similar to SimulateUserLogin but for a guest user.
   void SimulateGuestLogin();
 
@@ -220,7 +240,19 @@ class AshTestBase : public testing::Test,
   display::Display GetPrimaryDisplay();
   display::Display GetSecondaryDisplay();
 
+  // Returns the WindowTreeTestHelper, creating if necessary.
+  ui::ws2::WindowTreeTestHelper* GetWindowTreeTestHelper();
+  ui::ws2::TestWindowTreeClient* GetTestWindowTreeClient();
+  ui::ws2::WindowTree* GetWindowTree();
+
  private:
+  std::unique_ptr<aura::Window> CreateTestWindowMash(
+      ui::mojom::WindowType window_type,
+      int shell_window_id,
+      std::map<std::string, std::vector<uint8_t>>* properties);
+
+  void CreateWindowTreeIfNecessary();
+
   // aura::EnvObserver:
   void OnWindowInitialized(aura::Window* window) override;
   void OnHostInitialized(aura::WindowTreeHost* host) override;
@@ -239,6 +271,10 @@ class AshTestBase : public testing::Test,
   std::unique_ptr<AshTestHelper> ash_test_helper_;
   std::unique_ptr<ui::test::EventGenerator> event_generator_;
 
+  std::unique_ptr<ui::ws2::TestWindowTreeClient> window_tree_client_;
+  std::unique_ptr<ui::ws2::WindowTree> window_tree_;
+  std::unique_ptr<ui::ws2::WindowTreeTestHelper> window_tree_test_helper_;
+
   DISALLOW_COPY_AND_ASSIGN(AshTestBase);
 };
 
