@@ -8,17 +8,15 @@
 #include <stdint.h>
 
 #include <memory>
-#include <string>
 #include <utility>
-#include <vector>
 
-#include "ash/app_list/test/app_list_test_helper.h"
 #include "ash/session/test_session_controller_client.h"
-#include "base/compiler_specific.h"
 #include "base/macros.h"
 #include "base/test/scoped_command_line.h"
 #include "ui/aura/test/mus/test_window_tree_client_setup.h"
 
+class PrefService;
+
 namespace aura {
 class Window;
 class WindowTreeClientPrivate;
@@ -39,7 +37,6 @@ class MashTestSuite;
 
 namespace ui {
 class ScopedAnimationDurationScaleMode;
-class InputDeviceClient;
 }
 
 namespace wm {
@@ -48,11 +45,11 @@ class WMState;
 
 namespace ash {
 
+class AppListTestHelper;
 class AshTestEnvironment;
 class AshTestViewsDelegate;
-class RootWindowController;
+class TestConnector;
 class TestShellDelegate;
-class TestSessionControllerClient;
 class WindowManagerService;
 
 enum class Config;
@@ -64,9 +61,6 @@ class AshTestHelper {
   explicit AshTestHelper(AshTestEnvironment* ash_test_environment);
   ~AshTestHelper();
 
-  // This is intended to be called from TestSuites, not individual configs.
-  static void set_config(Config config) { config_ = config; }
-
   // Returns the configuration that tests are run in. See ash::Config enum for
   // details.
   static Config config() { return config_; }
@@ -85,9 +79,7 @@ class AshTestHelper {
   // primary root Window.
   aura::Window* CurrentContext();
 
-  void RunAllPendingInMessageLoop();
-
-  void NotifyClientAboutAcceleratedWidgets();
+  PrefService* GetLocalStatePrefService();
 
   TestShellDelegate* test_shell_delegate() { return test_shell_delegate_; }
   void set_test_shell_delegate(TestShellDelegate* test_shell_delegate) {
@@ -128,48 +120,46 @@ class AshTestHelper {
   friend class AshTestSuite;
   friend class mash::test::MashTestSuite;
 
+  // Forces creation of the WindowService. The WindowService is normally created
+  // on demand, this force the creation.
+  void CreateWindowService();
+
   // Called when running in mash to create the WindowManager.
   void CreateMashWindowManager();
 
   // Called when running in ash to create Shell.
   void CreateShell();
 
-  // Creates a new RootWindowController based on |display_spec|. The origin is
-  // set to |next_x| and on exit |next_x| is set to the origin + the width.
-  RootWindowController* CreateRootWindowController(
-      const std::string& display_spec,
-      int* next_x);
-
   static Config config_;
 
   std::unique_ptr<aura::test::EnvWindowTreeClientSetter>
       env_window_tree_client_setter_;
   AshTestEnvironment* ash_test_environment_;  // Not owned.
-  TestShellDelegate* test_shell_delegate_;  // Owned by ash::Shell.
+  TestShellDelegate* test_shell_delegate_ = nullptr;  // Owned by ash::Shell.
   std::unique_ptr<ui::ScopedAnimationDurationScaleMode> zero_duration_mode_;
 
   std::unique_ptr<::wm::WMState> wm_state_;
   std::unique_ptr<AshTestViewsDelegate> test_views_delegate_;
 
   // Check if DBus Thread Manager was initialized here.
-  bool dbus_thread_manager_initialized_;
+  bool dbus_thread_manager_initialized_ = false;
   // Check if Bluez DBus Manager was initialized here.
-  bool bluez_dbus_manager_initialized_;
+  bool bluez_dbus_manager_initialized_ = false;
+  // Check if PowerPolicyController was initialized here.
+  bool power_policy_controller_initialized_ = false;
 
   aura::TestWindowTreeClientSetup window_tree_client_setup_;
   std::unique_ptr<WindowManagerService> window_manager_service_;
   std::unique_ptr<aura::WindowTreeClientPrivate> window_tree_client_private_;
-  // Id for the next Display created by CreateRootWindowController().
-  int64_t next_display_id_ = 1;
 
   std::unique_ptr<TestSessionControllerClient> session_controller_client_;
 
-  std::unique_ptr<ui::InputDeviceClient> input_device_client_;
-
   std::unique_ptr<base::test::ScopedCommandLine> command_line_;
 
   std::unique_ptr<AppListTestHelper> app_list_test_helper_;
 
+  std::unique_ptr<TestConnector> test_connector_;
+
   DISALLOW_COPY_AND_ASSIGN(AshTestHelper);
 };
 
