@@ -21,8 +21,8 @@
 #include "components/session_manager/session_manager_types.h"
 #include "services/service_manager/public/cpp/service_test.h"
 #include "services/ui/public/cpp/property_type_converters.h"
-#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
 #include "services/ui/public/interfaces/window_tree.mojom.h"
+#include "services/ui/public/interfaces/window_tree_constants.mojom.h"
 #include "ui/aura/env.h"
 #include "ui/aura/mus/property_converter.h"
 #include "ui/aura/mus/window_tree_client.h"
@@ -59,6 +59,7 @@ class WindowTreeClientDelegate : public aura::WindowTreeClientDelegate {
       aura::WindowTreeHostMus* window_tree_host) override {}
   void OnLostConnection(aura::WindowTreeClient* client) override {}
   void OnPointerEventObserved(const ui::PointerEvent& event,
+                              int64_t display_id,
                               aura::Window* target) override {}
   aura::PropertyConverter* GetPropertyConverter() override {
     return &property_converter_;
@@ -92,7 +93,12 @@ void OnEmbed(bool success) {
   ASSERT_TRUE(success);
 }
 
-TEST_F(WindowManagerServiceTest, OpenWindow) {
+#if defined(ADDRESS_SANITIZER)
+#define MAYBE_OpenWindow DISABLED_OpenWindow
+#else
+#define MAYBE_OpenWindow OpenWindow
+#endif
+TEST_F(WindowManagerServiceTest, MAYBE_OpenWindow) {
   display::ScreenBase screen;
   screen.display_list().AddDisplay(
       display::Display(1, gfx::Rect(0, 0, 200, 200)),
