@@ -13,8 +13,8 @@
 #include "base/memory/ptr_util.h"
 #include "ui/keyboard/keyboard_controller.h"
 #include "ui/keyboard/keyboard_switches.h"
-#include "ui/keyboard/keyboard_test_util.h"
 #include "ui/keyboard/keyboard_ui.h"
+#include "ui/keyboard/test/keyboard_test_util.h"
 
 namespace ash {
 
@@ -56,8 +56,7 @@ class TestLayoutManager : public WorkspaceLayoutManager {
 // Verifies that the always on top controller is notified of keyboard bounds
 // changing events.
 TEST_F(VirtualKeyboardAlwaysOnTopControllerTest, NotifyKeyboardBoundsChanged) {
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
+  auto* keyboard_controller = keyboard::KeyboardController::Get();
   aura::Window* root_window = Shell::GetPrimaryRootWindow();
   aura::Window* always_on_top_container =
       Shell::GetContainer(root_window, kShellWindowId_AlwaysOnTopContainer);
@@ -73,9 +72,6 @@ TEST_F(VirtualKeyboardAlwaysOnTopControllerTest, NotifyKeyboardBoundsChanged) {
   controller->ActivateKeyboard(keyboard_controller);
 
   // Mock a keyboard appearing.
-  aura::Window* keyboard_container = keyboard_controller->GetContainerWindow();
-  ASSERT_TRUE(keyboard_container);
-  keyboard_container->Show();
   aura::Window* contents_window =
       keyboard_controller->ui()->GetContentsWindow();
   const int kKeyboardHeight = 200;
