@@ -14,6 +14,7 @@
 #include "base/macros.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/views/widget/widget.h"
+#include "ui/views/widget/widget_delegate.h"
 #include "ui/wm/core/window_util.h"
 
 namespace ash {
@@ -53,7 +54,7 @@ class TestClientControlledStateDelegate
 
   const gfx::Rect& requested_bounds() const { return requested_bounds_; }
 
-  void reset() {
+  void Reset() {
     old_state_ = mojom::WindowStateType::DEFAULT;
     new_state_ = mojom::WindowStateType::DEFAULT;
     requested_bounds_.SetRect(0, 0, 0, 0);
@@ -70,6 +71,26 @@ class TestClientControlledStateDelegate
   DISALLOW_COPY_AND_ASSIGN(TestClientControlledStateDelegate);
 };
 
+class TestWidgetDelegate : public views::WidgetDelegateView {
+ public:
+  TestWidgetDelegate() = default;
+  ~TestWidgetDelegate() override = default;
+
+  // views::WidgetDelegateView:
+  bool CanResize() const override { return can_snap_; }
+  bool CanMaximize() const override { return can_snap_; }
+
+  void EnableSnap() {
+    can_snap_ = true;
+    GetWidget()->OnSizeConstraintsChanged();
+  }
+
+ private:
+  bool can_snap_ = false;
+
+  DISALLOW_COPY_AND_ASSIGN(TestWidgetDelegate);
+};
+
 }  // namespace
 
 class ClientControlledStateTest : public AshTestBase {
@@ -80,17 +101,21 @@ class ClientControlledStateTest : public AshTestBase {
   void SetUp() override {
     AshTestBase::SetUp();
 
-    widget_ = std::make_unique<views::Widget>();
+    widget_delegate_ = new TestWidgetDelegate();
+
     views::Widget::InitParams params;
     params.ownership = views::Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
     params.parent = Shell::GetPrimaryRootWindow()->GetChildById(
         kShellWindowId_DefaultContainer);
     params.bounds = kInitialBounds;
+    params.delegate = widget_delegate_;
+
+    widget_ = std::make_unique<views::Widget>();
     widget_->Init(params);
     wm::WindowState* window_state = wm::GetWindowState(window());
     window_state->set_allow_set_bounds_direct(true);
     auto delegate = std::make_unique<TestClientControlledStateDelegate>();
-    delegate_ = delegate.get();
+    state_delegate_ = delegate.get();
     auto state = std::make_unique<ClientControlledState>(std::move(delegate));
     state_ = state.get();
     window_state->SetStateObject(std::move(state));
@@ -102,11 +127,13 @@ class ClientControlledStateTest : public AshTestBase {
     AshTestBase::TearDown();
   }
 
+  TestWidgetDelegate* widget_delegate() { return widget_delegate_; }
+
  protected:
   aura::Window* window() { return widget_->GetNativeWindow(); }
   WindowState* window_state() { return GetWindowState(window()); }
   ClientControlledState* state() { return state_; }
-  TestClientControlledStateDelegate* delegate() { return delegate_; }
+  TestClientControlledStateDelegate* delegate() { return state_delegate_; }
   views::Widget* widget() { return widget_.get(); }
   ScreenPinningController* GetScreenPinningController() {
     return Shell::Get()->screen_pinning_controller();
@@ -114,7 +141,8 @@ class ClientControlledStateTest : public AshTestBase {
 
  private:
   ClientControlledState* state_ = nullptr;
-  TestClientControlledStateDelegate* delegate_ = nullptr;
+  TestClientControlledStateDelegate* state_delegate_ = nullptr;
+  TestWidgetDelegate* widget_delegate_ = nullptr;  // owned by itself.
   std::unique_ptr<views::Widget> widget_;
 
   DISALLOW_COPY_AND_ASSIGN(ClientControlledStateTest);
@@ -138,6 +166,10 @@ TEST_F(ClientControlledStateTest, Maximize) {
   // Bounds is controlled by client.
   EXPECT_EQ(kInitialBounds, widget()->GetWindowBoundsInScreen());
 
+  // Bounds can't be changed in maximized state.
+  widget()->SetBounds(gfx::Rect(0, 0, 100, 100));
+  EXPECT_EQ(gfx::Rect(), delegate()->requested_bounds());
+
   widget()->Restore();
   EXPECT_TRUE(widget()->IsMaximized());
   EXPECT_EQ(kInitialBounds, widget()->GetWindowBoundsInScreen());
@@ -263,7 +295,7 @@ TEST_F(ClientControlledStateTest, IgnoreWorkspace) {
   state()->EnterNextState(window_state(), delegate()->new_state(),
                           ClientControlledState::kAnimationNone);
   EXPECT_TRUE(widget()->IsMaximized());
-  delegate()->reset();
+  delegate()->Reset();
 
   UpdateDisplay("1000x800");
 
@@ -286,6 +318,61 @@ TEST_F(ClientControlledStateTest, SetBounds) {
   EXPECT_EQ(new_bounds, widget()->GetWindowBoundsInScreen());
 }
 
+TEST_F(ClientControlledStateTest, CenterWindow) {
+  display::Screen* screen = display::Screen::GetScreen();
+  gfx::Rect bounds = screen->GetPrimaryDisplay().work_area();
+
+  const WMEvent center_event(WM_EVENT_CENTER);
+  window_state()->OnWMEvent(&center_event);
+  EXPECT_NEAR(bounds.CenterPoint().x(),
+              delegate()->requested_bounds().CenterPoint().x(), 1);
+  EXPECT_NEAR(bounds.CenterPoint().y(),
+              delegate()->requested_bounds().CenterPoint().y(), 1);
+}
+
+TEST_F(ClientControlledStateTest, SnapWindow) {
+  // Snap disabled.
+  display::Screen* screen = display::Screen::GetScreen();
+  gfx::Rect work_area = screen->GetPrimaryDisplay().work_area();
+  ASSERT_FALSE(window_state()->CanResize());
+  ASSERT_FALSE(window_state()->CanSnap());
+
+  // The event should be ignored.
+  const WMEvent snap_left_event(WM_EVENT_CYCLE_SNAP_LEFT);
+  window_state()->OnWMEvent(&snap_left_event);
+  EXPECT_FALSE(window_state()->IsSnapped());
+  EXPECT_TRUE(delegate()->requested_bounds().IsEmpty());
+
+  const WMEvent snap_right_event(WM_EVENT_CYCLE_SNAP_RIGHT);
+  window_state()->OnWMEvent(&snap_right_event);
+  EXPECT_FALSE(window_state()->IsSnapped());
+  EXPECT_TRUE(delegate()->requested_bounds().IsEmpty());
+
+  // Snap enabled.
+  widget_delegate()->EnableSnap();
+  ASSERT_TRUE(window_state()->CanResize());
+  ASSERT_TRUE(window_state()->CanSnap());
+
+  window_state()->OnWMEvent(&snap_left_event);
+  EXPECT_NEAR(work_area.CenterPoint().x(),
+              delegate()->requested_bounds().right(), 1);
+  EXPECT_EQ(work_area.height(), delegate()->requested_bounds().height());
+  EXPECT_TRUE(delegate()->requested_bounds().origin().IsOrigin());
+  EXPECT_EQ(mojom::WindowStateType::DEFAULT, delegate()->old_state());
+  EXPECT_EQ(mojom::WindowStateType::LEFT_SNAPPED, delegate()->new_state());
+
+  delegate()->Reset();
+
+  window_state()->OnWMEvent(&snap_right_event);
+  EXPECT_NEAR(work_area.CenterPoint().x(), delegate()->requested_bounds().x(),
+              1);
+  EXPECT_EQ(work_area.height(), delegate()->requested_bounds().height());
+  EXPECT_EQ(work_area.bottom_right(),
+            delegate()->requested_bounds().bottom_right());
+  EXPECT_EQ(mojom::WindowStateType::DEFAULT, delegate()->old_state());
+  EXPECT_EQ(mojom::WindowStateType::RIGHT_SNAPPED, delegate()->new_state());
+}
+
 // Pin events should be applied immediately.
 TEST_F(ClientControlledStateTest, Pinned) {
   ASSERT_FALSE(window_state()->IsPinned());
