@@ -7,6 +7,8 @@
 #include <memory>
 #include <utility>
 
+#include "ash/accessibility/accessibility_controller.h"
+#include "ash/accessibility/test_accessibility_controller_client.h"
 #include "ash/public/cpp/ash_switches.h"
 #include "ash/public/cpp/config.h"
 #include "ash/session/session_controller.h"
@@ -14,6 +16,7 @@
 #include "ash/shutdown_controller.h"
 #include "ash/shutdown_reason.h"
 #include "ash/system/power/power_button_controller.h"
+#include "ash/system/power/power_button_controller_test_api.h"
 #include "ash/system/power/power_button_test_base.h"
 #include "ash/touch/touch_devices_controller.h"
 #include "ash/wm/lock_state_controller_test_api.h"
@@ -24,7 +27,7 @@
 #include "base/time/time.h"
 #include "chromeos/dbus/fake_power_manager_client.h"
 #include "chromeos/dbus/fake_session_manager_client.h"
-#include "ui/display/manager/chromeos/display_configurator.h"
+#include "ui/display/manager/display_configurator.h"
 #include "ui/display/manager/fake_display_snapshot.h"
 #include "ui/display/types/display_constants.h"
 #include "ui/gfx/geometry/size.h"
@@ -32,6 +35,11 @@
 namespace ash {
 namespace {
 
+// Shorthand for some long constants.
+constexpr power_manager::BacklightBrightnessChange_Cause kUserCause =
+    power_manager::BacklightBrightnessChange_Cause_USER_REQUEST;
+constexpr power_manager::BacklightBrightnessChange_Cause kOtherCause =
+    power_manager::BacklightBrightnessChange_Cause_OTHER;
 bool cursor_visible() {
   return Shell::Get()->cursor_manager()->IsCursorVisible();
 }
@@ -75,6 +83,9 @@ class LockStateControllerTest : public PowerButtonTestBase {
     test_animator_ = new TestSessionStateAnimator;
     lock_state_controller_->set_animator_for_test(test_animator_);
     lock_state_test_api_->set_shutdown_controller(&test_shutdown_controller_);
+
+    a11y_controller_ = Shell::Get()->accessibility_controller();
+    a11y_controller_->SetClient(a11y_client_.CreateInterfacePtrAndBind());
   }
 
  protected:
@@ -91,6 +102,15 @@ class LockStateControllerTest : public PowerButtonTestBase {
     test_animator_->Advance(test_animator_->GetDuration(speed) * factor);
   }
 
+  void SendBrightnessChange(
+      double percent,
+      power_manager::BacklightBrightnessChange_Cause cause) {
+    power_manager::BacklightBrightnessChange change;
+    change.set_percent(percent);
+    change.set_cause(cause);
+    power_manager_client_->SendScreenBrightnessChanged(change);
+  }
+
   void ExpectPreLockAnimationStarted() {
     SCOPED_TRACE("Failure in ExpectPreLockAnimationStarted");
     EXPECT_LT(0u, test_animator_->GetAnimationCount());
@@ -198,6 +218,22 @@ class LockStateControllerTest : public PowerButtonTestBase {
         SessionStateAnimator::ANIMATION_GRAYSCALE_BRIGHTNESS));
   }
 
+  void ExpectShutdownAnimationFinished() {
+    SCOPED_TRACE("Failure in ExpectShutdownAnimationFinished");
+    EXPECT_EQ(0u, test_animator_->GetAnimationCount());
+    EXPECT_FALSE(test_animator_->AreContainersAnimated(
+        SessionStateAnimator::ROOT_CONTAINER,
+        SessionStateAnimator::ANIMATION_GRAYSCALE_BRIGHTNESS));
+  }
+
+  void ExpectShutdownAnimationCancel() {
+    SCOPED_TRACE("Failure in ExpectShutdownAnimationCancel");
+    EXPECT_LT(0u, test_animator_->GetAnimationCount());
+    EXPECT_TRUE(test_animator_->AreContainersAnimated(
+        SessionStateAnimator::ROOT_CONTAINER,
+        SessionStateAnimator::ANIMATION_UNDO_GRAYSCALE_BRIGHTNESS));
+  }
+
   void ExpectWallpaperIsShowing() {
     SCOPED_TRACE("Failure in ExpectWallpaperIsShowing");
     EXPECT_LT(0u, test_animator_->GetAnimationCount());
@@ -249,8 +285,13 @@ class LockStateControllerTest : public PowerButtonTestBase {
     lock_state_controller_->OnLockScreenHide(closure);
   }
 
+  // Simulate that shutdown sound duration callback is done.
+  void ShutdownSoundPlayed() { a11y_controller_->FlushMojoForTest(); }
+
   TestShutdownController test_shutdown_controller_;
   TestSessionStateAnimator* test_animator_ = nullptr;  // not owned
+  AccessibilityController* a11y_controller_ = nullptr;  // not owned
+  TestAccessibilityControllerClient a11y_client_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(LockStateControllerTest);
@@ -342,14 +383,14 @@ TEST_F(LockStateControllerTest, LegacyIgnorePowerButtonIfScreenIsOff) {
 
   // When the screen brightness is at 0%, we shouldn't do anything in response
   // to power button presses.
-  power_manager_client_->SendBrightnessChanged(0, true);
+  SendBrightnessChange(0, kUserCause);
   PressPowerButton();
   EXPECT_FALSE(lock_state_test_api_->is_animating_lock());
   ReleasePowerButton();
 
   // After increasing the brightness to 10%, we should start the timer like
   // usual.
-  power_manager_client_->SendBrightnessChanged(10, true);
+  SendBrightnessChange(10, kUserCause);
   PressPowerButton();
   EXPECT_TRUE(lock_state_test_api_->is_animating_lock());
   ReleasePowerButton();
@@ -378,7 +419,7 @@ TEST_F(LockStateControllerTest, LegacyHonorPowerButtonInDockedMode) {
 
   // When all of the displays are turned off (e.g. due to user inactivity), the
   // power button should be ignored.
-  power_manager_client_->SendBrightnessChanged(0, true);
+  SendBrightnessChange(0, kUserCause);
   internal_display->set_current_mode(nullptr);
   external_display->set_current_mode(nullptr);
   power_button_controller_->OnDisplayModeChanged(outputs);
@@ -637,15 +678,15 @@ TEST_F(LockStateControllerTest, DisableTouchscreenForScreenOff) {
   base::RunLoop().RunUntilIdle();
 
   // Manually turn the screen off and check that the touchscreen is enabled.
-  power_manager_client_->SendBrightnessChanged(0, true /* user_initiated */);
+  SendBrightnessChange(0, kUserCause);
   EXPECT_TRUE(Shell::Get()->touch_devices_controller()->GetTouchscreenEnabled(
-      TouchscreenEnabledSource::GLOBAL));
+      TouchDeviceEnabledSource::GLOBAL));
 
   // It should be disabled if the screen is turned off due to user inactivity.
-  power_manager_client_->SendBrightnessChanged(100, true /* user_initiated */);
-  power_manager_client_->SendBrightnessChanged(0, false /* user_initiated */);
+  SendBrightnessChange(100, kUserCause);
+  SendBrightnessChange(0, kOtherCause);
   EXPECT_FALSE(Shell::Get()->touch_devices_controller()->GetTouchscreenEnabled(
-      TouchscreenEnabledSource::GLOBAL));
+      TouchDeviceEnabledSource::GLOBAL));
 }
 
 // Tests that the kTouchscreenUsableWhileScreenOff switch keeps the touchscreen
@@ -660,9 +701,47 @@ TEST_F(LockStateControllerTest, TouchscreenUnableWhileScreenOff) {
   base::RunLoop().RunUntilIdle();
 
   // The touchscreen should remain enabled.
-  power_manager_client_->SendBrightnessChanged(0, false /* user_initiated */);
+  SendBrightnessChange(0, kOtherCause);
   EXPECT_TRUE(Shell::Get()->touch_devices_controller()->GetTouchscreenEnabled(
-      TouchscreenEnabledSource::GLOBAL));
+      TouchDeviceEnabledSource::GLOBAL));
+}
+
+// Tests that continue pressing the power button for a while after power menu is
+// shown should trigger the cancellable pre-shutdown animation.
+TEST_F(LockStateControllerTest, ShutDownAfterShowPowerMenu) {
+  Initialize(ButtonType::NORMAL, LoginStatus::USER);
+  PressPowerButton();
+  EXPECT_TRUE(power_button_test_api_->IsMenuOpened());
+  ASSERT_TRUE(power_button_test_api_->TriggerPreShutdownTimeout());
+  EXPECT_TRUE(lock_state_test_api_->shutdown_timer_is_running());
+
+  ExpectShutdownAnimationStarted();
+  AdvancePartially(SessionStateAnimator::ANIMATION_SPEED_SHUTDOWN, 0.5f);
+  // Release the power button before the shutdown timer fires.
+  ReleasePowerButton();
+  EXPECT_FALSE(lock_state_test_api_->shutdown_timer_is_running());
+  ExpectShutdownAnimationCancel();
+
+  power_button_controller_->DismissMenu();
+  EXPECT_FALSE(power_button_test_api_->IsMenuOpened());
+
+  // Press the button again and make the shutdown timeout fire this time.
+  // Check that we start the timer for actually requesting the shutdown.
+  PressPowerButton();
+  ASSERT_TRUE(power_button_test_api_->TriggerPreShutdownTimeout());
+  EXPECT_TRUE(lock_state_test_api_->shutdown_timer_is_running());
+
+  Advance(SessionStateAnimator::ANIMATION_SPEED_SHUTDOWN);
+  ExpectShutdownAnimationFinished();
+  lock_state_test_api_->trigger_shutdown_timeout();
+
+  ShutdownSoundPlayed();
+  EXPECT_TRUE(lock_state_test_api_->real_shutdown_timer_is_running());
+  EXPECT_EQ(0, NumShutdownRequests());
+
+  // When the timeout fires, we should request a shutdown.
+  lock_state_test_api_->trigger_real_shutdown_timeout();
+  EXPECT_EQ(1, NumShutdownRequests());
 }
 
 }  // namespace ash
