@@ -10,20 +10,24 @@
 #include <string>
 #include <vector>
 
-#include "ash/ash_constants.h"
-#include "ash/ash_layout_constants.h"
 #include "ash/frame/custom_frame_view_ash.h"
 #include "ash/frame/detached_title_area_renderer.h"
+#include "ash/public/cpp/ash_constants.h"
+#include "ash/public/cpp/ash_layout_constants.h"
 #include "ash/public/cpp/immersive/immersive_fullscreen_controller_delegate.h"
-#include "ash/window_manager.h"
+#include "ash/public/cpp/window_properties.h"
+#include "ash/shell.h"
 #include "ash/wm/move_event_handler.h"
 #include "ash/wm/panels/panel_frame_view.h"
 #include "ash/wm/property_util.h"
 #include "ash/wm/window_properties.h"
 #include "ash/wm/window_util.h"
+#include "ash/ws/window_service_owner.h"
 #include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
 #include "services/ui/public/interfaces/window_manager.mojom.h"
+#include "services/ui/ws2/window_properties.h"
+#include "services/ui/ws2/window_service.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/transient_window_client.h"
 #include "ui/aura/mus/property_converter.h"
@@ -91,6 +95,7 @@ class ImmersiveFullscreenControllerDelegateMus
     SetVisibleFraction(0);
   }
   void OnImmersiveRevealEnded() override { DestroyTitleAreaWindow(); }
+  void OnImmersiveFullscreenEntered() override {}
   void OnImmersiveFullscreenExited() override { DestroyTitleAreaWindow(); }
   void SetVisibleFraction(double visible_fraction) override {
     aura::Window* title_area_window = GetTitleAreaWindow();
@@ -187,10 +192,14 @@ class WmNativeWidgetAura : public views::NativeWidgetAura {
       custom_frame_view_->SetHeaderHeight({height});
   }
 
+  void set_cursor(const ui::Cursor& cursor) { cursor_ = cursor; }
+
   // views::NativeWidgetAura:
   views::NonClientFrameView* CreateNonClientFrameView() override {
-    move_event_handler_ = std::make_unique<MoveEventHandler>(
-        window_manager_client_, GetNativeView());
+    if (window_manager_client_) {
+      move_event_handler_ = std::make_unique<MoveEventHandler>(
+          window_manager_client_, GetNativeView());
+    }
     // TODO(sky): investigate why we have this. Seems this should be the same
     // as not specifying client area insets.
     if (remove_standard_frame_)
@@ -206,6 +215,7 @@ class WmNativeWidgetAura : public views::NativeWidgetAura {
     custom_frame_view_ =
         new CustomFrameViewAsh(GetWidget(), immersive_delegate_.get(),
                                enable_immersive_, window_style_);
+
     // Only the header actually paints any content. So the rest of the region is
     // marked as transparent content (see below in NonClientFrameController()
     // ctor). So, it is necessary to provide a texture-layer for the header
@@ -217,11 +227,17 @@ class WmNativeWidgetAura : public views::NativeWidgetAura {
     return custom_frame_view_;
   }
 
+  gfx::NativeCursor GetCursor(const gfx::Point& point) override {
+    return cursor_;
+  }
+
  private:
   const bool remove_standard_frame_;
   const bool enable_immersive_;
   const mojom::WindowStyle window_style_;
 
+  // TODO: this is no longer necessary once --mash is removed,
+  // https://crbug.com/842365.
   std::unique_ptr<MoveEventHandler> move_event_handler_;
 
   aura::WindowManagerClient* window_manager_client_;
@@ -232,6 +248,11 @@ class WmNativeWidgetAura : public views::NativeWidgetAura {
   // by the Widget's view hierarchy (e.g. it's a child of Widget's root View).
   CustomFrameViewAsh* custom_frame_view_ = nullptr;
 
+  // The cursor for this widget. CompoundEventFilter will retrieve this cursor
+  // via GetCursor and update the CursorManager's active cursor as appropriate
+  // (i.e. when the mouse pointer is over this widget).
+  ui::Cursor cursor_;
+
   DISALLOW_COPY_AND_ASSIGN(WmNativeWidgetAura);
 };
 
@@ -246,11 +267,18 @@ class ClientViewMus : public views::ClientView {
 
   // views::ClientView:
   bool CanClose() override {
-    if (!frame_controller_->window())
-      return true;
-
-    frame_controller_->window_manager_client()->RequestClose(
-        frame_controller_->window());
+    // CanClose() is called when the user requests the window to close (such as
+    // clicking the close button). As this window is managed by a remote client
+    // pass the request to the remote client and return false (to cancel the
+    // close). If the remote client wants the window to close, it will close it
+    // in a way that does not reenter this code path.
+    if (frame_controller_->window_manager_client()) {
+      frame_controller_->window_manager_client()->RequestClose(
+          frame_controller_->window());
+    } else {
+      Shell::Get()->window_service_owner()->window_service()->RequestClose(
+          frame_controller_->window());
+    }
     return false;
   }
 
@@ -267,9 +295,10 @@ NonClientFrameController::NonClientFrameController(
     aura::Window* context,
     const gfx::Rect& bounds,
     ui::mojom::WindowType window_type,
+    aura::PropertyConverter* property_converter,
     std::map<std::string, std::vector<uint8_t>>* properties,
-    WindowManager* window_manager)
-    : window_manager_client_(window_manager->window_manager_client()),
+    aura::WindowManagerClient* window_manager_client)
+    : window_manager_client_(window_manager_client),
       widget_(new views::Widget),
       window_(nullptr) {
   // To simplify things this code creates a Widget. While a Widget is created
@@ -301,8 +330,6 @@ NonClientFrameController::NonClientFrameController(
   window_->AddObserver(this);
   params.native_widget = native_widget;
   aura::SetWindowType(window_, window_type);
-  aura::PropertyConverter* property_converter =
-      window_manager->property_converter();
   for (auto& property_pair : *properties) {
     property_converter->SetPropertyFromTransportValue(
         window_, property_pair.first, &property_pair.second);
@@ -334,14 +361,12 @@ NonClientFrameController* NonClientFrameController::Get(aura::Window* window) {
 // static
 gfx::Insets NonClientFrameController::GetPreferredClientAreaInsets() {
   return gfx::Insets(
-      GetAshLayoutSize(AshLayoutSize::NON_BROWSER_CAPTION_BUTTON).height(), 0,
-      0, 0);
+      GetAshLayoutSize(AshLayoutSize::kNonBrowserCaption).height(), 0, 0, 0);
 }
 
 // static
 int NonClientFrameController::GetMaxTitleBarButtonWidth() {
-  return GetAshLayoutSize(AshLayoutSize::NON_BROWSER_CAPTION_BUTTON).width() *
-         3;
+  return GetAshLayoutSize(AshLayoutSize::kNonBrowserCaption).width() * 3;
 }
 
 void NonClientFrameController::SetClientArea(
@@ -353,6 +378,11 @@ void NonClientFrameController::SetClientArea(
       ->SetHeaderHeight(insets.top());
 }
 
+void NonClientFrameController::StoreCursor(const ui::Cursor& cursor) {
+  static_cast<WmNativeWidgetAura*>(widget_->native_widget())
+      ->set_cursor(cursor);
+}
+
 NonClientFrameController::~NonClientFrameController() {
   aura::client::GetTransientWindowClient()->RemoveObserver(this);
   if (window_)
@@ -386,10 +416,14 @@ bool NonClientFrameController::CanMinimize() const {
                      ui::mojom::kResizeBehaviorCanMinimize) != 0;
 }
 
+bool NonClientFrameController::CanActivate() const {
+  // kCanFocus is used for both focus and activation.
+  return window_ && window_->GetProperty(ui::ws2::kCanFocus) &&
+         views::WidgetDelegate::CanActivate();
+}
+
 bool NonClientFrameController::ShouldShowWindowTitle() const {
-  // Only draw the title if the client hasn't declared any additional client
-  // areas which might conflict with it.
-  return window_ && additional_client_areas_.empty();
+  return window_ && window_->GetProperty(kWindowTitleShownKey);
 }
 
 views::ClientView* NonClientFrameController::CreateClientView(
