@@ -5,16 +5,16 @@
 #include "ash/wm/overview/scoped_transform_overview_window.h"
 
 #include <algorithm>
-#include <memory>
-#include <vector>
 
-#include "ash/public/cpp/ash_switches.h"
+#include "ash/public/cpp/ash_features.h"
+#include "ash/shell.h"
+#include "ash/wm/overview/cleanup_animation_observer.h"
 #include "ash/wm/overview/overview_utils.h"
 #include "ash/wm/overview/overview_window_animation_observer.h"
 #include "ash/wm/overview/scoped_overview_animation_settings.h"
 #include "ash/wm/overview/window_grid.h"
+#include "ash/wm/overview/window_selector.h"
 #include "ash/wm/overview/window_selector_item.h"
-#include "ash/wm/splitview/split_view_controller.h"
 #include "ash/wm/window_mirror_view.h"
 #include "ash/wm/window_state.h"
 #include "ash/wm/window_transient_descendant_iterator.h"
@@ -27,12 +27,15 @@
 #include "ui/compositor/layer.h"
 #include "ui/compositor/layer_observer.h"
 #include "ui/compositor/paint_recorder.h"
+#include "ui/compositor/scoped_layer_animation_settings.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/safe_integer_conversions.h"
 #include "ui/gfx/transform_util.h"
 #include "ui/views/widget/widget.h"
 #include "ui/wm/core/coordinate_conversion.h"
+#include "ui/wm/core/shadow_controller.h"
 #include "ui/wm/core/shadow_types.h"
+#include "ui/wm/core/window_animations.h"
 #include "ui/wm/core/window_util.h"
 
 namespace ash {
@@ -103,15 +106,21 @@ class ScopedTransformOverviewWindow::LayerCachingAndFilteringObserver
 
 // WindowMask is applied to overview windows to give them rounded edges while
 // they are in overview mode.
-class ScopedTransformOverviewWindow::WindowMask : public ui::LayerDelegate {
+class ScopedTransformOverviewWindow::WindowMask : public ui::LayerDelegate,
+                                                  public aura::WindowObserver {
  public:
   explicit WindowMask(aura::Window* window)
       : layer_(ui::LAYER_TEXTURED), window_(window) {
+    window_->AddObserver(this);
     layer_.set_delegate(this);
     layer_.SetFillsBoundsOpaquely(false);
   }
 
-  ~WindowMask() override { layer_.set_delegate(nullptr); }
+  ~WindowMask() override {
+    if (window_)
+      window_->RemoveObserver(this);
+    layer_.set_delegate(nullptr);
+  }
 
   void set_top_inset(int top_inset) { top_inset_ = top_inset; }
   ui::Layer* layer() { return &layer_; }
@@ -146,6 +155,19 @@ class ScopedTransformOverviewWindow::WindowMask : public ui::LayerDelegate {
   void OnDeviceScaleFactorChanged(float old_device_scale_factor,
                                   float new_device_scale_factor) override {}
 
+  // aura::WindowObserver:
+  void OnWindowBoundsChanged(aura::Window* window,
+                             const gfx::Rect& old_bounds,
+                             const gfx::Rect& new_bounds,
+                             ui::PropertyChangeReason reason) override {
+    layer_.SetBounds(new_bounds);
+  }
+
+  void OnWindowDestroying(aura::Window* window) override {
+    window_->RemoveObserver(this);
+    window_ = nullptr;
+  }
+
   ui::Layer layer_;
   int top_inset_ = 0;
   // Pointer to the window of which this is a mask to.
@@ -163,21 +185,20 @@ ScopedTransformOverviewWindow::ScopedTransformOverviewWindow(
       overview_started_(false),
       original_opacity_(window->layer()->GetTargetOpacity()),
       weak_ptr_factory_(this) {
-  if (IsNewOverviewUi())
-    type_ = GetWindowDimensionsType(window);
+  type_ = GetWindowDimensionsType(window);
 }
 
 ScopedTransformOverviewWindow::~ScopedTransformOverviewWindow() = default;
 
 void ScopedTransformOverviewWindow::RestoreWindow(bool reset_transform) {
-  ::wm::SetShadowElevation(window_, original_shadow_elevation_);
-
+  Shell::Get()->shadow_controller()->UpdateShadowForWindow(window_);
   wm::GetWindowState(window_)->set_ignored_by_shelf(ignored_by_shelf_);
   if (minimized_widget_) {
     mask_.reset();
-    // TODO(oshima): Use unminimize animation instead of hiding animation.
-    minimized_widget_->CloseNow();
-    minimized_widget_.reset();
+    // Fade out the minimized widget. This animation continues past the
+    // lifetime of |this|.
+    FadeOutWidgetOnExit(std::move(minimized_widget_),
+                        OVERVIEW_ANIMATION_EXIT_OVERVIEW_MODE_FADE_OUT);
     return;
   }
 
@@ -191,7 +212,7 @@ void ScopedTransformOverviewWindow::RestoreWindow(bool reset_transform) {
     // Add requests to cache render surface and perform trilinear filtering for
     // the exit animation of overview mode. The requests will be removed when
     // the exit animation finishes.
-    if (switches::IsTrilinearFilteringEnabled()) {
+    if (features::IsTrilinearFilteringEnabled()) {
       for (auto& settings : animation_settings_list) {
         settings->CacheRenderSurface();
         settings->TrilinearFiltering();
@@ -213,16 +234,15 @@ void ScopedTransformOverviewWindow::BeginScopedAnimation(
   // Remove the mask before animating because masks affect animation
   // performance. Observe the animation and add the mask after animating if the
   // animation type is layouting selector items.
-  if (IsNewOverviewUi()) {
-    mask_.reset();
-    selector_item_->SetShadowBounds(base::nullopt);
-    selector_item_->DisableBackdrop();
+  mask_.reset();
+  selector_item_->SetShadowBounds(base::nullopt);
+  selector_item_->DisableBackdrop();
 
-    if (window_->GetProperty(aura::client::kShowStateKey) !=
-        ui::SHOW_STATE_MINIMIZED) {
-      window_->layer()->SetMaskLayer(original_mask_layer_);
-    }
+  if (window_->GetProperty(aura::client::kShowStateKey) !=
+      ui::SHOW_STATE_MINIMIZED) {
+    window_->layer()->SetMaskLayer(original_mask_layer_);
   }
+
   for (auto* window : wm::GetTransientTreeIterator(GetOverviewWindow())) {
     auto settings = std::make_unique<ScopedOverviewAnimationSettings>(
         animation_type, window);
@@ -243,10 +263,9 @@ void ScopedTransformOverviewWindow::BeginScopedAnimation(
     animation_settings->push_back(std::move(settings));
   }
 
-  if (IsNewOverviewUi() &&
-      animation_type == OVERVIEW_ANIMATION_LAY_OUT_SELECTOR_ITEMS) {
-    if (animation_settings->size() > 0u)
-      animation_settings->front()->AddObserver(this);
+  if (animation_type == OVERVIEW_ANIMATION_LAY_OUT_SELECTOR_ITEMS &&
+      animation_settings->size() > 0u) {
+    animation_settings->front()->AddObserver(this);
   }
 }
 
@@ -431,10 +450,6 @@ gfx::Rect ScopedTransformOverviewWindow::ShrinkRectToFitPreservingAspectRatio(
   gfx::Rect new_bounds(bounds.x() + horizontal_offset,
                        bounds.y() + vertical_offset, width, height);
 
-  if (!IsNewOverviewUi()) {
-    DCHECK_EQ(ScopedTransformOverviewWindow::GridWindowFillMode::kNormal,
-              type());
-  }
   switch (type()) {
     case ScopedTransformOverviewWindow::GridWindowFillMode::kLetterBoxed:
     case ScopedTransformOverviewWindow::GridWindowFillMode::kPillarBoxed: {
@@ -474,6 +489,12 @@ gfx::Rect ScopedTransformOverviewWindow::ShrinkRectToFitPreservingAspectRatio(
   return new_bounds;
 }
 
+gfx::Rect ScopedTransformOverviewWindow::GetMaskBoundsForTesting() const {
+  if (!mask_)
+    return gfx::Rect();
+  return mask_->layer()->bounds();
+}
+
 void ScopedTransformOverviewWindow::Close() {
   if (immediate_close_for_tests) {
     CloseWidget();
@@ -486,8 +507,7 @@ void ScopedTransformOverviewWindow::Close() {
 }
 
 void ScopedTransformOverviewWindow::PrepareForOverview() {
-  original_shadow_elevation_ = window_->GetProperty(::wm::kShadowElevationKey);
-  ::wm::SetShadowElevation(window_, ::wm::kShadowElevationNone);
+  Shell::Get()->shadow_controller()->UpdateShadowForWindow(window_);
 
   DCHECK(!overview_started_);
   overview_started_ = true;
@@ -501,12 +521,17 @@ void ScopedTransformOverviewWindow::PrepareForOverview() {
   // requests will be removed in dtor. So the requests will be valid during the
   // enter animation and the whole time during overview mode. For the exit
   // animation of overview mode, we need to add those requests again.
-  if (switches::IsTrilinearFilteringEnabled()) {
+  if (features::IsTrilinearFilteringEnabled()) {
     for (auto* window : wm::GetTransientTreeIterator(GetOverviewWindow())) {
       cached_and_filtered_layer_observers_.push_back(
           std::make_unique<LayerCachingAndFilteringObserver>(window->layer()));
     }
   }
+
+  // Apply rounded edge mask. Windows which are animated into overview mode
+  // will have their mask removed before the animation begins and reapplied
+  // after the animation ends.
+  CreateAndApplyMaskAndShadow();
 }
 
 void ScopedTransformOverviewWindow::CloseWidget() {
@@ -536,9 +561,6 @@ ScopedTransformOverviewWindow::GetOverviewWindowForMinimizedState() const {
 }
 
 void ScopedTransformOverviewWindow::UpdateWindowDimensionsType() {
-  if (!IsNewOverviewUi())
-    return;
-
   type_ = GetWindowDimensionsType(window_);
   window_selector_bounds_.reset();
 }
@@ -547,23 +569,27 @@ void ScopedTransformOverviewWindow::CancelAnimationsListener() {
   StopObservingImplicitAnimations();
 }
 
-void ScopedTransformOverviewWindow::OnImplicitAnimationsCompleted() {
-  // Add the mask which gives the window selector items rounded corners, and add
-  // the shadow around the window.
-  DCHECK(IsNewOverviewUi());
-
-  ui::Layer* layer = minimized_widget_
-                         ? minimized_widget_->GetContentsView()->layer()
-                         : window_->layer();
+void ScopedTransformOverviewWindow::ResizeMinimizedWidgetIfNeeded() {
   if (!minimized_widget_)
-    original_mask_layer_ = window_->layer()->layer_mask_layer();
+    return;
 
-  mask_ = std::make_unique<WindowMask>(GetOverviewWindow());
-  mask_->layer()->SetBounds(layer->bounds());
-  mask_->set_top_inset(GetTopInset());
-  layer->SetMaskLayer(mask_->layer());
-  selector_item_->SetShadowBounds(base::make_optional(GetTransformedBounds()));
-  selector_item_->EnableBackdropIfNeeded();
+  gfx::Rect bounds(window_->GetBoundsInScreen());
+  if (bounds.size() == window_->GetBoundsInScreen().size())
+    return;
+
+  wm::WindowMirrorView* mirror_view =
+      static_cast<wm::WindowMirrorView*>(minimized_widget_->GetContentsView());
+  if (mirror_view) {
+    mirror_view->RecreateMirrorLayers();
+    bounds.Inset(0, 0, 0,
+                 bounds.height() - mirror_view->GetPreferredSize().height());
+    minimized_widget_->SetBounds(bounds);
+  }
+}
+
+void ScopedTransformOverviewWindow::OnImplicitAnimationsCompleted() {
+  CreateAndApplyMaskAndShadow();
+  selector_item_->OnDragAnimationCompleted();
 }
 
 void ScopedTransformOverviewWindow::CreateMirrorWindowForMinimizedState() {
@@ -595,6 +621,30 @@ void ScopedTransformOverviewWindow::CreateMirrorWindowForMinimizedState() {
   }
   minimized_widget_->SetBounds(bounds);
   minimized_widget_->Show();
+
+  minimized_widget_->SetOpacity(0.f);
+  ScopedOverviewAnimationSettings animation_settings(
+      OVERVIEW_ANIMATION_ENTER_OVERVIEW_MODE_TABLET_FADE_IN,
+      minimized_widget_->GetNativeWindow());
+  minimized_widget_->SetOpacity(1.f);
+}
+
+void ScopedTransformOverviewWindow::CreateAndApplyMaskAndShadow() {
+  // Add the mask which gives the window selector items rounded corners, and add
+  // the shadow around the window.
+  ui::Layer* layer = minimized_widget_
+                         ? minimized_widget_->GetContentsView()->layer()
+                         : window_->layer();
+
+  if (!minimized_widget_)
+    original_mask_layer_ = window_->layer()->layer_mask_layer();
+
+  mask_ = std::make_unique<WindowMask>(GetOverviewWindow());
+  mask_->layer()->SetBounds(layer->bounds());
+  mask_->set_top_inset(GetTopInset());
+  layer->SetMaskLayer(mask_->layer());
+  selector_item_->SetShadowBounds(base::make_optional(GetTransformedBounds()));
+  selector_item_->EnableBackdropIfNeeded();
 }
 
 }  // namespace ash
