@@ -35,7 +35,7 @@ class Widget;
 namespace ash {
 
 class OverviewWindowDragController;
-class SplitViewOverviewOverlay;
+class SplitViewDragIndicators;
 class WindowGrid;
 class WindowSelectorDelegate;
 class WindowSelectorItem;
@@ -93,15 +93,14 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
   void WindowClosing(WindowSelectorItem* window);
 
   // Called to set bounds for window grids. Used for split view.
-  void SetBoundsForWindowGridsInScreen(const gfx::Rect& bounds);
   void SetBoundsForWindowGridsInScreenIgnoringWindow(
       const gfx::Rect& bounds,
       WindowSelectorItem* ignored_item);
 
-  // Called to show or hide the split view overview overlay. This will do
+  // Called to show or hide the split view drag indicators. This will do
   // nothing if split view is not enabled. |event_location| is used to reparent
-  // |split_view_overview_overlays_|'s widget, if necessary.
-  void SetSplitViewOverviewOverlayIndicatorState(
+  // |split_view_drag_indicators_|'s widget, if necessary.
+  void SetSplitViewDragIndicatorsIndicatorState(
       IndicatorState indicator_state,
       const gfx::Point& event_location);
   // Retrieves the window grid whose root window matches |root_window|. Returns
@@ -109,7 +108,13 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
   WindowGrid* GetGridWithRootWindow(aura::Window* root_window);
 
   // Add |window| to the grid in |grid_list_| with the same root window. Does
-  // nothing if the grid already contains |window|.
+  // nothing if the grid already contains |window|. This may be called in two
+  // scenarioes: 1) when a item in split view mode was previously snapped but
+  // should now be returned to the window grid (e.g. split view divider dragged
+  // to either edge, or a window is snapped to a postion that already has a
+  // snapped window); 2) when a window (not from overview) is dragged while
+  // overview is open and the window is dropped on the new selector item, the
+  // dragged window is then added to the overview.
   void AddItem(aura::Window* window);
 
   // Removes the window selector item from the overview window grid.
@@ -120,15 +125,37 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
   void Drag(WindowSelectorItem* item, const gfx::Point& location_in_screen);
   void CompleteDrag(WindowSelectorItem* item,
                     const gfx::Point& location_in_screen);
+  void StartSplitViewDragMode(const gfx::Point& location_in_screen);
+  void Fling(WindowSelectorItem* item,
+             const gfx::Point& location_in_screen,
+             float velocity_x,
+             float velocity_y);
   void ActivateDraggedWindow();
   void ResetDraggedWindowGesture();
 
-  // Positions all of the windows in the overview.
-  void PositionWindows(bool animate);
+  // Called when a window's tab(s) start/continue/end being dragged around if
+  // overview mode is active.
+  // TODO(xdai): Currently it doesn't work for multi-display scenario.
+  void OnWindowDragStarted(aura::Window* dragged_window);
+  void OnWindowDragContinued(aura::Window* dragged_window,
+                             const gfx::Point& location_in_screen);
+  void OnWindowDragEnded(aura::Window* dragged_window,
+                         const gfx::Point& location_in_screen);
+
+  // Positions all of the windows in the overview, except |ignored_item|.
+  void PositionWindows(bool animate,
+                       WindowSelectorItem* ignored_item = nullptr);
 
   // If we are in middle of ending overview mode.
   bool IsShuttingDown() const;
 
+  // Checks if the grid associated with a given |root_window| needs to have the
+  // wallpaper animated. Returns false if one of the grids windows covers the
+  // the entire workspace, true otherwise.
+  bool ShouldAnimateWallpaper(aura::Window* root_window);
+
+  bool IsWindowInOverview(const aura::Window* window);
+
   WindowSelectorDelegate* delegate() { return delegate_; }
 
   bool restoring_minimized_windows() const {
@@ -142,8 +169,8 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
     return grid_list_;
   }
 
-  SplitViewOverviewOverlay* split_view_overview_overlay() {
-    return split_view_overview_overlay_.get();
+  SplitViewDragIndicators* split_view_drag_indicators() {
+    return split_view_drag_indicators_.get();
   }
 
   OverviewWindowDragController* window_drag_controller() {
@@ -202,6 +229,9 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
   // Called when the display area for the overview window grids changed.
   void OnDisplayBoundsChanged();
 
+  // Returns true if all its window grids don't have any window item.
+  bool IsEmpty();
+
   // Tracks observed windows.
   std::set<aura::Window*> observed_windows_;
 
@@ -223,7 +253,7 @@ class ASH_EXPORT WindowSelector : public display::DisplayObserver,
 
   // The owner of the widget which displays splitview related information in
   // overview mode. This will be nullptr if split view is not enabled.
-  std::unique_ptr<SplitViewOverviewOverlay> split_view_overview_overlay_;
+  std::unique_ptr<SplitViewDragIndicators> split_view_drag_indicators_;
 
   // Tracks the index of the root window the selection widget is in.
   size_t selected_grid_index_ = 0;
