@@ -8,7 +8,6 @@
 #include <map>
 #include <utility>
 
-#include "ash/keyboard/keyboard_observer_register.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/public/cpp/window_properties.h"
 #include "ash/root_window_controller.h"
@@ -254,7 +253,6 @@ PanelLayoutManager::PanelLayoutManager(Window* panel_container)
       dragged_panel_(nullptr),
       shelf_(nullptr),
       last_active_panel_(nullptr),
-      keyboard_observer_(this),
       weak_factory_(this) {
   DCHECK(panel_container);
   Shell::Get()->activation_client()->AddObserver(this);
@@ -456,13 +454,6 @@ void PanelLayoutManager::OnShelfAlignmentChanged(aura::Window* root_window) {
     Relayout();
 }
 
-void PanelLayoutManager::OnVirtualKeyboardStateChanged(bool activated,
-                                                       Window* root_window) {
-  UpdateKeyboardObserverFromStateChanged(activated, root_window,
-                                         panel_container_->GetRootWindow(),
-                                         &keyboard_observer_);
-}
-
 /////////////////////////////////////////////////////////////////////////////
 // PanelLayoutManager, WindowObserver implementation:
 
@@ -902,7 +893,7 @@ void PanelLayoutManager::UpdateCallouts() {
 ////////////////////////////////////////////////////////////////////////////////
 // keyboard::KeyboardControllerObserver implementation:
 
-void PanelLayoutManager::OnKeyboardWorkspaceOccludedBoundsChanging(
+void PanelLayoutManager::OnKeyboardWorkspaceOccludedBoundsChanged(
     const gfx::Rect& keyboard_bounds) {
   const gfx::Rect& parent_bounds = panel_container_->bounds();
   int available_space = parent_bounds.height() - keyboard_bounds.height();
@@ -936,8 +927,4 @@ void PanelLayoutManager::OnKeyboardWorkspaceOccludedBoundsChanging(
   OnWindowResized();
 }
 
-void PanelLayoutManager::OnKeyboardClosed() {
-  keyboard_observer_.RemoveAll();
-}
-
 }  // namespace ash
