@@ -5,17 +5,19 @@
 #ifndef ASH_WM_SPLITSVIEW_SPLIT_VIEW_CONTROLLER_H_
 #define ASH_WM_SPLITSVIEW_SPLIT_VIEW_CONTROLLER_H_
 
+#include "ash/accessibility/accessibility_observer.h"
 #include "ash/ash_export.h"
+#include "ash/display/screen_orientation_controller.h"
 #include "ash/public/interfaces/split_view.mojom.h"
 #include "ash/shell_observer.h"
 #include "ash/wm/tablet_mode/tablet_mode_observer.h"
 #include "ash/wm/window_state_observer.h"
+#include "base/containers/flat_map.h"
 #include "base/macros.h"
 #include "base/observer_list.h"
 #include "base/time/time.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
 #include "mojo/public/cpp/bindings/interface_ptr_set.h"
-#include "third_party/WebKit/public/platform/modules/screen_orientation/WebScreenOrientationLockType.h"
 #include "ui/aura/window_observer.h"
 #include "ui/display/display.h"
 #include "ui/display/display_observer.h"
@@ -31,17 +33,20 @@ namespace ash {
 class SplitViewControllerTest;
 class SplitViewDivider;
 class SplitViewWindowSelectorTest;
+class OverviewWindowAnimationObserver;
 
 // The controller for the split view. It snaps a window to left/right side of
 // the screen. It also observes the two snapped windows and decides when to exit
 // the split view mode.
+// TODO(xdai): Make it work for multi-display non mirror environment.
 class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
                                        public aura::WindowObserver,
                                        public ash::wm::WindowStateObserver,
                                        public ::wm::ActivationChangeObserver,
                                        public ShellObserver,
                                        public display::DisplayObserver,
-                                       public TabletModeObserver {
+                                       public TabletModeObserver,
+                                       public AccessibilityObserver {
  public:
   enum State { NO_SNAP, LEFT_SNAPPED, RIGHT_SNAPPED, BOTH_SNAPPED };
 
@@ -80,6 +85,8 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // Returns true if split view mode is active.
   bool IsSplitViewModeActive() const;
 
+  OrientationLockType GetCurrentScreenOrientation() const;
+
   // Returns true if |screen_orientation_| is a landscape orientation.
   bool IsCurrentScreenOrientationLandscape() const;
 
@@ -88,7 +95,9 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // primary orientation.
   bool IsCurrentScreenOrientationPrimary() const;
 
-  // Snaps window to left/right.
+  // Snaps window to left/right. It will try to remove |window| from the
+  // overview window grid first before snapping it if |window| is currently
+  // showing in the overview window grid.
   void SnapWindow(aura::Window* window, SnapPosition snap_position);
 
   // Swaps the left and right windows. This will do nothing if one of the
@@ -102,7 +111,10 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   aura::Window* GetDefaultSnappedWindow();
 
   // Gets the window bounds according to the snap state |snap_state| and the
-  // separator position |separator_position_|.
+  // divider position |divider_position_|. The returned snapped window bounds
+  // are adjusted to its minimum size if the desired bounds are smaller than
+  // its minumum bounds. Note: the snapped window bounds can't be pushed
+  // outside of the workspace area.
   gfx::Rect GetSnappedWindowBoundsInParent(aura::Window* window,
                                            SnapPosition snap_position);
   gfx::Rect GetSnappedWindowBoundsInScreen(aura::Window* window,
@@ -110,6 +122,14 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   gfx::Rect GetDisplayWorkAreaBoundsInParent(aura::Window* window) const;
   gfx::Rect GetDisplayWorkAreaBoundsInScreen(aura::Window* window) const;
 
+  // Gets the desired snapped window bounds accoridng to the snap state
+  // |snap_state| and the divider pistion |divider_position_|.
+  gfx::Rect GetSnappedWindowBoundsInScreenUnadjusted(aura::Window* window,
+                                                     SnapPosition snap_postion);
+
+  // Gets the default value of |divider_position_|.
+  int GetDefaultDividerPosition(aura::Window* window) const;
+
   void StartResize(const gfx::Point& location_in_screen);
   void Resize(const gfx::Point& location_in_screen);
   void EndResize(const gfx::Point& location_in_screen);
@@ -121,6 +141,12 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // Ends the split view mode.
   void EndSplitView();
 
+  // Called when a window's tab(s) start/end being dragging around.
+  void OnWindowDragStarted(aura::Window* dragged_window);
+  void OnWindowDragEnded(aura::Window* dragged_window,
+                         SnapPosition desired_snap_position,
+                         const gfx::Point& last_location_in_screen);
+
   void AddObserver(Observer* observer);
   void RemoveObserver(Observer* observer);
 
@@ -128,7 +154,10 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   void AddObserver(mojom::SplitViewObserverPtr observer) override;
 
   // aura::WindowObserver:
-  void OnWindowDestroying(aura::Window* window) override;
+  void OnWindowDestroyed(aura::Window* window) override;
+  void OnWindowPropertyChanged(aura::Window* window,
+                               const void* key,
+                               intptr_t old) override;
 
   // ash::wm::WindowStateObserver:
   void OnPostWindowStateTypeChange(
@@ -151,6 +180,9 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // TabletModeObserver:
   void OnTabletModeEnding() override;
 
+  // AccessibilityObserver:
+  void OnAccessibilityStatusChanged() override;
+
   aura::Window* left_window() { return left_window_; }
   aura::Window* right_window() { return right_window_; }
   int divider_position() const { return divider_position_; }
@@ -158,27 +190,28 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   SnapPosition default_snap_position() const { return default_snap_position_; }
   SplitViewDivider* split_view_divider() { return split_view_divider_.get(); }
   bool is_resizing() const { return is_resizing_; }
-  blink::WebScreenOrientationLockType screen_orientation() const {
-    return screen_orientation_;
+  base::WeakPtr<OverviewWindowAnimationObserver>
+  snapped_window_animation_observer() {
+    return snapped_window_animation_observer_;
   }
 
  private:
   friend class SplitViewControllerTest;
   friend class SplitViewWindowSelectorTest;
+  class TabDraggedWindowObserver;
 
-  // Starts/Stops observing |window|.
+  // Start observing |window|.
   void StartObserving(aura::Window* window);
-  void StopObserving(aura::Window* window);
+  // Stop observing the window at associated with |snap_position|. Also updates
+  // shadows and sets |left_window_| or |right_window_| to nullptr.
+  void StopObserving(SnapPosition snap_position);
 
-  // Notifies observers that the split view state has been changed.
-  void NotifySplitViewStateChanged(State previous_state, State state);
+  // Update split view state and notify its observer about the change.
+  void UpdateSplitViewStateAndNotifyObservers();
 
   // Notifies observers that the split view divider position has been changed.
   void NotifyDividerPositionChanged();
 
-  // Gets the default value of |divider_position_|.
-  int GetDefaultDividerPosition(aura::Window* window) const;
-
   // Updates the black scrim layer's bounds and opacity while dragging the
   // divider. The opacity increases as the split divider gets closer to the edge
   // of the screen.
@@ -195,6 +228,13 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // resizing.
   void UpdateDividerPosition(const gfx::Point& location_in_screen);
 
+  // Get the window bounds for left_or_top and right_or_bottom snapped windows.
+  // Note the bounds returned by this function doesn't take the snapped windows
+  // minimum sizes into account.
+  void GetSnappedWindowBoundsInScreenInternal(aura::Window* window,
+                                              gfx::Rect* left_or_top_rect,
+                                              gfx::Rect* right_or_bottom_rect);
+
   // Splits the |work_area_rect| by |divider_rect| and outputs the two halves.
   // |left_or_top_rect|, |divider_rect| and |right_or_bottom_rect| should align
   // vertically or horizontally depending on |is_split_vertically|.
@@ -220,17 +260,23 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // the length of the longer side of the current display's work area bounds.
   int GetDividerEndPosition();
 
-  // If there are two snapped windows, closing/minimizing one of them will open
-  // overview window grid on the closed/minimized window side of the screen. If
-  // there is only one snapped windows, closing/minimizing the sanpped window
-  // will end split view mode and adjust the overview window grid bounds if the
-  // overview mode is active at that moment.
-  void OnSnappedWindowMinimizedOrDestroyed(aura::Window* window);
+  // Called after a to-be-snapped window |window| got snapped. It updates the
+  // split view states and notifies observers about the change. It also restore
+  // the snapped window's transform if it's not identity and activate it.
+  void OnWindowSnapped(aura::Window* window);
+
+  // If there are two snapped windows, closing/minimizing/tab-dragging one of
+  // them will open overview window grid on the closed/minimized/tab-dragged
+  // window side of the screen. If there is only one snapped windows, closing/
+  // minimizing/tab-dragging the sanpped window will end split view mode and
+  // adjust the overview window grid bounds if the overview mode is active at
+  // that moment.
+  void OnSnappedWindowDetached(aura::Window* window);
 
   // If the desired bounds of the snapped windows bounds |left_or_top_rect| and
   // |right_or_bottom_rect| are smaller than the minimum bounds of the snapped
-  // windows, adjust the desired bounds to the minimum bounds by pushing the
-  // snapped windows out of the work area display area.
+  // windows, adjust the desired bounds to the minimum bounds. Note the snapped
+  // windows can't be pushed out of the work area display area.
   void AdjustSnappedWindowBounds(gfx::Rect* left_or_top_rect,
                                  gfx::Rect* right_or_bottom_rect);
 
@@ -243,11 +289,6 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // on the minimum size of current snapped windows.
   void GetDividerOptionalPositionRatios(std::vector<float>* positionRatios);
 
-  // Selects the window that needs the quick/smooth resize. It's needed as
-  // Android only supports smooth resize for one app window at a time. If there
-  // are two snapped Arc app windows, always return the one who stacked above
-  // the other.
-  aura::Window* GetWindowForSmoothResize();
   // Gets the expected window component depending on current screen orientation
   // for resizing purpose.
   int GetWindowComponentForResize(aura::Window* window);
@@ -256,16 +297,45 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   gfx::Point GetEndDragLocationInScreen(aura::Window* window,
                                         const gfx::Point& location_in_screen);
 
-  // Restore |window| transform to identity transform if applicable and activate
-  // the window. Also Stack the other snapped window below |window| so that the
-  // two snapped windows are always the top two windows when split view mode is
-  // active.
-  void RestoreAndActivateSnappedWindow(aura::Window* window);
+  // Restores |window| transform to identity transform if applicable.
+  void RestoreTransformIfApplicable(aura::Window* window);
+
+  // Activates the snapped window |window| and stacks the other snapped window
+  // below |window| so that the two snapped windows are always the top two
+  // windows when split view mode is active.
+  void ActivateAndStackSnappedWindow(aura::Window* window);
+
+  // During resizing, it's possible that the resizing bounds of the snapped
+  // window is smaller than its minimum bounds, in this case we apply a
+  // translation to the snapped window to make it visually be placed outside of
+  // the workspace area.
+  void SetWindowsTransformDuringResizing();
+  // Restore the snapped windows transform to identity transform after resizing.
+  void RestoreWindowsTransformAfterResizing();
+
+  // Set |transform| for |window| and its transient descendants.
+  void SetTransform(aura::Window* window, const gfx::Transform& transform);
+
+  // Removes the window item that contains |window| from the overview window
+  // grid if |window| is currently showing in overview window grid. It should be
+  // called before trying to snap the window.
+  void RemoveWindowFromOverviewIfApplicable(aura::Window* window);
+
+  // Returns the window that is currently snapped at |snap_position|.
+  aura::Window* GetSnappedWindowAt(SnapPosition snap_position);
+
+  // Inserts |window| into overview window grid if overview mode is active. Do
+  // nothing if overview mode is inactive at the moment.
+  void InsertWindowToOverview(aura::Window* window);
 
   // Starts/Ends overview mode if the overview mode is inactive/active.
   void StartOverview();
   void EndOverview();
 
+  // Finalizes and cleans up after stopping dragging the divider bar to resize
+  // snapped windows.
+  void FinishWindowResizing(aura::Window* window);
+
   // Bindings for the SplitViewController interface.
   mojo::BindingSet<mojom::SplitViewController> bindings_;
 
@@ -284,6 +354,9 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // closer to the edge of the screen.
   std::unique_ptr<ui::Layer> black_scrim_layer_;
 
+  // The window observer that obseves the tab-dragged window.
+  std::unique_ptr<TabDraggedWindowObserver> dragged_window_observer_;
+
   // The distance between the origin of the divider and the origin of the screen
   // in screen coordinates.
   //     |<---     divider_position_    --->|
@@ -306,9 +379,8 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // versa.
   SnapPosition default_snap_position_ = NONE;
 
-  // The current orientation of the screen.
-  blink::WebScreenOrientationLockType screen_orientation_ =
-      blink::kWebScreenOrientationLockDefault;
+  // The previous orientation of the screen.
+  OrientationLockType previous_screen_orientation_ = OrientationLockType::kAny;
 
   // If the divider is currently being dragging.
   bool is_resizing_ = false;
@@ -316,12 +388,14 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // The time when splitview starts. Used for metric collection purpose.
   base::Time splitview_start_time_;
 
-  // The window that needs smooth resize. It's needed as Android only supports
-  // smooth resize for one app window at a time. It's always the snapped Arc app
-  // window if there is one. And if there are two snapped Arc app windows, it's
-  // the one who stacked above the other, see GetWindowForSmoothResize() for
-  // details.
-  aura::Window* smooth_resize_window_ = nullptr;
+  // The map from a to-be-snapped window to its overview item's bounds if the
+  // window comes from the overview.
+  base::flat_map<aura::Window*, gfx::Rect> overview_window_item_bounds_map_;
+
+  // Weak ptr to the observer that observes the snapped window's transform
+  // animaiton if it comes from the overview.
+  base::WeakPtr<OverviewWindowAnimationObserver>
+      snapped_window_animation_observer_ = nullptr;
 
   base::ObserverList<Observer> observers_;
   mojo::InterfacePtrSet<mojom::SplitViewObserver> mojo_observers_;
