@@ -14,6 +14,7 @@
 #include "ash/ash_export.h"
 
 namespace aura {
+class PropertyConverter;
 class Window;
 }
 
@@ -28,10 +29,13 @@ namespace ash {
 class WindowManager;
 
 // Creates and parents a new top-level window and returns it. The returned
-// aura::Window is owned by its parent.
+// aura::Window is owned by its parent. A value of null is returned if invalid
+// poarameters are supplied.
+// TODO(ws): Refine this for the Window Service as-a-library (no WindowManager).
 ASH_EXPORT aura::Window* CreateAndParentTopLevelWindow(
     WindowManager* window_manager,
     ui::mojom::WindowType window_type,
+    aura::PropertyConverter* property_converter,
     std::map<std::string, std::vector<uint8_t>>* properties);
 
 }  // namespace ash
