@@ -21,6 +21,7 @@
 #include "base/debug/alias.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
+#include "base/metrics/dummy_histogram.h"
 #include "base/metrics/histogram_functions.h"
 #include "base/metrics/metrics_hashes.h"
 #include "base/metrics/persistent_histogram_allocator.h"
@@ -31,6 +32,7 @@
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/synchronization/lock.h"
+#include "base/sys_info.h"
 #include "base/values.h"
 #include "build/build_config.h"
 
@@ -38,15 +40,6 @@ namespace base {
 
 namespace {
 
-// A constant to be stored in the dummy field and later verified. This could
-// be either 32 or 64 bit but clang won't truncate the value without an error.
-// TODO(bcwhite): Remove this once crbug/736675 is fixed.
-#if defined(ARCH_CPU_64_BITS) && !defined(OS_NACL)
-constexpr uintptr_t kDummyValue = 0xFEEDC0DEDEADBEEF;
-#else
-constexpr uintptr_t kDummyValue = 0xDEADBEEF;
-#endif
-
 bool ReadHistogramArguments(PickleIterator* iter,
                             std::string* histogram_name,
                             int* flags,
@@ -84,6 +77,11 @@ bool ReadHistogramArguments(PickleIterator* iter,
 
 bool ValidateRangeChecksum(const HistogramBase& histogram,
                            uint32_t range_checksum) {
+  // Normally, |histogram| should have type HISTOGRAM or be inherited from it.
+  // However, if it's expired, it will actually be a DUMMY_HISTOGRAM.
+  // Skip the checks in that case.
+  if (histogram.GetHistogramType() == DUMMY_HISTOGRAM)
+    return true;
   const Histogram& casted_histogram =
       static_cast<const Histogram&>(histogram);
 
@@ -161,6 +159,12 @@ class Histogram::Factory {
 HistogramBase* Histogram::Factory::Build() {
   HistogramBase* histogram = StatisticsRecorder::FindHistogram(name_);
   if (!histogram) {
+    // TODO(gayane): |HashMetricName()| is called again in Histogram
+    // constructor. Refactor code to avoid the additional call.
+    bool should_record =
+        StatisticsRecorder::ShouldRecordHistogram(HashMetricName(name_));
+    if (!should_record)
+      return DummyHistogram::GetInstance();
     // To avoid racy destruction at shutdown, the following will be leaked.
     const BucketRanges* created_ranges = CreateRanges();
     const BucketRanges* registered_ranges =
@@ -225,17 +229,19 @@ HistogramBase* Histogram::Factory::Build() {
     }
   }
 
-  CHECK_EQ(histogram_type_, histogram->GetHistogramType()) << name_;
-  if (bucket_count_ != 0 &&
-      !histogram->HasConstructionArguments(minimum_, maximum_, bucket_count_)) {
+  if (histogram_type_ != histogram->GetHistogramType() ||
+      (bucket_count_ != 0 && !histogram->HasConstructionArguments(
+                                 minimum_, maximum_, bucket_count_))) {
     // The construction arguments do not match the existing histogram.  This can
     // come about if an extension updates in the middle of a chrome run and has
-    // changed one of them, or simply by bad code within Chrome itself.  We
-    // return NULL here with the expectation that bad code in Chrome will crash
-    // on dereference, but extension/Pepper APIs will guard against NULL and not
-    // crash.
-    DLOG(ERROR) << "Histogram " << name_ << " has bad construction arguments";
-    return nullptr;
+    // changed one of them, or simply by bad code within Chrome itself.  A NULL
+    // return would cause Chrome to crash; better to just record it for later
+    // analysis.
+    UmaHistogramSparse("Histogram.MismatchedConstructionArguments",
+                       static_cast<Sample>(HashMetricName(name_)));
+    DLOG(ERROR) << "Histogram " << name_
+                << " has mismatched construction arguments";
+    return DummyHistogram::GetInstance();
   }
   return histogram;
 }
@@ -262,6 +268,16 @@ HistogramBase* Histogram::FactoryTimeGet(const std::string& name,
                     flags);
 }
 
+HistogramBase* Histogram::FactoryMicrosecondsTimeGet(const std::string& name,
+                                                     TimeDelta minimum,
+                                                     TimeDelta maximum,
+                                                     uint32_t bucket_count,
+                                                     int32_t flags) {
+  return FactoryGet(name, static_cast<Sample>(minimum.InMicroseconds()),
+                    static_cast<Sample>(maximum.InMicroseconds()), bucket_count,
+                    flags);
+}
+
 HistogramBase* Histogram::FactoryGet(const char* name,
                                      Sample minimum,
                                      Sample maximum,
@@ -279,6 +295,15 @@ HistogramBase* Histogram::FactoryTimeGet(const char* name,
                         flags);
 }
 
+HistogramBase* Histogram::FactoryMicrosecondsTimeGet(const char* name,
+                                                     TimeDelta minimum,
+                                                     TimeDelta maximum,
+                                                     uint32_t bucket_count,
+                                                     int32_t flags) {
+  return FactoryMicrosecondsTimeGet(std::string(name), minimum, maximum,
+                                    bucket_count, flags);
+}
+
 std::unique_ptr<HistogramBase> Histogram::PersistentCreate(
     const char* name,
     Sample minimum,
@@ -538,45 +563,42 @@ void Histogram::WriteAscii(std::string* output) const {
   WriteAsciiImpl(true, "\n", output);
 }
 
-bool Histogram::ValidateHistogramContents(bool crash_if_invalid,
-                                          int identifier) const {
-  enum Fields : int {
-    kUnloggedBucketRangesField,
-    kUnloggedSamplesField,
-    kLoggedSamplesField,
-    kIdField,
-    kHistogramNameField,
-    kFlagsField,
-    kLoggedBucketRangesField,
-    kDummyField,
-  };
-
-  uint32_t bad_fields = 0;
-  if (!unlogged_samples_)
-    bad_fields |= 1 << kUnloggedSamplesField;
-  else if (!unlogged_samples_->bucket_ranges())
-    bad_fields |= 1 << kUnloggedBucketRangesField;
-  if (!logged_samples_)
-    bad_fields |= 1 << kLoggedSamplesField;
-  else if (!logged_samples_->bucket_ranges())
-    bad_fields |= 1 << kLoggedBucketRangesField;
-  else if (logged_samples_->id() == 0)
-    bad_fields |= 1 << kIdField;
-  if (flags() == 0)
-    bad_fields |= 1 << kFlagsField;
-  if (dummy_ != kDummyValue)
-    bad_fields |= 1 << kDummyField;
-
-  const bool is_valid = (bad_fields & ~(1 << kFlagsField)) == 0;
-  if (is_valid || !crash_if_invalid)
-    return is_valid;
-
-  // Abort if a problem is found (except "flags", which could legally be zero).
-  std::string debug_string = base::StringPrintf(
-      "%s/%" PRIu32 "#%d", histogram_name(), bad_fields, identifier);
-  CHECK(false) << debug_string;
-  debug::Alias(&bad_fields);
-  return false;
+void Histogram::ValidateHistogramContents() const {
+  CHECK(unlogged_samples_);
+  CHECK(unlogged_samples_->bucket_ranges());
+  CHECK(logged_samples_);
+  CHECK(logged_samples_->bucket_ranges());
+#if !defined(OS_NACL)
+  if (0U == logged_samples_->id() && (flags() & kIsPersistent)) {
+    // ID should never be zero. If it is, then it's probably because the
+    // entire memory page was cleared. Check that this is true.
+    // TODO(bcwhite): Remove this.
+    // https://bugs.chromium.org/p/chromium/issues/detail?id=836875
+    size_t page_size = SysInfo::VMAllocationGranularity();
+    if (page_size == 0)
+      page_size = 1024;
+    const int* address = reinterpret_cast<const int*>(
+        reinterpret_cast<uintptr_t>(logged_samples_->meta()) &
+        ~(page_size - 1));
+    // Check a couple places so there is evidence in a crash report as to
+    // where it was non-zero.
+    CHECK_EQ(0, address[0]);
+    CHECK_EQ(0, address[1]);
+    CHECK_EQ(0, address[2]);
+    CHECK_EQ(0, address[4]);
+    CHECK_EQ(0, address[8]);
+    CHECK_EQ(0, address[16]);
+    CHECK_EQ(0, address[32]);
+    CHECK_EQ(0, address[64]);
+    CHECK_EQ(0, address[128]);
+    CHECK_EQ(0, address[256]);
+    CHECK_EQ(0, address[512]);
+    // Now check every address.
+    for (size_t i = 0; i < page_size / sizeof(int); ++i)
+      CHECK_EQ(0, address[i]);
+  }
+#endif
+  CHECK_NE(0U, logged_samples_->id());
 }
 
 void Histogram::SerializeInfoImpl(Pickle* pickle) const {
@@ -594,9 +616,8 @@ Histogram::Histogram(const char* name,
                      Sample minimum,
                      Sample maximum,
                      const BucketRanges* ranges)
-    : HistogramBase(name), dummy_(kDummyValue) {
-  // TODO(bcwhite): Make this a DCHECK once crbug/734049 is resolved.
-  CHECK(ranges) << name << ": " << minimum << "-" << maximum;
+    : HistogramBase(name) {
+  DCHECK(ranges) << name << ": " << minimum << "-" << maximum;
   unlogged_samples_.reset(new SampleVector(HashMetricName(name), ranges));
   logged_samples_.reset(new SampleVector(unlogged_samples_->id(), ranges));
 }
@@ -609,9 +630,8 @@ Histogram::Histogram(const char* name,
                      const DelayedPersistentAllocation& logged_counts,
                      HistogramSamples::Metadata* meta,
                      HistogramSamples::Metadata* logged_meta)
-    : HistogramBase(name), dummy_(kDummyValue) {
-  // TODO(bcwhite): Make this a DCHECK once crbug/734049 is resolved.
-  CHECK(ranges) << name << ": " << minimum << "-" << maximum;
+    : HistogramBase(name) {
+  DCHECK(ranges) << name << ": " << minimum << "-" << maximum;
   unlogged_samples_.reset(
       new PersistentSampleVector(HashMetricName(name), ranges, meta, counts));
   logged_samples_.reset(new PersistentSampleVector(
@@ -679,15 +699,9 @@ std::unique_ptr<SampleVector> Histogram::SnapshotAllSamples() const {
 }
 
 std::unique_ptr<SampleVector> Histogram::SnapshotUnloggedSamples() const {
-  // TODO(bcwhite): Remove these CHECKs once crbug/734049 is resolved.
-  HistogramSamples* unlogged = unlogged_samples_.get();
-  CHECK(unlogged_samples_);
-  CHECK(unlogged_samples_->id());
-  CHECK(bucket_ranges());
   std::unique_ptr<SampleVector> samples(
       new SampleVector(unlogged_samples_->id(), bucket_ranges()));
   samples->Add(*unlogged_samples_);
-  debug::Alias(&unlogged);
   return samples;
 }
 
@@ -857,6 +871,11 @@ class LinearHistogram::Factory : public Histogram::Factory {
 
   void FillHistogram(HistogramBase* base_histogram) override {
     Histogram::Factory::FillHistogram(base_histogram);
+    // Normally, |base_histogram| should have type LINEAR_HISTOGRAM or be
+    // inherited from it. However, if it's expired, it will actually be a
+    // DUMMY_HISTOGRAM. Skip filling in that case.
+    if (base_histogram->GetHistogramType() == DUMMY_HISTOGRAM)
+      return;
     LinearHistogram* histogram = static_cast<LinearHistogram*>(base_histogram);
     // Set range descriptions.
     if (descriptions_) {
@@ -1029,6 +1048,72 @@ HistogramBase* LinearHistogram::DeserializeInfoImpl(PickleIterator* iter) {
   return histogram;
 }
 
+//------------------------------------------------------------------------------
+// ScaledLinearHistogram: This is a wrapper around a LinearHistogram that
+// scales input counts.
+//------------------------------------------------------------------------------
+
+ScaledLinearHistogram::ScaledLinearHistogram(const char* name,
+                                             Sample minimum,
+                                             Sample maximum,
+                                             uint32_t bucket_count,
+                                             int32_t scale,
+                                             int32_t flags)
+    : histogram_(static_cast<LinearHistogram*>(
+          LinearHistogram::FactoryGet(name,
+                                      minimum,
+                                      maximum,
+                                      bucket_count,
+                                      flags))),
+      scale_(scale) {
+  DCHECK(histogram_);
+  DCHECK_LT(1, scale);
+  DCHECK_EQ(1, minimum);
+  CHECK_EQ(static_cast<Sample>(bucket_count), maximum - minimum + 2)
+      << " ScaledLinearHistogram requires buckets of size 1";
+
+  remainders_.resize(histogram_->bucket_count(), 0);
+}
+
+ScaledLinearHistogram::~ScaledLinearHistogram() = default;
+
+void ScaledLinearHistogram::AddScaledCount(Sample value, int count) {
+  if (count == 0)
+    return;
+  if (count < 0) {
+    NOTREACHED();
+    return;
+  }
+  const int32_t max_value =
+      static_cast<int32_t>(histogram_->bucket_count() - 1);
+  if (value > max_value)
+    value = max_value;
+  if (value < 0)
+    value = 0;
+
+  int scaled_count = count / scale_;
+  subtle::Atomic32 remainder = count - scaled_count * scale_;
+
+  // ScaledLinearHistogram currently requires 1-to-1 mappings between value
+  // and bucket which alleviates the need to do a bucket lookup here (something
+  // that is internal to the HistogramSamples object).
+  if (remainder > 0) {
+    remainder =
+        subtle::NoBarrier_AtomicIncrement(&remainders_[value], remainder);
+    // If remainder passes 1/2 scale, increment main count (thus rounding up).
+    // The remainder is decremented by the full scale, though, which will
+    // cause it to go negative and thus requrire another increase by the full
+    // scale amount before another bump of the scaled count.
+    if (remainder >= scale_ / 2) {
+      scaled_count += 1;
+      subtle::NoBarrier_AtomicIncrement(&remainders_[value], -scale_);
+    }
+  }
+
+  if (scaled_count > 0)
+    histogram_->AddCount(value, scaled_count);
+}
+
 //------------------------------------------------------------------------------
 // This section provides implementation for BooleanHistogram.
 //------------------------------------------------------------------------------
@@ -1195,11 +1280,10 @@ HistogramType CustomHistogram::GetHistogramType() const {
 }
 
 // static
-std::vector<Sample> CustomHistogram::ArrayToCustomRanges(
-    const Sample* values, uint32_t num_values) {
+std::vector<Sample> CustomHistogram::ArrayToCustomEnumRanges(
+    base::span<const Sample> values) {
   std::vector<Sample> all_values;
-  for (uint32_t i = 0; i < num_values; ++i) {
-    Sample value = values[i];
+  for (Sample value : values) {
     all_values.push_back(value);
 
     // Ensure that a guard bucket is added. If we end up with duplicate
